# -*- coding: utf-8 -*-
##############################################################################
#
#    Module for Process Industries for OpenERP
#    Copyright (C) 2008-Today HUMANS & DIVERSITY. (http://www.hu-div.fr)
#    All Right Reserved
#
#    Author : Claude Montaigne <claude.montaigne@hu-div.fr>
#
#    WARNING: This program as such is intended to be used by professional
#    programmers who take the whole responsability of assessing all potential
#    consequences resulting from its eventual inadequacies and bugs
#    End users who are looking for a ready-to-use solution with commercial
#    garantees and support are strongly adviced to contract a Free Software
#    Service Company
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################## 

from datetime import datetime
from dateutil.relativedelta import relativedelta
from osv import fields
from osv import osv
from tools.translate import _
#hdcm 20130704 import ir
import netsvc
import time
import decimal_precision as dp

#HDcm 2012_02_03 
class procurement_order(osv.osv):
    _inherit = 'procurement.order'
     
    _columns = {
        'hd_production_id': fields.many2one('mrp.production', 'Manufacturing Order'),
        #hdcm 2012_12_31 'product_qty': fields.float('Quantity', required=True, states={'draft':[('readonly',False)]}, readonly=True, digits_compute=dp.get_precision('Product UoM 10 moins 6')),
        'product_qty': fields.float('Quantity', required=True, states={'draft':[('readonly',False)]}, readonly=True, digits_compute=dp.get_precision('Qty Module Process')),
        #hdcm 2012_12_31 'product_uos_qty': fields.float('UoS Quantity', states={'draft':[('readonly',False)]}, readonly=True, digits_compute=dp.get_precision('Product UoM 10 moins 6')),
        'product_uos_qty': fields.float('UoS Quantity', states={'draft':[('readonly',False)]}, readonly=True, digits_compute=dp.get_precision('Qty Module Process')),
    }
#/HDcm 2012_02_03  

    #HDcm certains champs n'etaient pas re-initialises
    def copy(self, cr, uid, id, default=None,context=None):
        #print " ***copy(procurement.order) -id ", id, " - default ", default, " - context ", context
        if not default: default = {}
        default.update({
            'date_planned': time.strftime('%Y-%m-%d %H:%M:%S'),
            'date_close': 0,
            'move_id': 0,
        })
        #print "/***copy(procurement.order) -id ", id, " - default ", default, " - context ", context
        return super(procurement_order, self).copy(cr, uid, id, default, context)

    def make_mo(self, cr, uid, ids, context=None):
        """ Make Manufacturing(production) order from procurement
        @return: New created Production Orders procurement wise 
        """
        #print " ***make_mo(procurement.order) - ids ", ids, " - context - ", context
        if context is None: context = {}
        res = {}
        company = self.pool.get('res.users').browse(cr, uid, uid, context).company_id
        production_obj = self.pool.get('mrp.production')
        move_obj = self.pool.get('stock.move')
        wf_service = netsvc.LocalService("workflow")
        procurement_obj = self.pool.get('procurement.order')
        for procurement in procurement_obj.browse(cr, uid, ids, context=context):
            res_id = procurement.move_id.id
            newdate = datetime.strptime(procurement.date_planned, '%Y-%m-%d %H:%M:%S') - relativedelta(days=procurement.product_id.produce_delay or 0.0)
            newdate = newdate - relativedelta(days=company.manufacturing_lead)
            produce_id = production_obj.create(cr, uid, {
                'origin': procurement.origin,
                'product_id': procurement.product_id.id,
                'product_qty': procurement.product_qty,
                'product_uom': procurement.product_uom.id,
                'product_uos_qty': procurement.product_uos and procurement.product_uos_qty or False,
                'product_uos': procurement.product_uos and procurement.product_uos.id or False,
                'location_src_id': procurement.location_id.id,
                'location_dest_id': procurement.location_id.id,
                'bom_id': procurement.bom_id and procurement.bom_id.id or False,
                'date_planned': newdate.strftime('%Y-%m-%d %H:%M:%S'),
                'move_prod_id': res_id,
                'company_id': procurement.company_id.id,
            })
            
            res[procurement.id] = produce_id
            self.write(cr, uid, [procurement.id], {'state': 'running', 'production_id': produce_id})   
            bom_result = production_obj.action_compute(cr, uid,
                    [produce_id], properties=[x.id for x in procurement.property_ids])
            #HDcm 2012_03_07 wf_service.trg_validate(uid, 'mrp.production', produce_id, 'button_confirm', cr)
            if res_id:
                move_obj.write(cr, uid, [res_id],
                        {'location_id': procurement.location_id.id})
            #HDcm 2012_03_07 
            # uniquement si on veut exécuter la 1ère étape en auto        
            # self.pool.get('mrp.production').button_add_to_mo(cr, uid,[produce_id])
            #/HDcm 2012_03_07
        self.production_order_create_note(cr, uid, ids, context=context)
        #print "/***make_mo(procurement.order) - res ", res
        return res
    
procurement_order()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
