# -*- coding: utf-8 -*-
##############################################################################
##
#    Module for Process Industries for OpenERP
#    Copyright (C) 2008-Today HUMANS & DIVERSITY. (http://www.hu-div.fr)
#    All Right Reserved
#
#    Author : Claude Montaigne <claude.montaigne@hu-div.fr>
#
#    WARNING: This program as such is intended to be used by professional
#    programmers who take the whole responsability of assessing all potential
#    consequences resulting from its eventual inadequacies and bugs
#    End users who are looking for a ready-to-use solution with commercial
#    garantees and support are strongly adviced to contract a Free Software
#    Service Company
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################## 

from openerp import netsvc
import time

from openerp.osv import osv,fields
from openerp.tools.translate import _
import openerp.addons.decimal_precision as dp
import string

class splitting_production_lot_memory(osv.osv_memory):
    _name = "splitting.production.lot.memory"

    _columns = {
        'name': fields.char('', size=64, readonly=True),    
        'lot_qty' : fields.float('', digits_compute=dp.get_precision('Qty Module Process'), required=True),
        'wizard_id' : fields.many2one('splitting.production.lot', string="Wizard"),
    }

splitting_production_lot_memory()

class splitting_production_lot(osv.osv_memory):
    _name = 'splitting.production.lot'
    _description = 'Splitting production lot'

    _columns = {
        'info': fields.char('Infos MO', size=128, readonly=True),    
        'lot_qty_lines' : fields.one2many('splitting.production.lot.memory', 'wizard_id', 'Lots qty'),
    }
    
    def default_get(self, cr, uid, fields, context=None):
        """
         To get default values for the object.
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param fields: List of fields for which we want default values
         @param context: A standard dictionary
         @return: A dictionary with default values for all field in ``fields``
        """
        #print "***default_get(splitting_production_lot) - fields ", fields, " - context ", context
        if context is None: context = {}
        res = super(splitting_production_lot, self).default_get(cr, uid, fields, context=context)
        record_id = context and context.get('active_id', False) or False
        prod_obj = self.pool.get('mrp.production')
        prod = prod_obj.browse(cr, uid, record_id, context=context)
        result1 = []
        if prod:
            if 'info' in fields:
                res.update({'info': 'Number of lots to manufacture : %s / Total quantity to be manufactured : %s %s' 
                            % (str(prod.hd_nb_production_lot), str(prod.product_qty), prod.product_uom.name)})
            for i in range(prod.hd_nb_production_lot):
                if prod.hd_nb_production_lot == 1:
                    result1.append({'name': 'Qty lot%s' % (i+1), 'lot_qty': prod.product_qty})
                elif i==0:
                     result1.append({'name': 'Qty lot%s' % (i+1), 'lot_qty': prod.product_qty//prod.hd_nb_production_lot + prod.product_qty%prod.hd_nb_production_lot})
                else:
                    result1.append({'name': 'Qty lot%s' % (i+1), 'lot_qty': prod.product_qty//prod.hd_nb_production_lot})
            if 'lot_qty_lines' in fields:
                res.update({'lot_qty_lines': result1})
        #print "/***default_get(splitting_production_lot) - res ", res
        return res
    
    def save_qty(self, cr, uid, ids, context=None):
        """ Splits moves in quantity given in the wizard.
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param ids: List of ids selected 
         @param context: A standard dictionary 
         @return: A dictionary which of fields with values. 
        """ 
        #print " ***save_qty(splitting_production_lot) - ids ", ids, " - context ", context
        if context is None: context = {}
        record_id = context and context.get('active_id', False) or False
        prod_obj = self.pool.get('mrp.production')
        prod = prod_obj.browse(cr, uid, record_id, context=context)
        move_obj = self.pool.get('stock.move')
        data = self.read(cr, uid, ids[0], context=context)
        # verifier que le nbre de ligne est correct
        if len(data['lot_qty_lines']) != prod.hd_nb_production_lot:
            raise osv.except_osv(_('UserError'), _('You can not add or delete lines'))
        # vérifier que les qty des lots sont positives et qu'une ligne n'a pas ete creee manuellement
        var_qty=0
        list_qty = []
        for line_id in data['lot_qty_lines']:
            line_qty_obj = self.pool.get('splitting.production.lot.memory')
            line_qty = line_qty_obj.browse(cr, uid, line_id, context=context)
            if line_qty.lot_qty <= 0:
                raise osv.except_osv(_('UserError'), _('the lots quantity must be positive'))
            if not line_qty.name:
                raise osv.except_osv(_('UserError'), _('You manually created lines, please re-run the \'Production start\''))
            var_qty += line_qty.lot_qty
            list_qty.append(line_qty.lot_qty)
        if var_qty != prod.product_qty:
            raise osv.except_osv(_('UserError'), _('the sum of the lots must be equal to the projected total quantity'))
        #hdcm2010_04_15 création du move du produit fini
        source = prod.product_id.product_tmpl_id.property_stock_production.id
        infos = {
            'name':'PROD:'+prod.name,
            'date_planned': prod.date_planned,
            'product_id': prod.product_id.id,
            'product_qty': prod.product_qty,
            'product_uom': prod.product_uom.id,
            'location_id': source,
            'location_dest_id': prod.location_dest_id.id,
            'move_dest_id': prod.move_prod_id.id,
            'state': 'waiting'
        }
        move1_prod_id = move_obj.create(cr, uid, infos)
        prod_obj.write(cr, uid, [prod.id], {'move_created_ids': [(6, 'WTF', [move1_prod_id])]})
        # si number_lot > 1, creer les moves create ids par copie pour les 2ème, 3ème, ... lots
        # et ré-écrire la qty du lot01 dans le 1er moves create ids
        if prod.hd_nb_production_lot > 1:
            move1_prod_obj=move_obj.browse(cr, uid, move1_prod_id)
            # écrire dans le move_created la qty real
            move_obj.write(cr,uid, [move1_prod_id], {'product_qty':list_qty[0]})
            # si le champ move_dest_id est <> False écrire la qty real dans ce move
            if move1_prod_obj.move_dest_id.id:
                move_obj.write(cr,uid, [move1_prod_obj.move_dest_id.id], {'product_qty':list_qty[0]})
            # faire une copie par lot suplémentaire
            for i in range(1,prod.hd_nb_production_lot):
                move_i_prod_id = move_obj.copy(cr, uid, move1_prod_id, {'product_qty': list_qty[i],
                                            'state':'waiting', 'move_history_ids': [], 'move_history_ids2': []})#HDcm 2012_03_26 603_02_chem
                #HDcm 2012_03_26 v603_02chem faire un lien history avec le 1er lot
                #print "history HD - parent ", move1_prod_id, " - child ", move_i_prod_id
                move_obj.write(cr, uid, [move1_prod_id], {'move_history_ids': [(4, move_i_prod_id)]})
                #/HDcm 2012_03_26 v603_02chem
                move_i_prod_obj = move_obj.browse(cr, uid, move_i_prod_id)
                # s'il y a un SO ou un procur, faire la même chose avec les moves du SO ou du procur
                if move_i_prod_obj.move_dest_id.id:
                    move2_i_prod_obj_id = move_obj.copy(cr, uid, move_i_prod_obj.move_dest_id.id, {
                                            'product_qty': list_qty[i],
                                            'state':'confirmed', 'move_history_ids': [], 'move_history_ids2': []})#HDcm 2012_03_26 603_02_chem
                    move_obj.write(cr,uid, [move1_prod_obj.id], {'move_dest_id':move2_i_prod_obj_id})
        # lancer la fonction action_in_production
        prod_obj.action_in_production(cr, uid, [record_id])
        
        #print "/***save_qty(splitting_production_lot)"
        return {'type': 'ir.actions.act_window_close'}

splitting_production_lot()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

