# -*- coding: utf-8 -*-
##############################################################################
#
#    Module for Process Industries for OpenERP
#    Copyright (C) 2008-Today HUMANS & DIVERSITY. (http://www.hu-div.fr)
#    All Right Reserved
#
#    Author : Claude Montaigne <claude.montaigne@hu-div.fr>
#
#    WARNING: This program as such is intended to be used by professional
#    programmers who take the whole responsability of assessing all potential
#    consequences resulting from its eventual inadequacies and bugs
#    End users who are looking for a ready-to-use solution with commercial
#    garantees and support are strongly adviced to contract a Free Software
#    Service Company
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################## 

import datetime
import time
from osv import osv, fields
import pooler
import netsvc
import math
from tools import config

import decimal_precision as dp
from tools.translate import _

class product_uom(osv.osv):
    _name = 'product.uom'
    _inherit = 'product.uom'
    
    _columns = {
        #hdcm 2012_12_31 'rounding': fields.float('Rounding Precision', digits_compute=dp.get_precision('Product UoM 10 moins 6'), required=True,
        'rounding': fields.float('Rounding Precision', digits_compute=dp.get_precision('Qty Module Process'), required=True,
            help="The computed quantity will be a multiple of this value. "\
                 "Use 1.0 for a UoM that cannot be further split, such as a piece."),
    }
    
    _defaults = {
        'rounding': lambda *a: 0.000001,
    }

    '''hdcm 2013_08_06'''
    #hdcm 2013_01_21
    '''hdcm 2013_09_10 def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        print " ***search(product_uom) - args ",args," - context ",context
        if not context: context = {}
        product_obj = self.pool.get('product.product')
        #verifier s'il y a un produit dans le context
        produit_mentionne = True
        if not (context.get('product_id', False) and context['product_id']): produit_mentionne = False
        #hdcm 2013_01_14 test de lot_control2
        if produit_mentionne and (context.get('cas_mo_normal_drafting_lines', False) or context.get('cas_so_normal_so_lines', False) or \
                                  context.get('cas_split_normal_move_lines', False) or context.get('cas_transf_normal_move_lines', False)):
            for x in args :
                if (x[0] == 'id') and (x[1] == 'in'): args.remove(x)
            args.extend([('id', 'in', self.search(cr, uid, [('category_id', '=', product_obj.browse(cr, uid, context['product_id']).uom_id.category_id.id)]))])
            
        print "/***search(product_uom) - args ",args," - context ",context
        return super(product_uom, self).search(cr, uid, args, offset, limit, order, context=context, count=count)'''


product_uom()

#HD cm 25-02-2009 Ajout de 3 champs many2many dans la classe product.product
class product_product(osv.osv):
    _inherit = 'product.product'
    _name = 'product.product'

    _columns = {
        'hd_track_lot' : fields.boolean('Manage by Lots' , 
                                        help="Automatic creation of a lot in purchase, production "
                                        "and sales (for products of 'stockable product' type)"),
        'hd_fragmentation' : fields.boolean('Can be fragmented' , 
                                        help="Allow the fragmentation in several lots when checking availability (for a Manufacturing order or "
                                        "a Delivery-order). For a delivery-order, the fragmentation is allowed, even if the quantity requested is not completely available"),
        'hd_threshold': fields.float('Threshold checking availability', digits_compute=dp.get_precision('Product UoM'), help="When checking availability (for a Manufacturing order or a Delivery-order), a lot will not be retained if the available quantity of this lot is below the threshold"),
    }
    _defaults = {
        'hd_track_lot': lambda *a: 0,
        'hd_fragmentation': lambda *a: 1,
        'hd_threshold': lambda *a: 0.0,
    }

    #hdcm 2009_09_15
    def unlink(self, cr, uid, ids, context={}):
        """interdire la suppression d'un produit ayant déjà fait l'objet d'un move"""
        for rec in self.browse(cr, uid, ids, context=context):
            move_ids = self.pool.get('stock.move').search(cr, uid, [('product_id', '=', rec.id)])
            if move_ids:
                raise osv.except_osv(_('Invalid action !'), _('Cannot delete a product if there is one or more stock move for this product'))
        return super(product_product, self).unlink(cr, uid, ids, context=context)
    #/hdcm 2009_09_15

    #HDcm 2012_05_03 604_03chem
    def create(self, cr, user, vals, context=None):
        #print "***create(product_product) - vals ",vals, " - context ", context
        if context is None: context = {}
        # verifier si HD Track lot est activé alors que le produit est de type consommable ou service
        if ('hd_track_lot' in vals) and (vals['hd_track_lot'] == True)\
            and ('type' in vals) and ((vals['type'] == 'consu') or (vals['type'] == 'service')):
            raise osv.except_osv(_('Invalid action !'), _('A product of type \'%s\' can not be managed by lot') % _(vals['type']))
        new_id = super(product_product, self).create(cr, user, vals, context)
        #print "/***create(product_product) - new_id ",new_id
        return new_id
    #/HDcm 2012_05_03 604_03chem

    '''# hdcm 2013_09_10
    def write(self, cr, uid, ids,vals,context={}):
        print " ***write(product.product) - ids ",ids," - vals ",vals," - context ",context
        if context is None: context = {}
        lot_obj = self.pool.get('stock.production.lot')
        move_obj = self.pool.get('stock.move')
        product = self.browse(cr, uid, ids[0], context=context)
        if product.hd_track_lot:
            #print "cas des produits geres par lot"
            if ('type' in vals) and (vals['type'] in ['consu', 'service']):
                raise osv.except_osv(_('Invalid action !'), _('A product of type \'%s\' can not be managed by lot') % _(vals['type']))
            lots = lot_obj.search(cr, uid, [('product_id','in',ids)], context=context)
            if len(lots) != 0 :# si des lots ont deja ete cree pour ce produit
                if 'uom_id' in vals:# HDcm 2012_04_25 604_03chem interdire le changement d'uom s'il y a deja des lots de cree pour ce produit
                    if self.pool.get('product.uom').browse(cr, uid, vals['uom_id']).id !=\
                       lot_obj.browse(cr, uid, lots[0]).hd_product_uom_id.id:
                        raise osv.except_osv(_('Invalid action !'), _('Impossible to change the uom if there are already lots for this product'))
                if ('hd_track_lot' in vals) and (vals['hd_track_lot'] == False):
                        raise osv.except_osv(_('Invalid action !'), _('Impossible to turn off \'Manage by Lots\' if there are already lots for this product'))
        else:
            print "cas des produits non geres par lots"
            # verifier s'il existe dja des moves pour ce produit qui ne sont pas dans l'etat cancel
            moves = move_obj.search(cr, uid, [('product_id','in',ids),('state', 'not in', ('cancel',))], context=context)
            if len(moves) != 0:
                raise osv.except_osv(_('Invalid action !'), _('Impossible to turn on \'Manage by Lots\' if there are already moves (not cancel) for this product'))
        print "/***write(product.product) - vals ",vals," - context ",context
        return super(product_product,self).write(cr, uid, ids, vals, context)'''

    # hdcm 2013_09_10
    def write(self, cr, uid, ids,vals,context={}):
        print " ***write(product.product) - ids ",ids," - vals ",vals," - context ",context
        if context is None: context = {}
        lot_obj = self.pool.get('stock.production.lot')
        move_obj = self.pool.get('stock.move')
        product = self.browse(cr, uid, ids[0], context=context)
        if product.hd_track_lot:
            #print "cas des produits geres par lot"
            if ('type' in vals) and (vals['type'] in ['consu', 'service']):
                raise osv.except_osv(_('Invalid action !'), _('A product of type \'%s\' can not be managed by lot') % _(vals['type']))
            lots = lot_obj.search(cr, uid, [('product_id','in',ids)], context=context)
            if len(lots) != 0 :# si des lots ont deja ete cree pour ce produit
                if 'uom_id' in vals:# HDcm 2012_04_25 604_03chem interdire le changement d'uom s'il y a deja des lots de cree pour ce produit
                    if self.pool.get('product.uom').browse(cr, uid, vals['uom_id']).id !=\
                       lot_obj.browse(cr, uid, lots[0]).hd_product_uom_id.id:
                        raise osv.except_osv(_('Invalid action !'), _('Impossible to change the uom if there are already lots for this product'))
                if ('hd_track_lot' in vals) and (vals['hd_track_lot'] == False):
                        raise osv.except_osv(_('Invalid action !'), _('Impossible to turn off \'Manage by Lots\' if there are already lots for this product'))
        else:
            print "cas des produits non geres par lots"
            # verifier s'il existe dja des moves pour ce produit qui ne sont pas dans l'etat cancel
            moves = move_obj.search(cr, uid, [('product_id','in',ids),('state', 'not in', ('cancel',))], context=context)
            if len(moves) != 0:
                if ('hd_track_lot' in vals) and (vals['hd_track_lot'] == True):#hdcm 2013_11_21
                    raise osv.except_osv(_('Invalid action !'), _('Impossible to turn on \'Manage by Lots\' if there are already moves (not cancel) for this product'))
        print "/***write(product.product) - vals ",vals," - context ",context
        return super(product_product,self).write(cr, uid, ids, vals, context)

    def on_change_hd_threshold(self, cr, uid, ids, hd_threshold):
        #print "***on_change_hd_threshold(product_product) - ids ",ids, " - hd_threshold ", hd_threshold
        result = {}
        warning = {}
        if hd_threshold < 0.0:
            result['hd_threshold'] = self.read(cr, uid, ids, ['hd_threshold'])[0]['hd_threshold']
            warning={'title':_('UserError'), 'message':_('The threshold must be at least equal to 0')}
        #print "/***on_change_hd_threshold(product_product) - value ",result, " - warning ", warning
        return {'value':result, 'warning':warning}



product_product()

