# -*- coding: utf-8 -*-
##############################################################################
#
#    Module for Process Industries for OpenERP
#    Copyright (C) 2008-Today HUMANS & DIVERSITY. (http://www.hu-div.fr)
#    All Right Reserved
#
#    Author : Claude Montaigne <claude.montaigne@hu-div.fr>
#
#    WARNING: This program as such is intended to be used by professional
#    programmers who take the whole responsability of assessing all potential
#    consequences resulting from its eventual inadequacies and bugs
#    End users who are looking for a ready-to-use solution with commercial
#    garantees and support are strongly adviced to contract a Free Software
#    Service Company
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import netsvc
import decimal_precision as dp
from tools.translate import _
from osv import fields, osv
import time

class change_lots_status(osv.osv_memory):
    _name = 'change.lots.status'
    _description = 'Change Lots Status'
    


    _columns = {
        'state': fields.selection( (
            ('to_control','To control'),
            ('conform','Conform'),
            ('not_conform','Not conform'),#hdcm 2010_02_08
            ('alert_conform','Alert conform'),
            ('out_of_date','Out of date'),
            ('to_destroy','To destroy'),
            ('destroyed','Destroyed'),
            ('declassified','Declassified'),
            ('to_reprocess','To reprocess')
                    ),'State'),
    }

    _defaults = {
        'state': lambda *a: 'to_control',
    }

    def make_change(self, cr, uid, ids, context=None):
        """ Creates multi splitting.
        @param self: The object pointer.
        @param cr: A database cursor
        @param uid: ID of the user currently logged in
        @param ids: List of IDs selected
        @param context: A standard dictionary
        @return: A dictionary .
        """
        #print " ***make_change(change_lots_status) - ids ", ids, " - context ", context
        if context is None: context = {}
        data = self.browse(cr, uid, ids[0], context=context)
        lot_ids = context and context.get('active_ids', False)
        assert lot_ids, _('Active ID is not set in Context')
        lot_obj = self.pool.get('stock.production.lot')
        wf_service = netsvc.LocalService("workflow")
        lot_ids_ok = []
        for lot in lot_obj.browse(cr, uid, lot_ids):
            if (data.state == 'to_control') and (lot.state in ('conform', 'not_conform', 'alert_conform', 'out_of_date', 'to_destroy', 'declassified', 'to_reprocess')):
                wf_service.trg_validate(uid, 'stock.production.lot', lot.id, 'button_to_control', cr)
                lot_ids_ok.append(lot.id)
            elif (data.state == 'conform') and (lot.state in ('to_control',)):
                wf_service.trg_validate(uid, 'stock.production.lot', lot.id, 'button_conform', cr)
                lot_ids_ok.append(lot.id)
            elif (data.state == 'not_conform') and (lot.state in ('to_control',)):
                wf_service.trg_validate(uid, 'stock.production.lot', lot.id, 'button_not_conform', cr)
                lot_ids_ok.append(lot.id)
            elif (data.state == 'alert_conform') and (lot.state in ('conform',)):
                wf_service.trg_validate(uid, 'stock.production.lot', lot.id, 'button_alert_conform', cr)
                lot_ids_ok.append(lot.id)
            elif (data.state == 'out_of_date') and (lot.state in ('alert_conform',)):
                wf_service.trg_validate(uid, 'stock.production.lot', lot.id, 'button_out_of_date', cr)
                lot_ids_ok.append(lot.id)
            elif (data.state == 'to_destroy') and (lot.state in ('to_control',)):
                wf_service.trg_validate(uid, 'stock.production.lot', lot.id, 'button_to_destroy', cr)
                lot_ids_ok.append(lot.id)
            elif (data.state == 'destroyed') and (lot.state in ('to_destroy',)):
                wf_service.trg_validate(uid, 'stock.production.lot', lot.id, 'button_destroyed', cr)
                lot_ids_ok.append(lot.id)
            elif (data.state == 'declassified') and (lot.state in ('to_control',)):
                wf_service.trg_validate(uid, 'stock.production.lot', lot.id, 'button_declassified', cr)
                lot_ids_ok.append(lot.id)
            elif (data.state == 'to_reprocess') and (lot.state in ('to_control',)):
                wf_service.trg_validate(uid, 'stock.production.lot', lot.id, 'button_to_reprocess', cr)
                lot_ids_ok.append(lot.id)
        #print "/***make_change(change_lots_status) - lot_ids_ok ", lot_ids_ok
        return {'type': 'ir.actions.act_window_close'}
        

change_lots_status()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

