# -*- coding: utf-8 -*-
##############################################################################
#
#    Module for Process Industries for OpenERP
#    Copyright (C) 2008-Today HUMANS & DIVERSITY. (http://www.hu-div.fr)
#    All Right Reserved
#
#    Author : Claude Montaigne <claude.montaigne@hu-div.fr>
#
#    WARNING: This program as such is intended to be used by professional
#    programmers who take the whole responsability of assessing all potential
#    consequences resulting from its eventual inadequacies and bugs
#    End users who are looking for a ready-to-use solution with commercial
#    garantees and support are strongly adviced to contract a Free Software
#    Service Company
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import netsvc
import time

from openerp.osv import osv,fields
from openerp.tools.translate import _
import openerp.addons.decimal_precision as dp
import string

class stock_return_picking_lot_memory(osv.osv_memory):
    _name = "stock.return.picking.lot.memory"
    _rec_name = 'product_id'

    _columns = {
        'product_id' : fields.many2one('product.product', string="Product", required=True, readonly=True),
        'quantity' : fields.float("Qty returned", digits_compute=dp.get_precision('Qty Module Chem'), required=True),
        'wizard_id' : fields.many2one('stock.return.picking.lot', string="Wizard"),
        'move_id' : fields.many2one('stock.move', "Move"),
        'product_uom' : fields.many2one('product.uom', string="Uom", required=True, readonly=True),
        'info': fields.char('Informations', size=256, readonly=True),    
    }

    def quantity_change(self, cr, uid, ids, move_id, context=None):
        #print "***quantity_change(stock_return_picking_lot_memory) - ids ", ids, " - move_id ", move_id, " - context ", context
        if context is None: context = {}
        res = {'value': {}, 'warning': {}}
        lot_obj = self.pool.get('stock.production.lot')#HDcm 2012_04_19 604_03chem
        if move_id:
            # retrouver le lot et sa qte virt
            lot_id = self.pool.get('stock.return.picking.lot').search_lot(cr, uid, move_id)
            lot = lot_obj.browse(cr, uid, lot_id)
            if not (abs(lot.hd_qty_real - lot.hd_qty_virt) < 0.000000000001):
                res['warning']={'title':_('Invalid action !'), 'message':_('The return is not allowed if an operation is in progress on this lot')}
                res['value'] = {'quantity': 0.0}
        #print "/***quantity_change(stock_return_picking_lot_memory) - res ", res
        return res

stock_return_picking_lot_memory()


class stock_return_picking_lot(osv.osv_memory):
    _name = 'stock.return.picking.lot'
    _description = 'Return Picking Lot'
    _columns = {
        'product_return_moves' : fields.one2many('stock.return.picking.lot.memory', 'wizard_id', 'Moves'),
        'invoice_state': fields.selection([('2binvoiced', 'To be refunded/invoiced'), ('none', 'No invoicing')], 'Invoicing',required=True),
    }

    def default_get(self, cr, uid, fields, context=None):
        """
         To get default values for the object.
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param fields: List of fields for which we want default values
         @param context: A standard dictionary
         @return: A dictionary with default values for all field in ``fields``
        """
        #print "***default_get(stock_return_picking_lot) - fields ", fields, " - context ", context
        result1 = []
        if context is None:
            context = {}
        res = super(stock_return_picking_lot, self).default_get(cr, uid, fields, context=context)
        record_id = context and context.get('active_id', False) or False
        lot_obj = self.pool.get('stock.production.lot')#HDcm 2012_04_19 604_03chem
        pick_obj = self.pool.get('stock.picking')
        pick = pick_obj.browse(cr, uid, record_id, context=context)
        if pick:
            if 'invoice_state' in fields:
                if pick.invoice_state=='invoiced':
                    res.update({'invoice_state': '2binvoiced'})
                else:
                    res.update({'invoice_state': 'none'})
            #return_history = self.get_return_history(cr, uid, record_id, context)       
            for line in pick.move_lines:
                #hdcm 2013_08_15
                if line.product_id.hd_track_lot:
                    lot_id = self.search_lot(cr, uid, line.id)
                    lot = lot_obj.browse(cr, uid, lot_id)
                    if lot.hd_qty_virt > 0:
                        result1.append({'product_id': line.product_id.id, 'quantity': 0.0000000,'move_id':line.id, 'prodlot_id': line.prodlot_id and line.prodlot_id.id or False, 
                                        'info': '(Lot % 7s - Qty maxi = %s % 3s)'%(lot_id, str(lot.hd_qty_virt), lot.hd_product_uom_id.name), 
                                        'product_uom': line.product_uom.id})
            if 'product_return_moves' in fields:
                res.update({'product_return_moves': result1})
        #print "/***default_get(stock_return_picking_lot) - res ", res
        return res

    def view_init(self, cr, uid, fields_list, context=None):
        """
         Creates view dynamically and adding fields at runtime.
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param context: A standard dictionary
         @return: New arch of view with new columns.
        """
        #print "***view_init(stock_return_picking_lot) - fields_list ", fields_list, " - context ", context
        if context is None:
            context = {}
        res = super(stock_return_picking_lot, self).view_init(cr, uid, fields_list, context=context)
        record_id = context and context.get('active_id', False)
        if record_id:
            lot_obj = self.pool.get('stock.production.lot')#HDcm 2012_04_19 604_03chem
            pick_obj = self.pool.get('stock.picking')
            pick = pick_obj.browse(cr, uid, record_id, context=context)
            if pick.state not in ['done','confirmed','assigned']:
                raise osv.except_osv(_('Warning!'), _("You may only return pickings that are Confirmed, Available or Done!"))
            valid_lines = 0
            # return_history = self.get_return_history(cr, uid, record_id, context)
            for m  in pick.move_lines:
                #hdcm 2013_08_16 
                if m.product_id.hd_track_lot:
                    lot_id = self.search_lot(cr, uid, m.id)
                    lot = lot_obj.browse(cr, uid, lot_id)
                    if lot.hd_qty_virt > 0:
                        valid_lines += 1
            if not valid_lines:
                raise osv.except_osv(_('Warning!'), _("Only products manage by lots can be returned)!"))
        #print "/***view_init(stock_return_picking_lot) - res ", res
        return res
    
    #HDcm 2012_04_19 604_03chem
    def search_lot(self, cr, uid, move_id):
        """ 
         recherche un lot correspondant a un move
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param move: id du move analyse
         @param context: A standard dictionary
         @return: id of lot or False.
        """
        #print " ***search_lot(stock_return_picking_hd) - move_id ", move_id
        move_obj = self.pool.get('stock.move')
        m = move_obj.browse(cr, uid, move_id)
        res = False
        if m.hd_prodlot_id.id: 
            res = m.hd_prodlot_id.id
            #print "/***search_lot(stock_return_picking_hd) - res ", res
            return res
        purchase_line_id = m.purchase_line_id.id
        sale_line_id = m.sale_line_id.id
        while True:
            #print "m ", m
            list_move_child = ([x for x in m.move_history_ids if ((x.purchase_line_id.id == purchase_line_id) and (x.sale_line_id.id == sale_line_id))])
            #print "list_move_child ", list_move_child
            if len(list_move_child) == 0:
                break
            m = list_move_child[0]
            if m.hd_prodlot_id.id:
                res = m.hd_prodlot_id.id
                break
        #print "/***search_lot(stock_return_picking_hd) - res ", res
        return res

    def create_returns(self, cr, uid, ids, context=None):
        """ 
         Creates return picking.
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param ids: List of ids selected
         @param context: A standard dictionary
         @return: A dictionary which of fields with values.
        """
        #print "***create_returns(stock_return_picking_lot) - ids ", ids, " - context ", context
        if context is None: context = {} 
        context['prog'] = 'create_returns(stock_return_picking_hd)'#HDcm 2012_04_19 604_03chem
        record_id = context and context.get('active_id', False) or False
        seq_obj = self.pool.get('ir.sequence')
        move_obj = self.pool.get('stock.move')
        lot_obj = self.pool.get('stock.production.lot')#HDcm 2012_04_19 604_03chem
        pick_obj = self.pool.get('stock.picking')
        uom_obj = self.pool.get('product.uom')
        data_obj = self.pool.get('stock.return.picking.lot.memory')
        wf_service = netsvc.LocalService("workflow")
        pick = pick_obj.browse(cr, uid, record_id, context=context)
        data = self.read(cr, uid, ids[0], context=context)
        date_cur = time.strftime('%Y-%m-%d %H:%M:%S')
        set_invoice_state_to_none = True
        returned_lines = 0
        split_picking_id = None#HDcm 2012_04_19 604_03chem
        
        val_id = data['product_return_moves']
        #print "val_id ", val_id
        for v in val_id:
            data_get = data_obj.browse(cr, uid, v, context=context)
            mov_id = data_get.move_id.id
            if not mov_id:
                raise osv.except_osv(_('Warning !'), _("You have manually created product lines, please delete them to proceed"))
            new_qty = data_get.quantity
            move = move_obj.browse(cr, uid, mov_id, context=context)
            
            if move.state != 'done': continue
            # retrouver le lot et sa qte virt
            lot_id = self.search_lot(cr, uid, move.id)
            lot = lot_obj.browse(cr, uid, lot_id)
            #hdcm 2013_08_19 v604 if new_qty > lot.hd_qty_virt :
            if (new_qty > 0) and (not (abs(lot.hd_qty_real - lot.hd_qty_virt) < 0.000000000001)):
                raise osv.except_osv(_('Invalid action !'), _('The return is not allowed if an operation is in progress on this lot'))
            if new_qty > uom_obj._compute_qty(cr, uid, lot.product_id.uom_id.id, lot.hd_qty_virt, move.product_uom.id) :
                raise osv.except_osv(_('Warning !'), '%s %s' % (_('The amount returned is superior to the virtual quantity of Lot'), lot_id))#hdcm 2012_11_22
            elif new_qty <= 0:
                continue
            if not split_picking_id:
                # creer un split incluant le déplacement
                prefix_name = seq_obj.get(cr, uid, 'stock.picking.' + 'split')
                #print "prefix_name ", prefix_name
                split_picking_id = pick_obj.copy(cr, uid, pick.id, {
                        'move_lines':[], 'state':'draft', 'type':'split','move_type': 'one', 'auto_picking': False,
                        'date':date_cur, 'invoice_state':data['invoice_state'],
                        })
                #reecrire name du picking
                pick_obj.write(cr, uid,[split_picking_id], {'name': prefix_name + '[%s-return]' % pick.name, 
                                                            'origin': pick.origin})
            returned_lines += 1
            # creer les lignes du split correspondant aux qtes saisies
            new_mov_split = move_obj.copy(cr, uid, move.id, {
                'product_qty': new_qty,
                'product_uos_qty': uom_obj._compute_qty(cr, uid, data_get.product_uom.id, new_qty, move.product_uos.id),
                'picking_id':split_picking_id, 'state':'draft',
                'location_id': move.location_dest_id.id, 'location_dest_id':move.location_id.id,
                'date':date_cur,'move_history_ids': [], 'move_history_ids2': [],
                'prodlot_id': False, 'hd_prodlot_id': False, 'hd_lot_org': lot.id
                })
        
        if not returned_lines:
            raise osv.except_osv(_('Warning!'), _("Please specify at least one non-zero quantity."))
        if set_invoice_state_to_none:
            pick_obj.write(cr, uid, [pick.id], {'invoice_state':'none'}, context=context)
            
        #HDcm 2012_04_19 604_03chem
        if split_picking_id :
            wf_service.trg_validate(uid, 'stock.picking', split_picking_id, 'button_confirm', cr)
            self.pool.get('stock.picking').action_assign(cr, uid, [split_picking_id], context=context)
            #HDcm 2012_04_25 cette ligne est mise en commentaire pour que le picking ne soit pas validé automatiquement
            #wf_service.trg_validate(uid, 'stock.picking', split_picking_id, 'button_done', cr)
        #/HDcm 2012_04_19 604_03chem
        
        view_obj = self.pool.get('ir.ui.view')
        view_id = view_obj.search(cr, uid, [('model', '=', 'stock.picking'), \
                                     ('name', '=', 'stock.splitting.form')])        
        
        #print "/***create_returns(stock_return_picking_lot)"
        return {
            'name': _('Returned Picking'),
            'res_model': 'stock.picking',
            'res_id': int(split_picking_id),
            'view_type':'form',
            'view_mode':'form',
            'view_id': view_id or False,
            'type':'ir.actions.act_window',
            'context':context,
        }

stock_return_picking_lot()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
