# -*- coding: utf-8 -*-
##############################################################################
#
#    Module for Process Industries for OpenERP
#    Copyright (C) 2008-Today HUMANS & DIVERSITY. (http://www.hu-div.fr)
#    All Right Reserved
#
#    Author : Claude Montaigne <claude.montaigne@hu-div.fr>
#
#    WARNING: This program as such is intended to be used by professional
#    programmers who take the whole responsability of assessing all potential
#    consequences resulting from its eventual inadequacies and bugs
#    End users who are looking for a ready-to-use solution with commercial
#    garantees and support are strongly adviced to contract a Free Software
#    Service Company
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################## 
from openerp.report import report_sxw


from mx import DateTime
import time
from datetime import datetime
from openerp.tools import float_compare, DEFAULT_SERVER_DATETIME_FORMAT

from osv import osv, fields
from osv import osv
from osv import orm

import netsvc

#hdcm 2013_11_20 import datetime

import pooler
from tools.translate import _
import decimal_precision as dp

CAS_MO_NORMAL = {'types': ('in', 'fab_stock', 'split'), 
                     'states': ('conform','alert_conform'), 
                     'location_under_control': 'not_allowable'}
CAS_MO_NORMAL_REPROCESSING = {'types': ('in', 'fab_stock', 'split'), 
                     'states': ('to_reprocess',), 
                     'location_under_control': 'not_allowable'}
'''hdcm 2013_09_24 v604 CAS_SO_NORMAL = {'types': ('in', 'fab_stock', 'split','sale_fab'), 
                     'states': ('conform','alert_conform'), 
                     'location_under_control': 'not_allowable'}'''
CAS_SO_NORMAL = {'types': ('in', 'fab_stock', 'split','sale_fab','sale_buy'), 
                     'states': ('conform','alert_conform'), 
                     'location_under_control': 'not_allowable'}
CAS_DO_MANUEL = {'types': ('in', 'fab_stock', 'split','sale_fab','sale_buy'), 
                     'states': ('conform','alert_conform'), 
                     'location_under_control': 'not_allowable'}
CAS_SPLIT_NORMAL = {'types': ('in', 'fab_stock', 'split'), 
                    'states': ('to_control','conform','alert_conform','out_of_date','to_destroy','declassified','to_reprocess'), 
                    'location_under_control': 'allowable'}
CAS_TRANSF_NORMAL = {'types': ('in', 'fab_stock', 'split'), 
                    'states': ('to_control','conform','alert_conform','out_of_date','to_destroy','declassified','to_reprocess'), 
                    'location_under_control': 'allowable'}
CAS_DISPL_NORMAL = {'types': ('in', 'fab_stock', 'split','sale_fab'), 
                    'states': ('to_control','conform','alert_conform','out_of_date','to_destroy','declassified','to_reprocess'), 
                    'location_under_control': 'allowable'}

class stock_production_lot(osv.osv):
    
    def name_get(self, cr, uid, ids, context=None):
        #print "***name_get(stock.production.lot) - ids  ",ids, "  - context  ",context
        if not ids:
            return []
        reads = self.read(cr, uid, ids, ['name', 'prefix', 'ref'], context)
        #hdcm 2013_08_05
        if type(reads) is not list:
            reads = [reads]
        #/hdcm 2013_08_05
        res = []
        for record in reads:
            name = record['name']
            prefix = record['prefix']
            if prefix:
                name = prefix + '/' + name
            if record['ref']:
                name = '%s [%s]' % (name, record['ref'])
            res.append((record['id'], name))
        #print "/***name_get(stock.production.lot) - res  ",res
        return res
    
    _name = "stock.production.lot"
    _inherit = 'stock.production.lot'
    _description = 'Lot'#hdcm 2013_10_29

    def _ref_lot_org(self, cr, uid, ids, name, arg, context={}):
        result = {}
        for stock_lot_obj in self.browse(cr, uid, ids, context):
            if stock_lot_obj.hd_lot_org_id:
                result[stock_lot_obj.id] = self.browse(cr,uid,stock_lot_obj.hd_lot_org_id).hd_ref_lot
            else:
                result[stock_lot_obj.id] = ""
        return result

    def _calc_lot(self, cr, uid, ids, field_name, arg, context=None):
        #print "***_calc_lot(stock.production.lot) - ids  ",ids, "  - context  ",context
        if context is None : context = {}
        res = {}
        if len(ids) and not ids[0]: 
            return False
        res_stock_lot_info = self._stock_lot_info(cr, uid, ids, context=context)
        for lot in self.browse(cr, uid, ids, context):
            #res_stock_lot_info=self._stock_lot_info(cr, uid, [lot.id])
            if len(res_stock_lot_info) == 0:
                res[lot.id] = {
                               #hdcm 2013_10_29 'hd_product_uom_id' : False,
                               'hd_qty_real' : False,
                               'hd_qty_virt' : False,
                               'hd_location_id' : False,
                               #'hd_qty_org' : False,
                               }
            else:
                res[lot.id] = {
                               #hdcm 2013_10_29 'hd_product_uom_id' : res_stock_lot_info[lot.id]['hd_product_uom_id'],
                               'hd_qty_real' : res_stock_lot_info[lot.id]['hd_qty_real'],
                               'hd_qty_virt' : res_stock_lot_info[lot.id]['hd_qty_virt'],
                               'hd_location_id' : res_stock_lot_info[lot.id]['hd_location_id'],
                               #'hd_qty_org' : res_stock_lot_info[lot.id]['hd_qty_org'],
                               }
        #print "/***_calc_lot(stock.production.lot) - res  ",res
        return res

    #hdcm 2010_01_28
    def _get_info_lot(self, cr, uid, ids, context={}):
        #print "***_get_info_lot(stock.production.lot) - ids ",ids," - context ",context
        #print time.clock()
        result = {}
        for move in self.pool.get('stock.move').browse(cr, uid, ids, context=context):
            if move.hd_prodlot_id :result[move.hd_prodlot_id.id] = True
            if move.hd_lot_org :result[move.hd_lot_org.id] = True
            if move.hd_lot_dest_transf :result[move.hd_lot_dest_transf.id] = True            
        #print time.clock()
        #print "/***_get_info_lot(stock.production.lot) - result.keys() ",result.keys()
        return result.keys()

    def _get_uom_product(self, cr, uid, ids, context=None):
        #print "***_get_uom_product(stock.production.lot) - ids ",ids," - context ",context
        if context is None: context = {}
        result = []
        result += self.pool.get('stock.production.lot').search(cr, uid, [('product_id','in',ids)], context=context)
        #print "/***_get_uom_product(stock.production.lot) - result ",result
        return result

    _columns = {
        'name': fields.char('Name', size=64, required=True, readonly=True),
        'hd_date_creat_lot':fields.datetime('Create date'),
        'hd_indtrack': fields.boolean('Tracked'), # permet de reconnaitre les lots crees avec le module chimie
        'state': fields.selection( (
            ('to_control','To control'),
            ('conform','Conform'),
            ('alert_conform','Alert conform'),
            ('out_of_date','Out of date'),
            ('not_conform','Not conform'),#hdcm 2010_02_08
            ('declassified','Declassified'),
            ('to_reprocess','To reprocess'),
            ('to_destroy','To destroy'),
            ('destroyed','Destroyed'),
                    ),'State'),
        'hd_type': fields.selection( (
            ('inv','Inventory'),
            ('in','Purchase'),
            ('sale_stock','Sale from stock'),
            ('sale_fab','Sale from production'),
            ('sale_buy','Sale from purchase'),#hdcm 2013_09_24 v7.0
            ('fab_stock','Production for stock'),
            ('int','Internal'),
            ('split','Splitting'),
            ('xxx','xxx')
                    ),'Type'),
        'hd_reserv': fields.selection( (
            ('no','None'),
            ('vente','Sale order'),
            ('interne','Internal'),
                    ),'Reservation'),
        'hd_reserv_partner_id':fields.many2one('res.partner', 'Customer'),
        'hd_reserv_production_id':fields.many2one('mrp.production', 'Production'),
        'hd_reserv_uid':fields.many2one('res.users', 'User'),
        'hd_product_uom_id': fields.related('product_id', 'uom_id', type='many2one', relation='product.uom', string="UOM", readonly="1", store=True),#hdcm 2013_10_29
        'hd_qty_real': fields.function(_calc_lot, method=True, type='float', string='Real qty', digits_compute=dp.get_precision('Qty Module Process'),
                            store={'stock.production.lot': (lambda self, cr, uid, ids, c={}: ids, None, 50),
                                  'stock.move': (_get_info_lot, ['product_qty', 'hd_prodlot_id', 'hd_lot_org', 'hd_lot_dest_transf', 'state'], 10),
                                  'product.product': (_get_uom_product, ['uom_id'], 10),}, multi='lots'),
        'hd_qty_virt': fields.function(_calc_lot, method=True, type='float', string='Virtual qty', digits_compute=dp.get_precision('Qty Module Process'), 
                            store={'stock.production.lot': (lambda self, cr, uid, ids, c={}: ids, None, 50),
                                  'stock.move': (_get_info_lot, ['product_qty', 'hd_prodlot_id', 'hd_lot_org', 'hd_lot_dest_transf', 'state'], 10),
                                  'product.product': (_get_uom_product, ['uom_id'], 10),}, multi='lots'),
        'hd_location_id': fields.function(_calc_lot, method=True, type='many2one', relation='stock.location', string='Location',
                            store={'stock.production.lot': (lambda self, cr, uid, ids, c={}: ids, None, 50),
                                   'stock.move': (_get_info_lot, ['product_qty', 'hd_prodlot_id', 'hd_lot_org', 'hd_lot_dest_transf', 'state'], 10),
                                   'product.product': (_get_uom_product, ['uom_id'], 10),}, multi='lots'),
        'hd_note': fields.text('Notes'),
        'hd_doc_org_id': fields.integer('Original document'),
        'hd_doc_inter_id': fields.integer('Intermediate document'),
        'hd_qty_org': fields.float('Original qty', digits_compute=dp.get_precision('Qty Module Process')), 
        'hd_lot_org_id': fields.integer('Lot org'),
        'hd_lot_dest_id': fields.integer('Destination lot'),
        'hd_production_id_spl': fields.many2one('mrp.production', 'Production Order'),
        'hd_production_id_int': fields.many2one('mrp.production', 'Production Order'),
        'hd_production_id_fab': fields.many2one('mrp.production', 'Production Order'),
        'hd_ref_lot': fields.char('Ref Lot', size=64),
        'hd_ref_lot_org': fields.function(_ref_lot_org, method=True, type='char', string='Ref Lot org'),
    }

    _defaults = {
        #'hd_date_creat_lot': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
        'hd_date_creat_lot': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
        'hd_indtrack': lambda *a: False,
        'hd_type': lambda *a: 'xxx',
        'hd_qty_real': lambda *a: 0.0,
        'hd_qty_virt': lambda *a: 0.0,
        'hd_qty_org': lambda *a: 0.0,
        'hd_reserv': lambda *a: 'no',
        'state': lambda *a: 'to_control',
    }

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        #print " ***search(stock_production_lot) - args ",args," - context ",context
        if not context: context = {}
        #verifier s'il y a un produit dans le context
        produit_mentionne = True
        if not (context.get('product_id', False) and context['product_id']): produit_mentionne = False
        #hdcm 2013_01_14 test de lot_control2
        if produit_mentionne and ((context.get('cas_mo_normal_drafting_lines', False) and (context.get('type_line', False) == 'standard')) \
                                  or context.get('cas_mo_normal_stock_pickings', False)):
            for x in args :
                if (x[0] == 'id') and (x[1] == 'in'): args.remove(x)
            context.update(CAS_MO_NORMAL)
            args.extend([('id', 'in', self.lot_control2(cr, uid, context=context)['lots_ok'])])
        elif produit_mentionne and ((context.get('cas_mo_normal_drafting_lines', False) and (context.get('type_line', False) == 'reprocessing')) or\
                            (context.get('cas_mo_normal_reprocessing_lines', False) and (context.get('type_line', False) == 'reprocessing')) or\
                            context.get('cas_mo_normal_stock_pickings', False)):
            for x in args :
                if (x[0] == 'id') and (x[1] == 'in'): args.remove(x)
            context.update(CAS_MO_NORMAL_REPROCESSING)
            # hdcm 2013_09_05 v604 args.extend([('id', 'in', self.lot_control2(cr, uid, context=context)['lots_ok'])])
            args = [('state','=','to_reprocess'), ('id', 'in', self.lot_control2(cr, uid, context=context)['lots_ok'])]
        elif produit_mentionne and (context.get('cas_so_normal_so_lines', False) or context.get('cas_so_normal_stock_pickings', False)):
            for x in args :
                if (x[0] == 'id') and (x[1] == 'in'): args.remove(x)
            context.update(CAS_SO_NORMAL)
            args.extend([('id', 'in', self.lot_control2(cr, uid, context=context)['lots_ok'])])
        elif produit_mentionne and context.get('cas_split_normal_move_lines', False):
            for x in args :
                if (x[0] == 'id') and (x[1] == 'in'): args.remove(x)
            context.update(CAS_SPLIT_NORMAL)
            args.extend([('id', 'in', self.lot_control2(cr, uid, context=context)['lots_ok'])])
        elif produit_mentionne and context.get('cas_transf_normal_move_lines', False):
            for x in args :
                if (x[0] == 'id') and (x[1] == 'in'): args.remove(x)
            context.update(CAS_TRANSF_NORMAL)
            args.extend([('id', 'in', self.lot_control2(cr, uid, context=context)['lots_ok'])])
        elif produit_mentionne and context.get('cas_displ_normal_move_lines', False):
            for x in args :
                if (x[0] == 'id') and (x[1] == 'in'): args.remove(x)
            context.update(CAS_DISPL_NORMAL)
            args.extend([('id', 'in', self.lot_control2(cr, uid, context=context)['lots_ok'])])
        elif context.get('cas_mo_normal_drafting_lines', False) or context.get('cas_mo_normal_stock_pickings', False) or\
                        context.get('cas_so_normal_so_lines', False) or context.get('cas_so_normal_stock_pickings', False) or\
                        context.get('cas_split_normal_move_lines', False) or\
                        context.get('cas_transf_normal_move_lines', False) or\
                        context.get('cas_displ_normal_move_lines', False):
            args.extend([('id', 'in', [])])

        #print "/***search(stock_production_lot) - args ",args," - context ",context
        return super(stock_production_lot, self).search(cr, uid, args, offset, limit, order, context=context, count=count)


    #hdcm 2010_02_03 
    def write(self, cr, uid, ids,vals,context={}):
        #print "***write(stock.production.lot) - ids ",ids," - vals ",vals," - context ",context
        if context is None: context = {}
        if ('hd_reserv' in vals) or ('hd_reserv_partner_id' in vals) or ('hd_reserv_production_id' in vals):# si un de ces champs a ete modifie
            user_reserv = self.browse(cr, uid, ids)[0].hd_reserv_uid.id
            if not user_reserv:# s'il n'y avait pas de reservation
                vals['hd_reserv_uid'] = uid
            else:
                idem_user = False
                if user_reserv == uid:idem_user = True
                #HDcm 2012_02_09 role_management = False
                group_management = False
                if self.pool.get('res.users').group_stock_manager_ok(cr, uid, context=context):group_management = True
                if context.get('prog', False) :  group_management = True#HDcm 2012_03_28    603_02chem - autoriser la suppression de la reservation par un prog
                if not (idem_user or group_management):
                    raise osv.except_osv(_('Invalid action !'), 
                         _('You can not modify this reservation .This modification must be realize by %s or a user of the group \'Warehouse / Manager\'')\
                          %(self.browse(cr, uid, ids)[0].hd_reserv_uid.name,))
                elif ('hd_reserv' in vals) and (vals['hd_reserv'] != 'no') or ('hd_reserv_partner_id' in vals) or ('hd_reserv_production_id' in vals):
                    vals['hd_reserv_uid'] = uid
                else:# raz des champs
                    vals['hd_reserv_uid'] = 0
                    vals['hd_reserv_partner_id'] = 0                  
                    vals['hd_reserv_production_id'] = 0
        #print "/***write(stock.production.lot) - ids ",ids," - vals ",vals," - context ",context
        return super(stock_production_lot,self).write(cr, uid, ids, vals, context)

#hd00 fonctions liées au workflow

    #HDcm 2012_02_09 dans 603, le test group_id ne semble pas fonctionner correctement dans une transition
    # l'utilisation d'une fonction controlant l'appartenance au groupe qualite permet de plus d'afficher un message d'erreur
    #HDcm 2012_02_16
    def group_control_quality_ok(self, cr, uid, ids, context=None):
        #print "***group_control_quality_ok(stock_production_lot) - ids ",ids," - context ", context
        if context is None: context = {}
        res = True
        for lot_obj in self.browse(cr, uid, ids):
            # verifier l'appartenance de l'utilisateur au groupe 'Quality / Control'
            if not self.pool.get('res.users').group_control_quality_ok(cr, uid, context=context):
                raise osv.except_osv(_('Invalid action !'),_('you must belong to group \'Quality / Control\' !'))
        #print "/***group_control_quality_ok(stock_production_lot) - res ", res
        return res

    # verifier si un lot est en cours de fabrication
    def production_running(self, cr, uid, ids):
        #print "***production_running(stock_production_lot) - ",ids
        for lot_obj in self.browse(cr, uid, ids):
            if lot_obj.hd_production_id_fab.state == "in_production":
                return True
        #print "/***production_running(stock_production_lot)"
        return False

    def verif_lot_reprocessing_used(self, cr, uid, ids):
        #print "***verif_lot_reprocessing_used(stock_production_lot) - ",ids
        # verifie si un lot a l'etat 'to_reprocess' est utilise dans une fab
        prod_line_used_obj = self.pool.get('mrp.production.product.line.used')
        for lot in self.browse(cr, uid, ids):
            if (lot.state == 'to_reprocess'):
                if len(prod_line_used_obj.search(cr, uid, [('lot_org', '=',lot.id),('type_line', '=', 'reprocessing'),('state', '!=', 'cancel')])):
                    return True
        #print "/***verif_lot_reprocessing_used(stock_production_lot)"
        return False

    def action_to_control_wkf(self, cr, uid, ids, context=None):
        #print "***action_to_control_wkf(stock_production_lot) - ",ids
        if context is None: context = {}
        if self.production_running(cr, uid, ids):
            raise osv.except_osv(_('Invalid action !'), _('Cannot modify the status of a lot whose MO is \'In Production\' !'))
        #hdcm 2013_02_25 si ce lot est en etat 'to_reprocess', verifier qu'il n'est pas utilise dans un MO...
        if self.verif_lot_reprocessing_used(cr, uid, ids):
            raise osv.except_osv(_('Invalid action !'), _('Cannot modify a lot \'To Reprocess\' used in a manufacturing order !'))
        for lot in self.browse(cr, uid, ids):
            message = _("Lot %s : '%s' to 'to_control'") % (lot.name, lot.state)
            self.log(cr, uid, lot.id, message, context=context)
        self.write(cr, uid, ids, {'state': 'to_control'})
        #print "/***action_to_control_wkf(stock_production_lot)"
        return True

    def action_conform_wkf(self, cr, uid, ids, context=None):
        #print "***action_conform_wkf(stock_production_lot) - ",ids
        if context is None: context = {}
        if self.production_running(cr, uid, ids):
            raise osv.except_osv(_('Invalid action !'), _('Cannot modify the status of a lot whose MO is \'In Production\' !'))
        for lot in self.browse(cr, uid, ids):
            message = _("Lot %s : '%s' to 'conform'") % (lot.name, lot.state)
            self.log(cr, uid, lot.id, message, context=context)
        self.write(cr, uid, ids, {'state': 'conform'})
        #print "/***action_conform_wkf(stock_production_lot) - "
        return True
    #hdcm 2010_02_08 ajout de l'etat not-conform

    def action_not_conform_wkf(self, cr, uid, ids, context=None):
        #print "***action_not_conform_wkf(stock_production_lot) - ",ids
        if context is None: context = {}
        if self.production_running(cr, uid, ids):
            raise osv.except_osv(_('Invalid action !'), _('Cannot modify the status of a lot whose MO is \'In Production\' !'))
        for lot in self.browse(cr, uid, ids):
            message = _("Lot %s : '%s' to 'not_conform'") % (lot.name, lot.state)
            self.log(cr, uid, lot.id, message, context=context)
        self.write(cr, uid, ids, {'state': 'not_conform'})
        #print "/***action_not_conform_wkf(stock_production_lot) - "
        return True

    def action_alert_conform_wkf(self, cr, uid, ids, context=None):
        #print "***action_alert_conform_wkf(stock_production_lot) - ",ids
        if context is None: context = {}
        if self.production_running(cr, uid, ids):
            raise osv.except_osv(_('Invalid action !'), _('Cannot modify the status of a lot whose MO is \'In Production\' !'))
        for lot in self.browse(cr, uid, ids):
            message = _("Lot %s : '%s' to 'alert_conform'") % (lot.name, lot.state)
            self.log(cr, uid, lot.id, message, context=context)
        self.write(cr, uid, ids, {'state': 'alert_conform'})
        #print "/***action_alert_conform_wkf(stock_production_lot) - "
        return True

    def action_out_of_date_wkf(self, cr, uid, ids, context=None):
        #print "***action_out_of_date_wkf(stock_production_lot) - ",ids
        if context is None: context = {}
        if self.production_running(cr, uid, ids):
            raise osv.except_osv(_('Invalid action !'), _('Cannot modify the status of a lot whose MO is \'In Production\' !'))
        for lot in self.browse(cr, uid, ids):
            message = _("Lot %s : '%s' to 'out_of_date'") % (lot.name, lot.state)
            self.log(cr, uid, lot.id, message, context=context)
        self.write(cr, uid, ids, {'state': 'out_of_date'})
        #print "/***action_out_of_date_wkf(stock_production_lot) - "
        return True

    def action_to_destroy_wkf(self, cr, uid, ids, context=None):
        #print "***action_to_destroy_wkf(stock_production_lot) - ",ids
        if context is None: context = {}
        if self.production_running(cr, uid, ids):
            raise osv.except_osv(_('Invalid action !'), _('Cannot modify the status of a lot whose MO is \'In Production\' !'))
        for lot in self.browse(cr, uid, ids):
            message = _("Lot %s : '%s' to 'to_destroy'") % (lot.name, lot.state)
            self.log(cr, uid, lot.id, message, context=context)
        self.write(cr, uid, ids, {'state': 'to_destroy'})
        #print "/***action_to_destroy_wkf(stock_production_lot) - "
        return True

    def action_destroyed_wkf(self, cr, uid, ids, context=None):
        #print "***action_destroyed_wkf(stock_production_lot) - ids",ids
        if context is None : context = {}
        if len(ids) and not ids[0]: return False
        if self.production_running(cr, uid, ids):
            raise osv.except_osv(_('Invalid action !'), _('Cannot modify the status of a lot whose MO is \'In Production\' !'))
        #HDcm 2012_03_30 v603_02chem - deplacer ce lot vers la 1ere zone scrap trouvee
        wf_service = netsvc.LocalService("workflow")
        location_obj = self.pool.get('stock.location')
        scraped_location_ids = location_obj.search(cr, uid, [('scrap_location','=',True)])
        if not scraped_location_ids:
            raise osv.except_osv(_('Invalid action !'), _('Cannot find a scrapped location !'))
        move_obj = self.pool.get('stock.move')
        for lot in self.browse(cr, uid, ids):
            # retrouver le move correspondant a ce lot
            move_id = move_obj.search(cr, uid, [('hd_prodlot_id', '=', lot.id)])[0]
            move = move_obj.browse(cr, uid, move_id)
            # preparer un deplacement
                # créer un picking de type displacement
            picking_id = self.pool.get('stock.picking').create(cr, uid, {
                        'type': 'displacement',
                        'move_type': 'one',
                        'state': 'draft',
                        'auto_picking': False,
                        'company_id': move.company_id.id,#HDcm 2012_03_08 v603_01chem
                        })
            # création du move pour ce deplacemet
            move_id = self.pool.get('stock.move').create(cr, uid, {
                        'name':move.name,
                        'picking_id':picking_id,
                        'product_id': move.product_id.id,
                        'product_uom' : move.product_uom.id,
                        'product_qty' : move.product_qty,
                        'location_id': move.location_dest_id.id,
                        'location_dest_id': scraped_location_ids[0],
                        'prodlot_id' : move.prodlot_id.id,
                        'state': 'draft',
                        'move_history_ids': [], 
                        'move_history_ids2': [],
                        'company_id': move.company_id.id,
                        })
            wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
        for lot in self.browse(cr, uid, ids):
            message = _("Lot %s : '%s' to 'destroyed'") % (lot.name, lot.state)
            self.log(cr, uid, lot.id, message, context=context)
        self.write(cr, uid, ids, {'state': 'destroyed'})
        #print "/***action_destroyed_wkf(stock_production_lot) - "
        return True

    def action_declassified_wkf(self, cr, uid, ids, context=None):
        #print "***action_declassified_wkf(stock_production_lot) - ",ids
        if context is None : context = {}
        if self.production_running(cr, uid, ids):
            raise osv.except_osv(_('Invalid action !'), _('Cannot modify the status of a lot whose MO is \'In Production\' !'))
        for lot in self.browse(cr, uid, ids):
            message = _("Lot %s : '%s' to 'declassified'") % (lot.name, lot.state)
            self.log(cr, uid, lot.id, message, context=context)
        self.write(cr, uid, ids, {'state': 'declassified'})
        #print "/***action_declassified_wkf(stock_production_lot) - "
        return True

    def action_to_reprocess_wkf(self, cr, uid, ids, context=None):
        #print "***action_to_reprocess_wkf(stock_production_lot) - ",ids
        if context is None : context = {}
        if self.production_running(cr, uid, ids):
            raise osv.except_osv(_('Invalid action !'), _('Cannot modify the status of a lot whose MO is \'In Production\' !'))
        for lot in self.browse(cr, uid, ids):
            message = _("Lot %s : '%s' to 'to_reprocess'") % (lot.name, lot.state)
            self.log(cr, uid, lot.id, message, context=context)
        self.write(cr, uid, ids, {'state': 'to_reprocess'})
        #print "/***action_to_reprocess_wkf(stock_production_lot) - "
        return True


    # cette fonction retourne des infos concernant un lot ou plusieurs lots
    # uniquement pour les lots tracés
    # la fonction retourne une liste de dictionnaires
    # chaque dictionnaire contient lot,produit,qté réelle,qté virtuelle et location  
    def _stock_lot_info(self, cr, uid, ids, context={}):
        #print "***_stock_lot_info(stock.production.lot) - ids ",ids," - context ",context
        #print time.clock()
        move_obj = self.pool.get('stock.move')#HDcm 2012_0412 604_02chem
        product_uom_obj = self.pool.get('product.uom')#HDcm 2012_0412 604_02chem
        res = {}
        for lot in self.browse(cr, uid, ids, context):
            if not lot.hd_indtrack: # vérifier que le lot est tracé par le module chimie
                continue
            hd_qty_org = lot.hd_qty_org
            hd_qty_real = 0.0
            hd_qty_virt = 0.0
            hd_location_id = 0
            hd_product_uom_id = 0 
            # search du move dont le champ lot_id contient #l'id du lot
            move_lot_ids = move_obj.search(cr, uid, [('hd_prodlot_id', '=', lot.id), ('state', '!=', 'cancel')])
            #print "move_lot_ids ", move_lot_ids
            if len(move_lot_ids) == 0:# si aucun move n'a été trouvé
                continue
            m = move_obj.browse(cr, uid, max(move_lot_ids))
            location_id = m.location_dest_id.id # mémoriser la zone de stockage où se trouve le lot
            product_uom_id = m.product_id.uom_id.id #hdcm 2010_01_28 mémoriser l'uom du produit
            #print "Deb - hd_qty_org ", hd_qty_org
            if m.picking_id.type == 'displacement':# traitement lot correspondant a 1 picking de type 'displacement'
                #hdcm 2010_01_28 hd_qty_real=lot.hd_qty_org
                hd_qty_real = hd_qty_org
                hd_qty_virt = hd_qty_real
            # si ce move est dans l'état done
            elif (m.state == 'done') or ((m.state == 'assigned') and (lot.hd_type == 'int')):
                hd_qty_real = hd_qty_org
                hd_qty_virt = hd_qty_real
            else:
                hd_qty_virt = hd_qty_org
            # search des moves dont le champ lot_org contient l'id du lot
            move_lot_org_ids = move_obj.search(cr, uid, [('hd_lot_org', '=', lot.id)])
            for m_lotorg in move_obj.browse(cr, uid, move_lot_org_ids):
                #print "Et1 traitement lot_org ", m_lotorg.id
                if (m_lotorg.state == 'done'): # si ce move est dans l'état done
                    # si le pickibg correspondant est de type 'displacement'
                    if m_lotorg.picking_id.type == 'displacement':
                        # alors retrouver la qte du lot d'origine du lot deplace
                        q = m_lotorg.hd_prodlot_id.hd_qty_org
                    else:# sinon prendre la qte mentionnee dans le move
                        q = product_uom_obj._compute_qty(cr, uid, m_lotorg.product_uom.id, m_lotorg.product_qty, to_uom_id=product_uom_id)
                    #print "Et1.1 "
                    hd_qty_real -= q
                    hd_qty_virt -= q
                elif m_lotorg.state in ('confirmed','assigned'):
                    hd_qty_virt -= product_uom_obj._compute_qty(cr, uid, m_lotorg.product_uom.id, m_lotorg.product_qty, to_uom_id=product_uom_id)
            # search des moves dont le champ lot_dest_transf contient l'id du lot
            move_lot_dest_transf_ids = move_obj.search(cr, uid, [('hd_lot_dest_transf', '=', lot.id)])
            for  m_lotdest in move_obj.browse(cr, uid, move_lot_dest_transf_ids):
                #print "Et2 traitement lot_dest ", m_lotdest.id
                # si ce move est dans l'état done
                if m_lotdest.state == 'done':
                    q = product_uom_obj._compute_qty(cr, uid, m_lotdest.product_uom.id, m_lotdest.product_qty, to_uom_id=product_uom_id)
                    #print "Et2.1 "
                    hd_qty_real += q
                    hd_qty_virt += q
                elif m_lotdest.state in ('confirmed','assigned'):
                    hd_qty_virt += product_uom_obj._compute_qty(cr, uid, m_lotdest.product_uom.id, m_lotdest.product_qty, to_uom_id=product_uom_id)
            #HDcm 2012_04_03
            res[lot.id] = {
                           #hdcm 2013_10_29 'hd_product_uom_id' : product_uom_id,
                           'hd_qty_real' : hd_qty_real,
                           'hd_qty_virt' : hd_qty_virt,
                           'hd_location_id' : location_id,
                           }
        #print time.clock()
        #print "/***_stock_lot_info(stock.production.lot) - res ",res
        return res

    def force_calc_lot(self, cr, uid, ids, context=None):
        #print "***force_calc_lot(stock_production_lot) - ",ids, " - context ", context
        if context is None: context = {}
        if len(ids) and not ids[0]: 
            return False
        self.write(cr, uid, ids,{}, context=context)
        #print "/***force_calc_lot(stock_production_lot)"
        return True

    def _lot_reserve(self, cr, uid, location_ids, product_id, product_qty,  context=None):
        #print "***_lot_reserve(stock_production_lot) - location_ids",location_ids," - product_id ",product_id," - product_qty ",product_qty," - context ",context
        #print time.clock()
        if context is None: context = {}
        result = []
        lots_ids = []
        if context.get('cas_mo_normal_stock_pickings', False) and context['cas_mo_normal_stock_pickings']:
            context.update({'product_id': product_id, 'location_ids': location_ids})
            context.update(CAS_MO_NORMAL)
            verif_lots = self.pool.get('stock.production.lot').lot_control2(cr, uid, context=context)
            lots_ids = verif_lots['lots_ok']
        if context.get('cas_so_normal_stock_pickings', False) and context['cas_so_normal_stock_pickings']:
            context.update({'product_id': product_id, 'location_ids': location_ids})
            context.update(CAS_SO_NORMAL)
            verif_lots = self.pool.get('stock.production.lot').lot_control2(cr, uid, context=context)
            lots_ids = verif_lots['lots_ok']
        #hdcm 2013_12_06 
        if context.get('cas_do_manuel', False) and context['cas_do_manuel']:
            context.update({'product_id': product_id, 'location_ids': location_ids})
            context.update(CAS_DO_MANUEL)
            verif_lots = self.pool.get('stock.production.lot').lot_control2(cr, uid, context=context)
            lots_ids = verif_lots['lots_ok']
        
        qty = 0.0
        total = 0.0
        product=self.pool.get('product.product').browse(cr, uid, product_id)
        #print "product.hd_threshold ", product.hd_threshold
        if len(lots_ids) > 0:
            for lot in self.browse(cr, uid,lots_ids):
                #print " lot_id ", lot.id, " - lot.hd_qty_virt ", lot.hd_qty_virt
                qty = self.pool.get('product.uom')._compute_qty(cr, uid, lot.hd_product_uom_id.id,lot.hd_qty_virt, to_uom_id=context.get('uom',False))
                #print "qty ", qty
                if (abs(qty - 0.0) < 0.000000000001): continue#hdcm 2013_02_07 si la qte virtuelle = 0 le lot n'est pas retenu
                if (qty < product.hd_threshold): continue#hdcm 2012_12_10 si la qte virtuelle est inferieure a hd_threshold du produit alors ne pas retenir ce lot
                if (not product.hd_fragmentation):# produit non fragmentable
                    if (qty >= product_qty):#hdcm 2012_12_10 si la fragmentation n'est pas autorisée et que le lot contient assez de produit
                        result.append((lot.id, lot.hd_location_id.id, product_qty))
                        break
                    else:
                        continue
                else:# produit fragmentable
                    if total + qty > product_qty:
                        qty = product_qty - total
                    total += qty
                    result.append((lot.id, lot.hd_location_id.id, qty))
                    if abs(total - product_qty) < 0.000000000001:
                        break#HDcm 2012_05_04 cette modif permet de retourner une liste de lot trouve meme si la qte n'est pas completement couverte
        #print time.clock()
        #print "/***_lot_reserve(stock_production_lot) - result ", result
        return result

    #hdcm 2013_01_11 test d'une nouvelle fonction lot_control
    def lot_control2(self, cr, uid, lots_requested=[], qty_requested=0.0, uom_requested=0, context=None):
        """ 
        @param cr: the current row, from the database cursor,
        @param uid: the current user’s ID for security checks,
        @param lots_requested: List des lots demandes 
                                    la fonction va verifier chaque lot de cette liste en fonction des criteres passes dans le contexte
                                    seuls les lots repondant a ces criteres appraitront dans la liste 'lots_ok'
                                    les lots ne repondant pas aux criteres apparaitront dans la liste 'lots_rejected'
        @param qty_requested: qte demandee 
        @param uom_requested: id de l'uom demandee 
        @param context: A standard dictionary for contextual values
                        ce dictionnaire contient les criteres de recherche :
                            types: Liste des types de lots autorises
                            states: Liste des etats de lots autorises
                            product_id : produit demande
                            shop_id : magasin dans lequel s'effectue la recherche (dans le cas des Sale Order)
                            location_ids : Liste des emplacements de stock a prendre en compte
                            location_under_control: 'allowable' - les emplacements sous controle sont inclus dans la recherche
                                                    'not_allowable' - les emplacements sous controle sont exclus dans la recherche
                                                    'only' - seules les emplacements sous controle sont pris en compte dans la recherche
        @return: si la liste des lots demandes en parametre est vide (dans ce cas lot_control2 effectue une recherche des lots possibles en fonction des criteres du contexte
                            {'lots_ok': [liste de lots utilisables], (tries en fonction des dates de peremption - lot le plus ancien en 1er) , 
                             'lots_rejected': [],
                             'infos': '',
                             'qty_available': qte disponible (somme des qtes virtuelles disponibles dans les lots de la liste 'lots_ok'),
                             'ind_qty_available': False}
                 si une liste de lots demandes est passee en parametre, dans ce cas lot_control2 
                                                        - verifie que ces lots repondent aux criteres du contexte
                                                                ceux qui repondent aux criteres sont dans la liste 'lots_ok'
                                                                ceux qui ne repondent aux criteres sont dans la liste 'lots_rejected'
                                                        - mentionne un message explicatif dans info_lots_rejected
                                                        - calcule la somme des qtes virtuelles disponibles dans les lots de la liste 'lots_ok'
                            {'lots_ok': [liste des lots demandes (acceptes)], (tries en fonction des dates de peremption - lot le plus ancien en 1er)
                             'lots_rejected': [liste des lots rejetes],
                             'infos': 'message explicatif sur les lots rejetes', 
                             'qty_available': qte disponible (somme des qtes virtuelles disponibles dans les lots de la liste 'lots_ok'),
                             'ind_qty_available: True (si qty_requested < qty_available - en tenant compte de uom_requested)}
       """
        #print " ***lot_control2(stock_production_lot) -lots_requested ", lots_requested, " - qty_requested ", qty_requested, " - uom_requested ", uom_requested, " - context ", context
        if context is None: context = {}
        if (len(lots_requested) > 0) and (not lots_requested[0]): lots_requested = []
        if not context.get('location_ids', False): context['location_ids'] = []
        if (len(context.get('location_ids')) > 0) and (not (context['location_ids'][0])): context['location_ids'] = []
        lots_ok = []
        lots_rejected = []
        message = ''
        qty_available = 0.0
        # verifier si un produit est mentionne
        if not (context.get('product_id', False) and context['product_id']): 
           #print "/***lot_control2(stock_production_lot) - lots_ok ",lots_ok
            return lots_ok
        res = {'lots_ok': [], 'lots_rejected': [], 'infos': '', 'qty_available': 0.0, 'ind_qty_available': False}
        stock_location_obj = self.pool.get('stock.location') 
        product_obj = self.pool.get('product.product')
        shop_obj = self.pool.get('sale.shop')
        uom_obj = self.pool.get('product.uom')
        product_line_used_obj = self.pool.get('mrp.production.product.line.used')
        move_obj = self.pool.get('stock.move')
        sale_order_line_obj = self.pool.get('sale.order.line')
        
        # trouver la liste des emplacements a prendre en compte
        locations = []
        if context.get('shop_id', False) and context['shop_id']:
            warehouse = shop_obj.browse(cr, uid, context['shop_id']).warehouse_id
            location_id = warehouse.lot_stock_id.id
            if context.get('location_under_control', False) and context['location_under_control'] == 'allowable':
                locations.extend(stock_location_obj.search(cr, uid, [('location_id', 'child_of', [location_id])]))
            elif context.get('location_under_control', False) and context['location_under_control'] == 'not_allowable':
                locations.extend(stock_location_obj.search(cr, uid, [('location_id', 'child_of', [location_id]), \
                                                                ('hd_zone_sous_controle', '=', False)]))
            elif context.get('location_under_control', False) and context['location_under_control'] == 'only':
                locations.extend(stock_location_obj.search(cr, uid, [('location_id', 'child_of', [location_id]), \
                                                                ('hd_zone_sous_controle', '=', True)]))
        if context.get('location_ids', False) and (len(context['location_ids']) > 0):
            for location_id in context['location_ids']:
                if context.get('location_under_control', False) and context['location_under_control'] == 'allowable':
                    locations.extend(stock_location_obj.search(cr, uid, [('location_id', 'child_of', [location_id])]))
                elif context.get('location_under_control', False) and context['location_under_control'] == 'not_allowable':
                    locations.extend(stock_location_obj.search(cr, uid, [('location_id', 'child_of', [location_id]), \
                                                                    ('hd_zone_sous_controle', '=', False)]))
                elif context.get('location_under_control', False) and context['location_under_control'] == 'only':
                    locations.extend(stock_location_obj.search(cr, uid, [('location_id', 'child_of', [location_id]), \
                                                                    ('hd_zone_sous_controle', '=', True)]))
        locations = list(set(locations))#pour supprimer les doublons
       #print "all locations ", locations
        
        # verification de lots reserver pour une production
        move = move_obj.browse(cr, uid, context.get('move'))
        cas_lot_fab_pour_fab = False
        #hdcm 2013_02_04!!! if context.get('MO', False) and context.get('move', False):
        if context.get('cas_mo_normal_stock_pickings', False) and context.get('move', False):
            #print "verification de lots reserver pour une production"
            ''''HDcm 2012_03_28 603_02chem - dans le cas d'un MO, voir si ce move correspond a une ligne product_line_used
            ayant un procure_method = a 'make_to_order. dans ce cas il faut rechercher uniquement le lot fabrique a 
            l'occasion de ce MO'''
            product_line_used_id_list =  product_line_used_obj.search(cr, uid, [('production_id_procur', '=', context.get('cas_mo_normal_stock_pickings')), 
                                                                                           ('move_id_spl', '=',context.get('move'))
                                                                                           ])
            if len(product_line_used_id_list) == 0 :
                product_line_used_id = False
            else:
                product_line_used_id = product_line_used_id_list[0]
            if product_line_used_id and \
                (product_line_used_obj.browse(cr, uid, product_line_used_id).procure_method == 'make_to_order'):
                cas_lot_fab_pour_fab = True
        # verification de lots reserver pour un SO
        cas_lot_fab_pour_so = False
        cas_lot_buy_pour_so = False
        if context.get('cas_so_normal_stock_pickings', False) and context.get('move', False):
            '''hdcm 2013_09_23 v604 if move.sale_line_id.type == 'make_to_order':
                cas_lot_fab_pour_so = True'''
            if move.sale_line_id.type == 'make_to_order':
                if move.product_id.supply_method == 'produce':
                    # cas d'un produit fabrique
                    cas_lot_fab_pour_so = True
                else:# cas d'un produit achete
                    cas_lot_buy_pour_so = True
                    
       #print "cas_lot_fab_pour_fab ", cas_lot_fab_pour_fab
       #print "cas_lot_fab_pour_so ", cas_lot_fab_pour_so
        # trouver la liste des lots possibles en fonction des criteres du context
        if cas_lot_fab_pour_fab:
            cas_hd_reserv = 'interne'
            cas_hd_reserv_production_id = move.picking_id.hd_production_id_spl.id
            cas_hd_reserv_partner_id = False
        #hdcm 2013_09_24 v604 elif cas_lot_fab_pour_so:
        elif cas_lot_fab_pour_so or cas_lot_buy_pour_so:
            cas_hd_reserv = 'vente'
            cas_hd_reserv_production_id = False
            cas_hd_reserv_partner_id = move.sale_line_id.order_partner_id.id
        else:
            cas_hd_reserv = 'no'
            cas_hd_reserv_production_id = False
            cas_hd_reserv_partner_id = False
            
        if context.get('cas_split_normal_move_lines', False) or \
           context.get('cas_transf_normal_move_lines', False) or \
           context.get('cas_displ_normal_move_lines', False):
           #print "CAS RECHERCHE SPLIT-TRANSFERT-DISPLACEMENT"
            lots_ok = self.search(cr, uid, [
                                                   ('product_id', '=', context.get('product_id')),
                                                   ('hd_reserv', '=', 'no'),
                                                   ('hd_type', 'in', context.get('types')),
                                                   ('state', 'in', context.get('states')),
                                                   ('hd_qty_virt','>', 0.0)],
                                         order='life_date, id asc')
        
        elif context.get('cas_mo_normal_drafting_lines', False) or \
           context.get('cas_so_normal_so_lines', False) or \
           (not (context.get('cas_check_assign_stock_move', False) or context.get('cas_action_move_stock_picking', False))):
           #print "CAS RECHERCHE AVEC EMPLACEMENT"
            lots_ok = self.search(cr, uid, [
                                                   ('product_id', '=', context.get('product_id')),
                                                   ('hd_reserv', '=', 'no'),
                                                   ('hd_type', 'in', context.get('types')),
                                                   ('state', 'in', context.get('states')),
                                                   ('hd_location_id', 'in', locations),
                                                   ('hd_qty_virt','>', 0.0)],
                                         order='life_date, id asc')
        
        elif len(lots_requested) and (context.get('cas_check_assign_stock_move', False) or context.get('cas_action_move_stock_picking', False)):
            #print "lots_requested and (cas_check_assign_stock_move or cas_action_move_stock_picking)"
            lots_ok = self.search(cr, uid, [('id', 'in', lots_requested),
                                                   ('product_id', '=', context.get('product_id')),
                                                   ('hd_reserv', '=', cas_hd_reserv),
                                                   ('hd_reserv_production_id', '=', cas_hd_reserv_production_id),
                                                   ('hd_reserv_partner_id', '=', cas_hd_reserv_partner_id),
                                                   ('hd_type', 'in', context.get('types')),
                                                   ('state', 'in', context.get('states')),
                                                   ('hd_location_id', 'in', locations),
                                                   ('hd_qty_virt','>=', 0.0)],
                                         order='life_date, id asc')
        elif len(lots_requested) and  not(context.get('cas_check_assign_stock_move', False) or context.get('cas_action_move_stock_picking', False)):
            #print "lots_requested and not (cas_check_assign_stock_move or cas_action_move_stock_picking)"
            lots_ok = self.search(cr, uid, [('id', 'in', lots_requested),
                                                   ('product_id', '=', context.get('product_id')),
                                                   ('hd_reserv', '=', cas_hd_reserv),
                                                   ('hd_reserv_production_id', '=', cas_hd_reserv_production_id),
                                                   ('hd_reserv_partner_id', '=', cas_hd_reserv_partner_id),
                                                   ('hd_type', 'in', context.get('types')),
                                                   ('state', 'in', context.get('states')),
                                                   ('hd_location_id', 'in', locations),
                                                   ('hd_qty_virt','>', 0.0)],
                                         order='life_date, id asc')
        else:
            #print "CAS GENERAL"
            lots_ok = self.search(cr, uid, [
                                                   ('product_id', '=', context.get('product_id')),
                                                   ('hd_reserv', '=', cas_hd_reserv),
                                                   ('hd_reserv_production_id', '=', cas_hd_reserv_production_id),
                                                   ('hd_reserv_partner_id', '=', cas_hd_reserv_partner_id),
                                                   ('hd_type', 'in', context.get('types')),
                                                   ('state', 'in', context.get('states')),
                                                   ('hd_location_id', 'in', locations),
                                                   ('hd_qty_virt','>', 0.0)
                                                   ],
                                         order='life_date, id asc')
        #print "lots_ok ", lots_ok
        
        # traitement des lots demandes
        if len(lots_requested) > 0:
            #print "traitement des lots demandes"
            lots_retenus = [x for x in lots_ok if x in lots_requested]
            lots_rejected = [x for x in lots_requested if x not in lots_retenus]
            del(lots_ok[:])
            lots_ok.extend(lots_retenus)
            # traitement de la qte
            for lot in self.browse(cr, uid, lots_ok):
                qty_available += lot.hd_qty_virt
            if (qty_requested > 0) :# calcul qte demandee
                q = uom_obj._compute_qty(cr, uid, uom_requested,qty_requested,to_uom_id=product_obj.browse(cr, uid, context['product_id']).uom_id.id)
                #print 'qty_requested ', q
                if not (qty_available < q): res['ind_qty_available'] = True
            elif qty_requested == 0.0:
                # dans le cas ou la qte n'est pas passee en parametre, verifier simplement que la qte virtuelle du lot n'est pas < 0
                # ce cas est utilise dans MO /stock Pickings (car la qte virtuelle du lot tient deja compte de la qte demandee)
                if not (qty_available < 0): res['ind_qty_available'] = True
            # constitution du message 'infos'
            if ((len(lots_ok) > 0) and (qty_requested > 0) and (not res['ind_qty_available'])) or (len(lots_rejected) > 0):
                '''hdcm 2013_08_15message += '%s%s\n%s\n%s\n\t%s%s\n\t%s%s\n\t%s%s\n\t%s%s\n' % (
                                            _('LOTS REQUESTED : '), ', '.join(str(x) for x in lots_requested if x), '============================================================================================',
                                            _('Used criterias for lots : '),
                                            _('Types : '), ', '.join(str(x) for x in context.get('types') if x),
                                            _('States : '), ', '.join(str(x) for x in context.get('states') if x),
                                            _('Product : '), product_obj.browse(cr, uid, context['product_id']).name,
                                            _('Locations : '), ', '.join(str(x.name) for x in stock_location_obj.browse(cr,uid,locations)),
                                            )'''
                message += '%s%s\n%s\n%s\n\t%s%s\n\t%s%s\n\t%s%s\n\t%s%s\n' % (
                                            _('LOTS REQUESTED : '), ', '.join(str(x) for x in lots_requested if x), '===============',
                                            _('Used criterias for lots : '),
                                            _('Types : '), ', '.join(str(x) for x in context.get('types') if x),
                                            _('States : '), ', '.join(str(x) for x in context.get('states') if x),
                                            _('Product : '), product_obj.browse(cr, uid, context['product_id']).name,
                                            _('Locations : '), ', '.join(str(x.name) for x in stock_location_obj.browse(cr,uid,locations)),
                                            )
                if ((len(lots_ok) > 0) and (qty_requested > 0) and (not res['ind_qty_available'])):
                    '''hdcm 2013_08_15 message += '%s\n%s%s\n%s\n%s\n\t%s%s%s%s\n' % (
                                            '============================================================================================',
                                            _('Following lots are accepted : '), ', '.join(str(x) for x in lots_ok if x), '-----------------',
                                            _('QUANTITY AVAILABLE INSUFFICIENT'),
                                            _('Qty available : '), str(qty_available), ' ', product_obj.browse(cr, uid, context['product_id']).uom_id.name,
                                                            )'''
                    message += '%s\n%s%s\n%s\n%s\n\t%s%s%s%s\n' % (
                                            '===============',
                                            _('Following lots are accepted : '), ', '.join(str(x) for x in lots_ok if x), '-----------------',
                                            _('QUANTITY AVAILABLE INSUFFICIENT'),
                                            _('Qty available : '), str(qty_available), ' ', product_obj.browse(cr, uid, context['product_id']).uom_id.name,
                                                            )
                    if qty_requested > 0:
                        message += '\t%s%s%s%s\n\n' % (
                                            _('Qty requested : '), str(qty_requested), ' ', uom_obj.browse(cr, uid, uom_requested).name,
                                         )
                
                if len(lots_rejected) > 0:
                    '''hdcm 2013_08_15 message += '%s\n%s%s\n' % (
                                            '============================================================================================',
                                            _('Following lots are refused : '), ', '.join(str(x) for x in lots_rejected if x), 
                                                        )'''
                    message += '%s\n%s%s\n' % (
                                            '===============',
                                            _('Following lots are refused : '), ', '.join(str(x) for x in lots_rejected if x), 
                                                        )
        
        res['lots_ok'] = lots_ok
        res['lots_rejected'] = lots_rejected
        res['infos'] = message
        res['qty_available'] = qty_available
        #print "/***lot_control2(stock_production_lot) - res ",res
        return res

    def unlink(self, cr, uid, ids, context={}):
        raise osv.except_osv(_('Invalid action !'), _('You can not remove a lot !'))
       
    # retour des reliquats des lots splits d'une fab
    # Principe : 
    #    -    si le lot à retourner a le même état que le lot d'origine, faire une opération de transfert
    #         sinon il faudra faire un stock_move_lot manuel
    def return_to_stock_one_lot(self, cr, uid, ids,  context=None):
        #print "***return_to_stock_one_lot(stock_production_lot) - ids ",ids," - context ",context
        if context is None : context = {}
        pick_obj = self.pool.get('stock.picking')
        move_obj = self.pool.get('stock.move')
        wf_service = netsvc.LocalService("workflow")
        ind_pick=False
        lot_split_in_ids = self.browse(cr, uid, ids)
        for lot_split_in in lot_split_in_ids:
            if lot_split_in.hd_type == 'fab_stock': continue
            if lot_split_in.hd_type == 'in': continue
            if (lot_split_in.state != self.browse(cr, uid, lot_split_in.hd_lot_org_id).state) or (lot_split_in.hd_qty_virt <= 0):
                continue
                #print "lot_split_in.id : ",lot_split_in.id
            if not ind_pick:
                ind_pick=True
                # créer un picking de type transfer
                picking_id = pick_obj.create(cr, uid, {
                            'origin': (lot_split_in.hd_production_id_spl.origin or '') +':'+lot_split_in.hd_production_id_spl.name,
                            'type': 'transfer',
                            'move_type': 'one',
                            'state': 'draft',
                            'auto_picking': False,
                            })
            # création du ou des moves pour ce transfert
            # --- rechercher le lot d'origine
            lot_dest_obj= self.browse(cr, uid, lot_split_in.hd_lot_org_id)
            move_id = move_obj.create(cr, uid, {
                        'name':lot_split_in.product_id.name,
                        'picking_id':picking_id,
                        'product_id': lot_split_in.product_id.id,
                        'product_qty': lot_split_in.hd_qty_virt,
                        'product_uom': lot_split_in.hd_product_uom_id.id,
                        'hd_lot_org':lot_split_in.id,
                        'hd_lot_dest_transf':lot_dest_obj.id,
                        'location_id': lot_split_in.hd_location_id.id,
                        'location_dest_id': lot_dest_obj.hd_location_id.id,
                        'state': 'draft',
                        })
        if ind_pick:
            wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
            wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_done', cr)
        
        ''''HDcm 2012_03_27 603_02chem retrouver dans les lot_split_in les lots d'origine
                                        et annuler la reservation de ces lots d'origine'''
        for lot_split_in_id in lot_split_in_ids:
            if lot_split_in_id.hd_lot_org_id and (self.browse(cr, uid,lot_split_in_id.hd_lot_org_id).hd_reserv != 'no'):
                context['prog'] = True
                self.write(cr, uid, [lot_split_in_id.hd_lot_org_id],{
                                                                     'hd_reserv' : 'no',
                                                                     }, context=context)
        #print "/***return_to_stock_one_lot(stock_production_lot)"
        return True


    # cette fonction est lancée lors du test automatique de la transition
    def trans_conform__conform_alerte_wkf(self, cr, uid, ids=False, context={}):
        #print"***trans_conform_conform_alerte_wkf -  uid : ", uid, " - ids : ", ids
        if uid==1 and ids: # cas du lancement automatique de la fonction par le root
            stock_lot_obj=self.browse(cr, uid, ids[0])
            # vérifier que la date 'alert_date' n'est pas dépassée (si cette date existe)
            if stock_lot_obj.alert_date and time.strftime('%Y-%m-%d')> stock_lot_obj.alert_date:
                #print"/***trans_conform__conform_alerte_wkf - date dépassée"
                return True
        #print"***/trans_conform_conform_alerte_wkf - date non dépassée"
        return False

    # cette fonction est lancée lors du test automatique de la transition
    def trans_conform_alerte__perime_wkf(self, cr, uid, ids=False, context={}):
        #print"***trans_conform_alerte__perime_wkf - "," - uid : ", uid, " - ids : ", ids, " context : ", context
        if uid==1 and ids: # cas du lancement automatique de la fonction par le root
            stock_lot_obj=self.browse(cr, uid, ids[0])
            # vérifier que la date 'life_date' n'est pas dépassée (si cette date existe)
            if stock_lot_obj.life_date and time.strftime('%Y-%m-%d')> stock_lot_obj.life_date:
                #print"/***trans_conform_alerte__perime_wkf - date dépassée"
                return True
        #print"***/trans_conform_alerte__perime_wkf - date non dépassée"
        return False
            
    # test automatique de la transition conform-alert_conform
    def test_conform_alerte_wkf(self, cr, uid, ids=False, context={}):
        #print "***test_conform_alerte_wkf - "," - uid : ",uid, " - ids : ",ids, " context : ",context
        if uid==1:    # cas du lancement automatique de la fonction par le root
            # search des lots dont indtrack=True ET state='conform'
            ids = self.pool.get('stock.production.lot').search(cr, uid, [('hd_indtrack', '=', 'True'),('state', '=', 'conform')])
            #print "ids ",ids
            # pour chaque lot, activer le WKF
            for id in ids:
                wf_service = netsvc.LocalService("workflow")
                wf_service.trg_write(uid, 'stock.production.lot', id, cr)
        #print "/***test_conform_alerte_wkf"
        return True

    # test automatique de la transition alert_conform - out_of_date
    def test_perime_wkf(self, cr, uid, ids=False, context={}):
        #print "***test_perime_wkf - "," - uid : ",uid, " - ids : ",ids, " context : ",context
        if uid==1:    # cas du lancement automatique de la fonction par le root
            # search des lots dont indtrack=True ET state='alert_conform'
            ids = self.pool.get('stock.production.lot').search(cr, uid, [('hd_indtrack', '=', 'True'),('state', '=', 'alert_conform')])
            #print "ids ",ids
            # pour chaque lot, activer le WKF
            for id in ids:
                wf_service = netsvc.LocalService("workflow")
                wf_service.trg_write(uid, 'stock.production.lot', id, cr)
        #print "/***test_perime_wkf"
        return True

    def get_name_lot(self, cr, uid, prefix, product, hd_prodlot_id):
        #print "***get_name_lot(stock_production_lot) - prefix : ",prefix, " product : ",product, " hd_prodlot_id : ",hd_prodlot_id
        name_lot = self.pool.get('ir.sequence').get(cr, uid, 'stock.lot.hd')
        if not name_lot:
            name_lot = prefix
            if product.code:
                name_lot += '[%s]'%(product.code)
            name_lot += '[%s]:%07d'%(time.strftime('%Y-%m-%d'), hd_prodlot_id)
        #print "/***get_name_lot(stock_production_lot) - name_lot : ",name_lot
        return name_lot

    def info_one_lot(self, cr, uid, lot_id, context=None):
        #print " ***info_one_lot(stock_production_lot) - lot_id %s - context %s"%(lot_id, context)
        if context is None: context = {}
        
        obj_lot = self.pool.get('stock.production.lot')
        obj_product=self.pool.get('product.product')
        obj_product_uom = self.pool.get('product.uom')
        obj_stock_location = self.pool.get('stock.location')
        col11='40%'
        col12='40%'
        col13='20%'
        col21='60%'
        col22='20%'
        col23='20%'
        lig_bar = 85
        lig_bar2 = 80
        res_info_one_lot=''
        #HDcm 2012_04_14 604_02chem forcer le recalcul des lots
        obj_lot.force_calc_lot(cr, uid, [lot_id], context=context)
        #hdcm 2013_11_18 for lot in obj_lot.browse(cr, uid, lot_ids):
        if lot_id:
            lot = obj_lot.browse(cr, uid, lot_id)
            # cas des lots de produits non tracés
            if lot.hd_type=='xxx':
                res_info_one_lot= '<p>This feature provides information only for products with the indicator "Manage by Lots" is activated</p>'
                return res_info_one_lot
            
            # TRAITEMENT DE L'ENTETE
            if lot.hd_type=='in':
                des_type=_('PURCHASE')
            elif lot.hd_type=='sale_stock':
                des_type=_('SALE FROM STOCK')
            elif lot.hd_type=='sale_fab':
                des_type=_('SALE FROM PRODUCTION')
            elif lot.hd_type=='sale_buy':
                des_type=_('SALE FROM PURCHASE')
            elif lot.hd_type=='fab_stock':
                des_type=_('MANUFACTURING FOR THE STOCK')
            elif lot.hd_type=='int':
                des_type=_('INTERNAL')
            elif lot.hd_type=='split':
                des_type=_('SPLITTING') 
            elif lot.hd_type=='inv':
                des_type=_('INVENTORY')
            
            name_prod=obj_product.browse(cr, uid, lot.product_id.id).name
            uom_name=obj_product_uom.browse(cr, uid, lot.hd_product_uom_id.id).name
            loc_name=obj_stock_location.browse(cr, uid, lot.hd_location_id.id).name
            q1 = obj_product_uom._compute_qty(cr, uid, lot.hd_product_uom_id.id, lot.hd_qty_real, to_uom_id=lot.product_id.uom_id.id)
            q2 = obj_product_uom._compute_qty(cr, uid, lot.hd_product_uom_id.id, lot.hd_qty_virt, to_uom_id=lot.product_id.uom_id.id)
            q = obj_product_uom._compute_qty(cr, uid, lot.hd_product_uom_id.id, lot.hd_qty_org, to_uom_id=lot.product_id.uom_id.id)
            
            res_info_one_lot += '<p>' + "="*lig_bar + '</p>\
                       <table class="info_lot_table_entete" cellspacing=0 cellpadding=0 width=100%>\
                       <tr><th width='+col11+'>Lot ' + des_type + '</th><th width='+col12+'>' + lot.name + '</th><th>'+_("Product")+'&nbsp;:&nbsp;' + name_prod + '</th></tr>\
                       <tr><td></td><td></td><td>'+_("Location")+'&nbsp;:&nbsp;' + loc_name + '</td></tr>\
                       </table>\
                       <p>'+ "="*lig_bar +'</p>'
                       
            res_info_one_lot += '<table class="info_lot_table_corps" cellspacing=0 cellpadding=0 width=100%>'
            
            # TRAITEMENT DE L'ORIGINE
            obj_purchase_order = self.pool.get('purchase.order')
            obj_sale_order = self.pool.get('sale.order')
            obj_mrp_production = self.pool.get('mrp.production')
            obj_stock_picking = self.pool.get('stock.picking')
            obj_stock_inventory_lot = self.pool.get('stock.inventory.lot')
            obj_stock_move = self.pool.get('stock.move')
            if lot.hd_type=='in':
                if lot.hd_doc_org_id:
                    purchase_order_obj=obj_purchase_order.browse(cr, uid, lot.hd_doc_org_id)
                    name_po=purchase_order_obj.name
                    name_partner=purchase_order_obj.partner_id.name
                    #hdcm v604 ref_partner=purchase_order_obj.partner_id.ref
                    ref_partner = purchase_order_obj.partner_id.ref and purchase_order_obj.partner_id.ref or ''
                    
                    res_info_one_lot += '<tr><td width='+col21+'>'+_("Outcome of : Purchase order")+' ('+name_po+')</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                    res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;%s : %s (%s)'%(_("Supplier"),name_partner,ref_partner) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                    
            elif lot.hd_type=='sale_stock':
                sale_order_obj=obj_sale_order.browse(cr, uid, lot.hd_doc_org_id)
                name_so=sale_order_obj.name
                name_partner=sale_order_obj.partner_id.name
                #hdcm v604 ref_partner=sale_order_obj.partner_id.ref.encode("ISO-885 Supplier9-15")
                ref_partner = sale_order_obj.partner_id.ref and sale_order_obj.partner_id.ref or ''
                
                res_info_one_lot += '<tr><td width='+col21+'>'+_("Outcome of : Sale from stock")+' (%s)'%(name_so) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;%s : %s (%s)'%(_("Customer"),name_partner,ref_partner) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                res_info_one_lot += '<tr><td width='+col21+'>'+_("Deducted from the lot")+' %s</p>'%(obj_lot.browse(cr, uid, lot.hd_lot_org_id).name) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                
            elif lot.hd_type=='sale_fab':    
                sale_order_obj=obj_sale_order.browse(cr, uid, lot.hd_doc_org_id)
                name_so=sale_order_obj.name
                name_partner=sale_order_obj.partner_id.name
                #hdcm v604 ref_partner=sale_order_obj.partner_id.ref
                ref_partner = sale_order_obj.partner_id.ref and sale_order_obj.partner_id.ref or ''
                mrp_production_obj=obj_mrp_production.browse(cr, uid, lot.hd_doc_inter_id)
                name_of=mrp_production_obj.name
                
                res_info_one_lot += '<tr><td width='+col21+'>'+_("Outcome of : Sale from production")+' (%s)'%(name_so) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;%s : %s (%s)'%(_("Customer"),name_partner,ref_partner) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                res_info_one_lot += '<tr><td width='+col21+'>'+_("Manufacturing order")+' : %s</p>'%(name_of) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                
                #!!!res_info_one_lot+=string.ljust('    '+'Procurement : %s'%(name_procurement),col1)+'\n'
            elif lot.hd_type=='sale_buy':    
                sale_order_obj=obj_sale_order.browse(cr, uid, lot.hd_doc_org_id)
                name_so=sale_order_obj.name
                name_partner=sale_order_obj.partner_id.name
                #hdcm v604 ref_partner=sale_order_obj.partner_id.ref
                ref_partner = sale_order_obj.partner_id.ref and sale_order_obj.partner_id.ref or ''
                
                res_info_one_lot += '<tr><td width='+col21+'>'+_("Outcome of : Sale from purchase")+' (%s)'%(name_so) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;%s : %s (%s)</p>'%(_("Customer"),name_partner,ref_partner) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                
            elif lot.hd_type=='fab_stock':
                mrp_production_obj=obj_mrp_production.browse(cr, uid, lot.hd_doc_org_id)
                name_of=mrp_production_obj.name
                res_info_one_lot += '<tr><td width='+col21+'>'+_("Outcome of : Manufacturing order")+' (%s)</p>'%(name_of) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                
            elif lot.hd_type=='int':
                res_info_one_lot += '<tr><td width='+col21+'>'+_("Took in the lot")+' %s</p>'%(obj_lot.browse(cr, uid, lot.hd_lot_org_id).name) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                
            elif lot.hd_type=='split':
                # retrouver le splitting
                stock_picking_obj=obj_stock_picking.browse(cr, uid, lot.hd_doc_org_id)
                name_split=stock_picking_obj.name
                if lot.hd_lot_org_id:#hdcm2010_01_04 test ajoute pour prendre en compte les produits consu geres
                    res_info_one_lot += '<tr><td width='+col21+'>'+_("Took in the lot")+' %s'%(obj_lot.browse(cr, uid, lot.hd_lot_org_id).name) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                    res_info_one_lot += '<tr><td width='+col21+'>'+_("Splitting operation")+' : %s</p>'%(name_split) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                    
            elif lot.hd_type=='inv':
                # retrouver l'inventory
                stock_inventory_lot=obj_stock_inventory_lot.browse(cr, uid, lot.hd_doc_org_id)
                res_info_one_lot += '<tr><td width='+col21+'>'+_("Created during inventory")+' (%s)</p>'%(stock_inventory_lot.name) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                
            res_info_one_lot += '<tr><td width='+col21+'></td><TH width='+col22+'>'+_("Real qty")+'</TH><TH width='+col23+'>'+_("Virtual qty")+'</TH></tr>\
                       <tr><td width='+col21+'>'+_("Qty origin")+' : '+ str(q) + ' '+ uom_name +  '</td><td width='+col22+'>' + str(q1) + '</td><td width='+col23+'>' + str(q2) + '</td></tr>\
                       </table>'
            
            # LOTS UTILISES POUR UNE FABRICATION
            if (lot.hd_type=='sale_fab') or (lot.hd_type=='fab_stock'):
                # rechercher les lots qui ont été utilisés pour la fabrication
                premier_passage=True
                nb_passage = 0
                for lot_util_obj in obj_lot.browse(cr, uid, obj_lot.search(cr, uid, [('hd_lot_dest_id', '=', lot.id)])):
                    if premier_passage:
                        
                        res_info_one_lot += '<table class="info_lot_table_corps" cellspacing=0 cellpadding=0 width=100%>'
                        res_info_one_lot += '<tr><td width='+col21+'>' + "-"*lig_bar2 + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                        res_info_one_lot += '<tr><td width='+col21+'>'+_("Lots used for manufacturing")+'</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                        
                        premier_passage=False
                    q = obj_product_uom._compute_qty(cr, uid, lot_util_obj.hd_product_uom_id.id, lot_util_obj.hd_qty_org, 
                                             to_uom_id=lot_util_obj.product_id.uom_id.id)
                    
                    res_info_one_lot += '<tr><td width='+col21+'>Lot : %s ('+_("Qty")+' : %s)'%(lot_util_obj.name,str(q)) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                    
                    nb_passage += 1
                if nb_passage:
                    res_info_one_lot += '</table>'
            
            # TRAITEMENT DES PRELEVEMENTS    
            #HDcm 2012_04_26 506_06chem if (lot.hd_type=='in') or (lot.hd_type=='fab_stock') or (lot.hd_type=='split') or (lot.hd_type=='inv'):
            #HDcm 2012_04_26 604_03chem ajout du cas 'sale_stock' pour tenir compte d'un retour produit apres une venete
            if (lot.hd_type=='in') or (lot.hd_type=='fab_stock') or (lot.hd_type=='split') or (lot.hd_type=='inv') or (lot.hd_type=='sale_stock'):
                # rechercher les lots qui ont pour origine ce lot
                premier_passage=True
                nb_passage = 0
                for lot_prel_obj in obj_lot.browse(cr, uid, obj_lot.search(cr, uid, [('hd_lot_org_id', '=', lot.id)])):
                    #print "lot_prel_obj.id ", lot_prel_obj.id
                    if premier_passage:
                        res_info_one_lot += '<table class="info_lot_table_corps" cellspacing=0 cellpadding=0 width=100%>'
                        res_info_one_lot += '<tr><td width='+col21+'>' + "-"*lig_bar2 + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                        res_info_one_lot += '<tr><td width='+col21+'>'+_("Taking(s)")+'</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                        
                        premier_passage=False
                    q = obj_product_uom._compute_qty(cr, uid, lot_prel_obj.hd_product_uom_id.id, lot_prel_obj.hd_qty_org, 
                                             to_uom_id=lot_prel_obj.product_id.uom_id.id)
                    
                    res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;Lot : %s'%(lot_prel_obj.name) +\
                                    '</td><td width='+col22+'>' + str(-q) + '</td><td width='+col23+'></td></tr>'
                    
                    # cas d'un pélèvement à partir d'un SO    
                    if lot_prel_obj.hd_type=='sale_stock': 
                        so_obj=obj_sale_order.browse(cr,uid,lot_prel_obj.hd_doc_org_id)
                        ref_partner = so_obj.partner_id.ref and so_obj.partner_id.ref or ''#hdcm 2013_10_01
                        
                        res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;&nbsp;&nbsp;%s    %s : %s (%s)'%(so_obj.name,_("Customer"),so_obj.partner_id.name,ref_partner) +\
                                    '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                        
                    # cas d'un pélèvement à partir d'un SPLIT
                    # retrouver le nom du split
                    if lot_prel_obj.hd_type=='split':
                        split_obj=obj_stock_picking.browse(cr,uid,lot_prel_obj.hd_doc_org_id)
                        
                        res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;&nbsp;&nbsp;%s'%(split_obj.name) +\
                                    '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                        
                    nb_passage += 1
                # cas d'un pélèvement à partir d'un transfert    
                # trouver les moves dont le lot_org correspond à ce lot.id ET qui n'a pas de lot_id ET qui soient 'done'
                for stock_move_obj in obj_stock_move.browse(cr, uid, obj_stock_move.search(cr, uid, [('hd_lot_org', '=', lot.id),('state','=','done')])):
                    if stock_move_obj.picking_id.id and (stock_move_obj.picking_id.type=='transfer'):
                        if premier_passage:
                            res_info_one_lot += '<table class="info_lot_table_corps" cellspacing=0 cellpadding=0 width=100%>'
                            res_info_one_lot += '<tr><td width='+col21+'>' + "-"*lig_bar2 + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                            res_info_one_lot += '<tr><td width='+col21+'>'+_("Taking(s)")+'</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                            
                            premier_passage=False
                        # retrouver le nom du transfert
                        transfer_name=obj_stock_picking.browse(cr,uid,stock_move_obj.picking_id.id).name
                        q = obj_product_uom._compute_qty(cr, uid, stock_move_obj.product_uom.id, stock_move_obj.product_qty, 
                                             to_uom_id=stock_move_obj.product_id.uom_id.id)
                        res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;%s'%(transfer_name) +\
                                    '</td><td width='+col22+'>' + str(-q) + '</td><td width='+col23+'></td></tr>'
                        
                        # mentionner le lot destinataire du transfert
                        res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;&nbsp;&nbsp;'+_("to lot")+' : %s'%(stock_move_obj.hd_lot_dest_transf.name) +\
                                    '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                        
                    nb_passage += 1
                if nb_passage:
                    res_info_one_lot += '</table>'
            elif lot.hd_type=='sale_stock':
                pass
            elif lot.hd_type=='int':
                pass
            
            
            # TRAITEMENT DES AJOUTS    
            #HDcm 2012_02_21 v506_06chem if (lot.hd_type=='in') or (lot.hd_type=='fab_stock') or (lot.hd_type=='split'):
            if (lot.hd_type=='in') or (lot.hd_type=='fab_stock') or (lot.hd_type=='split') or (lot.hd_type=='inv'):#HDcm 2012_02_21
                # trouver les move dont le lot lot_dest_transf correspond à ce lot.id ET qui soient 'done'
                obj_stock_move = self.pool.get('stock.move')
                obj_stock_picking = self.pool.get('stock.picking')
                premier_passage=True
                nb_passage = 0
                for stock_move_obj in obj_stock_move.browse(cr, uid, obj_stock_move.search(cr, uid, [('hd_lot_dest_transf', '=', lot.id),('state','=','done')])):
                    if premier_passage:
                        res_info_one_lot += '<table class="info_lot_table_corps" cellspacing=0 cellpadding=0 width=100%>'
                        res_info_one_lot += '<tr><td width='+col21+'>' + "-"*lig_bar2 + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                        res_info_one_lot += '<tr><td width='+col21+'>'+_("Addition(s)")+'</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                        
                        premier_passage=False
                    # retrouver le nom du retour
                    transfer_name=obj_stock_picking.browse(cr,uid,stock_move_obj.picking_id.id).name
                    q = obj_product_uom._compute_qty(cr, uid, stock_move_obj.product_uom.id, stock_move_obj.product_qty, 
                                             to_uom_id=stock_move_obj.product_id.uom_id.id)
                    res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;%s'%(transfer_name) + '</td><td width='+col22+'>' + str(q) + '</td><td width='+col23+'></td></tr>'
                    
                    # mentionner le lot destinataire du transfert
                    res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;&nbsp;&nbsp;'+_("from lot")+' : %s'%(stock_move_obj.hd_lot_org.name) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                    
                    nb_passage += 1
                if nb_passage:
                    res_info_one_lot += '</table>'
                
            elif lot.hd_type=='sale_stock':
                pass
            elif lot.hd_type=='int':
                pass
            
            # CAS DU LOT INT : RETROUVER LE LOT DE FAB    
            if lot.hd_type=='int' and lot.hd_lot_dest_id:
                res_info_one_lot += '<table class="info_lot_table_corps" cellspacing=0 cellpadding=0 width=100%>'
                res_info_one_lot += '<tr><td width='+col21+'>' + "-"*lig_bar2 + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                res_info_one_lot += '<tr><td width='+col21+'>'+_("For the manufacture of the lot")+'</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                
                name_lot_of=obj_lot.browse(cr, uid, lot.hd_lot_dest_id).name
                qty_lot_of=obj_lot.browse(cr, uid, lot.hd_lot_dest_id).hd_qty_org
                uom_lot_of=obj_lot.browse(cr, uid, lot.hd_lot_dest_id).hd_product_uom_id.id
                q = obj_product_uom._compute_qty(cr, uid, uom_lot_of, qty_lot_of, to_uom_id=lot.product_id.uom_id.id)
                res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;%s ('+_("qty")+' : %s)'%(name_lot_of,str(q)) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                
                res_info_one_lot += '</table>'
            
            # TRAITEMENT DES RESERVATIONS ET DES RETOURS
            if (lot.hd_type=='in') or (lot.hd_type=='fab_stock') or (lot.hd_type=='split'):
                
                # si la qté réelle est diff de la quantité virtuelle, 
                # la diff représente les qtés réservées ET/OU les retours
                premier_passage=True
                nb_passage = 0
                if lot.hd_qty_real <> lot.hd_qty_virt:
                    # CAS DES RESERVATIONS
                    # trouver les move dont le lot lot_org correspond à ce lot.id ET qui ne soient pas 'done'
                    obj_stock_move = self.pool.get('stock.move')
                    obj_sale_order_line = self.pool.get('sale.order.line')
                    for stock_move_obj in obj_stock_move.browse(cr, uid, obj_stock_move.search(cr, uid, [('hd_lot_org', '=', lot.id),('state','!=','draft'),('state','!=','waiting'),('state','!=','done'),('state','!=','cancel')])):
                        if premier_passage:
                            res_info_one_lot += '<table class="info_lot_table_corps" cellspacing=0 cellpadding=0 width=100%>'
                            res_info_one_lot += '<tr><td width='+col21+'>' + "-"*lig_bar2 + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                            res_info_one_lot += '<tr><td width='+col21+'>'+_("Reservation(s) by")+'</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                            
                            premier_passage=False
                        # CAS D'UNE RESERVATION PAR SO
                        if stock_move_obj.sale_line_id.id: # retrouver le SO
                            so_obj=obj_sale_order_line.browse(cr,uid,stock_move_obj.sale_line_id.id).order_id
                            ref_partner = so_obj.partner_id.ref and so_obj.partner_id.ref or ''#hdcm 2013_10_01
                            q = obj_product_uom._compute_qty(cr, uid, stock_move_obj.product_uom.id, stock_move_obj.product_qty, 
                                             to_uom_id=stock_move_obj.product_id.uom_id.id)
                            res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;%s'%(so_obj.name) + '</td><td width='+col22+'></td><td width='+col23+'>' + str(-q) + '</td></tr>'
                            res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;&nbsp;&nbsp;%s : %s (%s)'%(_("Customer"),so_obj.partner_id.name,ref_partner) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                            
                        # CAS D'UNE RESERVATION PAR UNE FAB
                        if stock_move_obj.move_dest_id.id and (stock_move_obj.product_qty==stock_move_obj.move_dest_id.product_qty): 
                            q = obj_product_uom._compute_qty(cr, uid, stock_move_obj.product_uom.id, stock_move_obj.product_qty, 
                                             to_uom_id=stock_move_obj.product_id.uom_id.id)
                            res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;%s'%(stock_move_obj.name) + '</td><td width='+col22+'></td><td width='+col23+'>' + str(-q) + '</td></tr>'
                            
                        # CAS D'UNE RESERVATION PAR SPLIT
                        elif stock_move_obj.picking_id.id and stock_move_obj.picking_id.type=="split":
                            obj_stock_picking = self.pool.get('stock.picking')
                            split_name=obj_stock_picking.browse(cr,uid,stock_move_obj.picking_id.id).name
                            q = obj_product_uom._compute_qty(cr, uid, stock_move_obj.product_uom.id, stock_move_obj.product_qty, 
                                             to_uom_id=stock_move_obj.product_id.uom_id.id)
                            res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;%s'%(split_name) + '</td><td width='+col22+'></td><td width='+col23+'>' + str(-q) + '</td></tr>'
                            
                        # CAS D'UNE RESERVATION PAR TRANSFERT
                        elif stock_move_obj.picking_id.id and stock_move_obj.picking_id.type=="transfer":
                            obj_stock_picking = self.pool.get('stock.picking')
                            transfer_name=obj_stock_picking.browse(cr,uid,stock_move_obj.picking_id.id).name
                            q = obj_product_uom._compute_qty(cr, uid, stock_move_obj.product_uom.id, stock_move_obj.product_qty, 
                                             to_uom_id=stock_move_obj.product_id.uom_id.id)
                            res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;%s'%(transfer_name) + '</td><td width='+col22+'></td><td width='+col23+'>' + str(-q) + '</td></tr>'
                            
                            # mentionner le lot destinataire du transfert
                            res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;&nbsp;&nbsp;'+_("to lot")+' : %s'%(stock_move_obj.hd_lot_dest_transf.name) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                            
                        nb_passage += 1
                    
                    # CAS DES RESERVATIONS POUR LES RETOURS
                    # trouver les move dont le lot lot_dest_transf correspond à ce lot.id ET qui ne soient pas 'done'
                    obj_stock_move = self.pool.get('stock.move')
                    for stock_move_obj in obj_stock_move.browse(cr, uid, obj_stock_move.search(cr, uid, [('hd_lot_dest_transf', '=', lot.id),('state','!=','draft'),('state','!=','waiting'),('state','!=','done'),('state','!=','cancel')])):
                        if premier_passage:
                            res_info_one_lot += '<tableclass="info_lot_table_corps" cellspacing=0 cellpadding=0 width=100%>'
                            res_info_one_lot += '<tr><td width='+col21+'>' + "-"*lig_bar2 + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                            res_info_one_lot += '<tr><td width='+col21+'>'+_("Reservation(s) by")+'</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                            
                            premier_passage=False
                        if stock_move_obj.picking_id.type=="transfer":
                            obj_stock_picking = self.pool.get('stock.picking')
                            transfer_name=obj_stock_picking.browse(cr,uid,stock_move_obj.picking_id.id).name
                            q = obj_product_uom._compute_qty(cr, uid, stock_move_obj.product_uom.id, stock_move_obj.product_qty, 
                                             to_uom_id=stock_move_obj.product_id.uom_id.id)
                            res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;%s'%(transfer_name) + '</td><td width='+col22+'></td><td width='+col23+'>' + str(q) + '</td></tr>'
                            
                            # mentionner le lot destinataire du transfert
                            res_info_one_lot += '<tr><td width='+col21+'>&nbsp;&nbsp;&nbsp;&nbsp;'+_("from lot")+' : %s'%(stock_move_obj.hd_lot_org.name) + '</td><td width='+col22+'></td><td width='+col23+'></td></tr>'
                            
                        nb_passage += 1
                    if nb_passage:
                        res_info_one_lot += '</table>'
        
        #print "/***info_one_lot(stock_production_lot) - res_info_one_lot %s"%(res_info_one_lot)
        return res_info_one_lot


stock_production_lot()

#######################################################################################
#  Ajout d'un champ booleen 'zone_sous_controle' dans la classe stock.location
#  ce champ est utilise par la fonction 'is_zone_sous_controle' lors d'1 move
#  pour savoir si la zone concernee par le move est sous le controle du service qualite
#######################################################################################

class stock_location(osv.osv):
    _name = "stock.location"
    _inherit = 'stock.location'
    _columns = {
        'hd_zone_sous_controle': fields.boolean('Area under control', required=True),

    }
    _defaults = {
        'hd_zone_sous_controle': lambda *a: 0,
    }

    def on_change_hd_zone_sous_controle(self, cr, uid, ids, hd_zone_sous_controle):
        #print "***on_change_hd_zone_sous_controle(stock_location) - ids ",ids, " - hd_zone_sous_controle ", hd_zone_sous_controle
        result = {}
        warning = {}
        if len(ids) == 0: return {'value':result, 'warning':warning}#hdcm 2013_01_09
        stock_location_obj = self.pool.get('stock.location')
        stock_location = stock_location_obj.browse(cr, uid, ids[0])
        if hd_zone_sous_controle:
            # verifier que la location est de type 'internal location'
            if stock_location.usage != 'internal':
                result['hd_zone_sous_controle'] = self.read(cr, uid, ids, ['hd_zone_sous_controle'])[0]['hd_zone_sous_controle']
                warning={'title':_('Invalid action'), 'message':_('This parameter can be enabled only for locations of type \'Internal Location\'')}
        #print "/***on_change_hd_zone_sous_controle(stock_location) - result ",result
        return {'value':result, 'warning':warning}
    

stock_location()


class stock_inventory_lot(osv.osv):
    _name = "stock.inventory.lot"
    _description = "Inventory lot"
    _columns = {
        'name': fields.char('Inventory lots', size=64, required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'date': fields.datetime('Date create', required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'date_done': fields.datetime('Date done'),
        'inventory_lot_line_id': fields.one2many('stock.inventory.lot.line', 'inventory_id', 'Inventories lots', readonly=True, states={'draft': [('readonly', False)]}),
        'move_ids': fields.many2many('stock.move', 'stock_inventory_lot_move_rel', 'inventory_id', 'move_id', 'Created Moves'),
        'picking_ids': fields.one2many('stock.picking', 'inventory_lot_id', 'Picking List', readonly=True),
        'state': fields.selection( (('draft', 'Draft'), ('done', 'Done')), 'Status', readonly=True),
    }
    _defaults = {
        'date': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
        'state': lambda *a: 'draft',
    }

    def action_done(self, cr, uid, ids, context=None):
        #print "***action_done(stock_inventory_lot) - ids ",ids," - context ",context
        if context is None: context = {}
        picking_obj = self.pool.get('stock.picking')#HDcm 2012_02_20
        wf_service = netsvc.LocalService("workflow")#HDcm 2012_04_12 604_02chem
        move_obj = self.pool.get('stock.move')#HDcm 2012_04_12 604_02chem
        for inv in self.browse(cr, uid, ids):
            if inv.state == 'done':
                continue
            move_ids = []
            for inventory_lot_line in [x.id for x in inv.inventory_lot_line_id]:
                inventory_lot_line_obj = self.pool.get('stock.inventory.lot.line').browse(cr, uid, inventory_lot_line)
                if abs(inventory_lot_line_obj.lot_id.hd_qty_real - inventory_lot_line_obj.product_qty) < 0.000000000001: # cas qty reelle du lot = qty ligne inventaire
                    #print "égalité"
                    continue
                # creation d'un lot INV en zone inventory
                # création d'un picking de type 'in'
                picking_inv_id = self.pool.get('stock.picking').create(cr, uid, {
                            'origin': 'INV:'+ inv.name,
                            'type': 'in',
                            'move_type': 'one',
                            'inventory_lot_id': inv.id,
                            })
                # cas qty reelle du lot > qty ligne inventaire
                if inventory_lot_line_obj.lot_id.hd_qty_real > inventory_lot_line_obj.product_qty:
                    qty_move = 0.0
                else:# cas qty reelle du lot < qty ligne inventaire
                    qty_move = inventory_lot_line_obj.product_qty - inventory_lot_line_obj.lot_id.hd_qty_real
                # création du move avec location_id et location_dest_id pointant sur la zone inventaire de ce produit
                location_inventory_id = inventory_lot_line_obj.product_id.product_tmpl_id.property_stock_inventory.id
                move_inv_id = self.pool.get('stock.move').create(cr, uid, {
                            'name':'INV:'+inv.name,
                            'date_planned': time.strftime('%Y-%m-%d %H:%M:%S'),
                            'picking_id':picking_inv_id,
                            'product_id': inventory_lot_line_obj.product_id.id,
                            'product_qty': qty_move,
                            'product_uom': inventory_lot_line_obj.product_uom.id,
                            'location_id': location_inventory_id,
                            'location_dest_id': location_inventory_id,
                            'state': 'draft',
                            })
                wf_service.trg_validate(uid, 'stock.picking', picking_inv_id, 'button_confirm', cr)
                context['appel_prog'] = True
                picking_obj.force_assign(cr, uid,[picking_inv_id], context=context)#HDcm 2012_04_04 v604_02chem
                picking_obj.action_move(cr, uid, [picking_inv_id])#HDcm 2012_02_20
                wf_service.trg_validate(uid, 'stock.picking', picking_inv_id, 'button_done', cr)
                
                #HDcm 2012_04_12 604_02chem - faire l'historique
                #HDcm 2012_04_12 604_02chem - rechercher le move correpondant au lot inventorie
                move_lot_inv_id = move_obj.search(cr, uid, [('hd_prodlot_id', '=', inventory_lot_line_obj.lot_id.id)])[0]
                #print "history 1 HD - parent ", move_lot_inv_id, " - child ", move_inv_id
                cr.execute('insert into stock_move_history_ids (parent_id,child_id) values (%s,%s)', (move_lot_inv_id, move_inv_id))
                
                # chercher l'id du lot INV cree                
                lot_inv_id = self.pool.get('stock.move').browse(cr, uid, move_inv_id).hd_prodlot_id.id
                # recopier dates et state du lot inventaire
                self.pool.get('stock.production.lot').write(cr,uid, [lot_inv_id], {
                            'life_date':inventory_lot_line_obj.lot_id.life_date,
                            'use_date':inventory_lot_line_obj.lot_id.use_date,
                            'removal_date':inventory_lot_line_obj.lot_id.removal_date,
                            'alert_date':inventory_lot_line_obj.lot_id.alert_date,
                            'state':inventory_lot_line_obj.lot_id.state
                            })
                # creer un transfert entre le lot traite et le lot qui vient d'etre cree
                picking_transfer_id = self.pool.get('stock.picking').create(cr, uid, {
                            'origin': 'INV:'+ inv.name,
                            'type': 'transfer',
                            'move_type': 'one',
                            'inventory_lot_id': inv.id,
                            })    
                # création du move 
                # cas qty reelle du lot > qty ligne inventaire
                if inventory_lot_line_obj.lot_id.hd_qty_real > inventory_lot_line_obj.product_qty:
                    qty_move_transfer = inventory_lot_line_obj.lot_id.hd_qty_real - inventory_lot_line_obj.product_qty
                    hd_lot_org = inventory_lot_line_obj.lot_id.id
                    hd_lot_dest_transf = lot_inv_id
                    location_id = inventory_lot_line_obj.location_id.id
                    location_dest_id = location_inventory_id
                else:# cas qty reelle du lot < qty ligne inventaire
                    qty_move_transfer = inventory_lot_line_obj.product_qty - inventory_lot_line_obj.lot_id.hd_qty_real
                    hd_lot_org = lot_inv_id
                    hd_lot_dest_transf = inventory_lot_line_obj.lot_id.id
                    location_id = location_inventory_id
                    location_dest_id = inventory_lot_line_obj.location_id.id
                move_transfer_id = self.pool.get('stock.move').create(cr, uid, {
                            'name':'INV:'+inv.name,
                            'date_planned': time.strftime('%Y-%m-%d %H:%M:%S'),
                            'picking_id':picking_transfer_id,
                            'product_id': inventory_lot_line_obj.product_id.id,
                            'product_qty': qty_move_transfer,
                            'product_uom': inventory_lot_line_obj.product_uom.id,
                            'hd_lot_org':hd_lot_org,
                            'hd_lot_dest_transf':hd_lot_dest_transf,
                            'location_id': location_id,
                            'location_dest_id': location_dest_id,
                            'state': 'draft',
                            })
                wf_service.trg_validate(uid, 'stock.picking', picking_transfer_id, 'button_confirm', cr)
                context['appel_prog'] = True
                picking_obj.force_assign(cr, uid,[picking_transfer_id], context=context)#HDcm 201_04_04 v604_01chem
                picking_obj.action_move(cr, uid, [picking_transfer_id])#HDcm 2012_02_20
                wf_service.trg_validate(uid, 'stock.picking', picking_transfer_id, 'button_done', cr)
                
                #HDcm 2012_04_12 604_02chem - faire l'historique
                #print "history 2 HD - parent ", move_inv_id, " - child ", move_transfer_id
                cr.execute('insert into stock_move_history_ids (parent_id,child_id) values (%s,%s)', (move_inv_id, move_transfer_id))
                
                move_ids.append(move_transfer_id)
            self.write(cr, uid, [inv.id], {'state': 'done', 'date_done': time.strftime('%Y-%m-%d %H:%M:%S'), 'move_ids': [(6, 0, move_ids)]})
        #print "/***action_done(stock_inventory_lot)"
        return True

    def unlink(self, cr, uid, ids, context={}):
        #Allows to delete 'stock inventory lot' in draft state
        for rec in self.browse(cr, uid, ids, context=context):
            if rec.state not in ['draft']:
                raise osv.except_osv(_('Invalid action !'), _('Cannot delete an Inventory Lot which is %s !')%(rec.state,))
            else:
                # suppression des inventory_lot_line correspondant a ce 'stock inventory lot'
                inventory_lot_line_ids = [x.id for x in rec.inventory_lot_line_id]
                self.pool.get('stock.inventory.lot.line').unlink(cr, uid, inventory_lot_line_ids, context=context)
        return super(stock_inventory_lot, self).unlink(cr, uid, ids, context=context)

    def action_cancel(self, cr, uid, ids, context={}):
        for inv in self.browse(cr, uid, ids):
            self.pool.get('stock.move').action_cancel(cr, uid, [x.id for x in inv.move_ids], context)
            self.write(cr, uid, [inv.id], {'state': 'draft'})
        return True

stock_inventory_lot()

class stock_inventory_lot_line(osv.osv):
    _name = "stock.inventory.lot.line"
    _description = "Inventory lot line"
    _columns = {
        'inventory_id': fields.many2one('stock.inventory.lot', 'Inventory Lot', ondelete='cascade', select=True),
        'lot_id': fields.many2one('stock.production.lot', 'Lot', required=True),
        'location_id': fields.many2one('stock.location', 'Location'),
        'product_id': fields.many2one('product.product', 'Product'),
        'product_uom': fields.many2one('product.uom', 'Product UOM'),
        'product_qty': fields.float('Qty inventoried', digits_compute=dp.get_precision('Qty Module Process'), required=True),
        'product_qty_before_inv': fields.float('Qty before inventory', digits_compute=dp.get_precision('Qty Module Process')),#hdcm 2013_02_19
        'state_lot': fields.related('lot_id', 'state', type='char', size=64, relation="stock.production.lot", string='State lot', store=True)#hdcm 2013_02_19
        
    }
    _sql_constraints = [
        ('inventory_lot_uniq', 'unique (inventory_id, lot_id)', 'The lot must be unique in this list !')
                        ]

    def on_change_lot_id(self, cr, uid, ids, inventory_id, lot_id):
        #print "***on_change_lot_id(stock_inventory_lot_line) - ids ",ids," - inventory_id ",inventory_id," - lot_id ",lot_id
        res = {'value': {}, 'warning': {}}
        if not lot_id:
            res['value'] = {'location_id': 0,'product_id': 0,'product_uom': 0, 'product_qty': 0, 'product_qty_before_inv': 0}
            return res
        # verifier si cette ligne n'est pas presente dans la liste
        if (len(self.pool.get('stock.inventory.lot.line').search(cr, uid, [('inventory_id','=',inventory_id),('lot_id','=', lot_id)]))==0):
            lot = self.pool.get('stock.production.lot').browse(cr, uid, lot_id)
            res['value'] = {'location_id': lot.hd_location_id.id,'product_id': lot.product_id.id,'product_uom': lot.hd_product_uom_id.id, 
                      'product_qty': lot.hd_qty_real, 'product_qty_before_inv': lot.hd_qty_real}
        else:
            res['value'] = {'lot_id': 0,'location_id': 0,'product_id': 0,'product_uom': 0, 'product_qty': 0, 'product_qty_before_inv': 0}
            res['warning'] = {'title':_('Information'), 'message':_('This lot is already in the list !')}        
        #print "/***on_change_lot_id(stock_inventory_lot_line) - res ",res
        return res

    def on_change_impossible(self, cr, uid, ids, lot_id):
        #print "***on_change_uom_id(stock_inventory_lot_line) - ids ",ids
        result = {}
        warning = {}
        if lot_id:
            lot = self.pool.get('stock.production.lot').browse(cr, uid, lot_id)
            result['product_uom'] = lot.hd_product_uom_id.id
            result['location_id'] = lot.hd_location_id.id
            result['product_id'] = lot.product_id.id
            result['product_qty_before_inv'] = lot.hd_qty_real
            warning = {'title':_('Information'), 'message':_('You can not edit this field')}
        #print "/***on_change_uom_id(stock_inventory_lot_line) - result ",result
        return {'value':result, 'warning':warning}

    def on_change_product_qty(self, cr, uid, ids, product_qty, lot_id):
        #print " ***on_change_product_qty(stock_inventory_lot_line) - ids ", ids, " - product_qty ", product_qty, " - lot_id ", lot_id
        res = {'value': {}, 'warning': {}}
        if (not lot_id) and (product_qty == 0):
            return res
        qty_return = 0.0
        if lot_id:
            lot = self.pool.get('stock.production.lot').browse(cr, uid, lot_id)
            qty_return = lot.hd_qty_real
        if (not product_qty) or (product_qty <= 0.0):
            res['value'] = {'product_qty': qty_return}
            res['warning'] = {'title':_('Information'), 'message':_('The quantity entered must be positive !')}
        #print "/***on_change_product_qty(stock_inventory_lot_line) - res ", res
        return res

    def unlink(self, cr, uid, ids, context={}):
        #ne pas autoriser unlink si le lot est issu d'un lot inventory
        for rec in self.browse(cr, uid, ids, context=context):
            move_id = self.pool.get('stock.move').search(cr, uid, [('hd_prodlot_id', '=', rec.lot_id.id)])[0]
            if self.pool.get('stock.move').browse(cr, uid, move_id).hd_lot_org.hd_type == 'inv':
                raise osv.except_osv(_('Invalid action !'), _('Cannot delete an Inventory Lot from lot inventory %s !') \
                                     %(self.pool.get('stock.move').browse(cr, uid, move_id).hd_lot_org.name,))
        return super(stock_inventory_lot_line, self).unlink(cr, uid, ids, context=context)

stock_inventory_lot_line()

class stock_picking(osv.osv):
    _name = "stock.picking"
    _inherit = "stock.picking"

    def create(self, cr, user, vals, context=None):
        #print " ***create(stock_picking) - vals ",vals, " - context ", context
        if ('name' not in vals) or (vals.get('name')=='/'):
            seq_obj_name =  self._name
            vals['name'] = self.pool.get('ir.sequence').get(cr, user, seq_obj_name)
        #hdcm 2009_09_8
        elif vals['type'] == 'split':
            #print "cas split"
            vals['name'] = self.pool.get('ir.sequence').get(cr, user, 'stock.picking.split')            
        elif vals['type'] == 'transfer':
            vals['name'] = self.pool.get('ir.sequence').get(cr, user, 'stock.picking.transfer')
        elif vals['type'] == 'displacement':
            vals['name'] = self.pool.get('ir.sequence').get(cr, user, 'stock.picking.displacement')
            #/hdcm 2009_09_8
            vals['auto_picking'] = True #hdcm 2010_02_01
        #print "vals ", vals
        new_id = super(stock_picking, self).create(cr, user, vals, context)
        #print "/***create(stock_picking) - new_id ",new_id
        return new_id

    _columns = {
        'type': fields.selection([('out','Sending Goods'),('in','Getting Goods'),('internal','Internal'),('split','Splitting'),('transfer','Transfer'),('displacement','Displacement')], 'Shipping Type', required=True, select=True),
        'inventory_lot_id': fields.many2one('stock.inventory.lot', 'Inventory Lots', ondelete='set null', readonly=True),
    }

    def unlink(self, cr, uid, ids, context=None):
        #print " ***unlink(stock_picking) - ids ",ids, " - context ", context
        move_obj = self.pool.get('stock.move')
        if context is None:
            context = {}
        for pick in self.browse(cr, uid, ids, context=context):
            '''hdcm v7.0 if pick.state in ['done','cancel']:
                raise osv.except_osv(_('Error!'), _('You cannot remove the picking which is in %s state!')%(pick.state,))'''
            #hdcm 30130702
            if pick.state not in ['draft']:
                if pick.type == 'split':
                    raise osv.except_osv(_('Invalid action !'), _('Cannot delete a Splitting Lot which is %s !')%(pick.state,))
                elif pick.type == 'transfer':
                    raise osv.except_osv(_('Invalid action !'), _('Cannot delete a Transfer Lot which is %s !')%(pick.state,))
                else:
                    raise osv.except_osv(_('Invalid action !'), _('Cannot delete a Picking which is %s !')%(pick.state,))
            #/hdcm 30130702
            else:
                ids2 = [move.id for move in pick.move_lines]
                ctx = context.copy()
                ctx.update({'call_unlink':True})
                if pick.state != 'draft':
                    #Cancelling the move in order to affect Virtual stock of product
                    move_obj.action_cancel(cr, uid, ids2, ctx)
                #Removing the move
                move_obj.unlink(cr, uid, ids2, ctx)
        #print "/***unlink(stock_picking) - ids ",ids, " - context ", context
        return super(stock_picking, self).unlink(cr, uid, ids, context=context)

    def copy(self, cr, uid, id, default=None, context=None):
        #print " ***copy(stock_picking) - default ",default, " - context ", context
        if default is None: default = {}
        default = default.copy()
        picking_obj = self.browse(cr, uid, id, context=context)
        #HDcm 2012_05_07 604803chem
        if picking_obj.type in ('split', 'transfer', 'displacement'):
            raise osv.except_osv(_('Invalid action !'), _('Cannot copy a Picking which is type \'%s\' !')%(picking_obj.type,))
        #/HDcm 2012_05_07 604803chem
        move_obj = self.pool.get('stock.move')
        if ('name' not in default) or (picking_obj.name == '/'):
            seq_obj_name = 'stock.picking.' + picking_obj.type
            default['name'] = self.pool.get('ir.sequence').get(cr, uid, seq_obj_name)
            default['origin'] = ''
            default['backorder_id'] = False
        if 'invoice_state' not in default and picking_obj.invoice_state == 'invoiced':
            default['invoice_state'] = '2binvoiced'
        res = super(stock_picking, self).copy(cr, uid, id, default, context)
        if res:
            picking_obj = self.browse(cr, uid, res, context=context)
            for move in picking_obj.move_lines:
                #HDcm 2013_07_02 OE move_obj.write(cr, uid, [move.id], {'tracking_id': False, 'prodlot_id': False, 'move_history_ids2': [(6, 0, [])], 'move_history_ids': [(6, 0, [])]})
                move_obj.write(cr, uid, [move.id], {'tracking_id': False, 'prodlot_id': False, 'move_history_ids2': [(6, 0, [])], 
                                                    'move_history_ids': [(6, 0, [])],
                                                    'hd_prodlot_id':False})
        #print "/***copy(stock_picking) - res ",res
        return res

    def action_confirm(self, cr, uid, ids, context=None):
        """ Confirms picking.
        @return: True
        """
        #print " ***action_confirm (stock.picking) - ids : ", ids," - context ", context
        #print time.clock()
        if context is None: context = {}
        context['prog'] = 'action_confirm(stock_picking)'#HDcm 2012_03_15 v603_02chem
        move_obj = self.pool.get('stock.move')
        pickings = self.browse(cr, uid, ids, context=context)
        for picking in self.browse(cr, uid, ids):#hdcm 2009_08_20
            if picking.type == 'transfer' and not picking.inventory_lot_id.id:#HDcm 2012_02_21 controle des informations de tranfert(sauf lors d'un inventaire), au cas où les infos aient changé depuis sa création
                for line in picking.move_lines:
                    context['appel_prog'] = True#HDcm 2012_02_17
                    #hdcm 2013_08_13 ctl1 = move_obj.transfer_lots_id_change(cr, uid, [], line.hd_lot_org.id, line.product_id.id, line.hd_lot_dest_transf.id, line.product_qty, context=context)
                    ctl1 = move_obj.transfer_lots_id_change(cr, uid, [], line.hd_lot_org.id, line.product_id.id, line.hd_lot_dest_transf.id, line.product_qty, line.product_uom.id, context=context)
                    if not ctl1:
                        raise osv.except_osv(_("The destination lot is no more compliant with the source lot!"), _("Modify  the transfer"))
                    #hdcm 2013_08_06 ctl2 = move_obj.transfer_qty_change(cr, uid, [], line.hd_lot_org.id, line.product_qty, context=context)
                    ctl2 = move_obj.transfer_qty_change(cr, uid, [], line.hd_lot_org.id, line.product_qty, line.product_uom.id, context=context)
                    if not ctl2:
                        raise osv.except_osv(_("The quantity of the destination lot is no more compliant with the source lot one !"), _("Modify the transfer"))
            elif picking.type == 'displacement': #hdcm 2009_09_24 recopier prodlot_id vers hd_prodlot_id
                for move in picking.move_lines:
                    move_obj.write(cr, uid, [move.id], {'hd_prodlot_id': move.prodlot_id.id})
            #hdcm 2013_12_17 rejeter les 'Internal Moves' qui integrent des produits geres par lot
            elif self.verif_pick_inetrnal_moves(cr, uid, picking.id):
                self.verif_pick_with_product_manage_by_lot(cr, uid, picking.id)
        
        #/hdcm 2009_08_20
        liste_picking_draft = []#HDcm 2012_03_29 v604_03chem
        for picking in self.browse(cr, uid, ids, context=context):
            if picking.state == 'draft': liste_picking_draft.append(picking.id)#HDcm 2012_03_29 v604_03chem
        self.write(cr, uid, ids, {'state': 'confirmed'})
        todo = []
        for picking in pickings:
            for r in picking.move_lines:
                if r.state == 'draft':
                    todo.append(r.id)
        todo = self.action_explode(cr, uid, todo, context)
        if len(todo):
            move_obj.action_confirm(cr, uid, todo, context=context)
        #hdcm 2009_08_20
        for picking in self.browse(cr, uid, ids):
            # dans le cas d'un picking de type 'splitting' ou 'transfer' , passer directement à l'étape suivante
            if (picking.type == 'split' and not picking.hd_production_id_spl) \
                or (picking.type == 'split' and picking.hd_production_id_spl and (picking.id in liste_picking_draft))\
                or picking.type == 'transfer'\
                or ((picking.type == 'internal') and picking.hd_production_id_spl):#HDcm 2012_03_29 v603_02chem
                #print "cas d'un picking de type splitting ou transfer"
                self.action_assign(cr, uid, [picking.id])
            #/hdcm 2009_08_20
            #hdcm 2009_08_25 dans le cas d'un picking de type 'displacement', faire le make picking automatiquement
            if picking.type == 'displacement':
                #print "cas d'un picking de type displacement"
                for move in picking.move_lines:
                    #HDcm 2012_04_12 506_06chem interdiction d'effectuer le deplacement d'un lot si une operation est en cours sur ce lot
                    if not (abs(move.prodlot_id.hd_qty_real - move.prodlot_id.hd_qty_virt) < 0.000000000001):
                        raise osv.except_osv(_('Invalid action !'), _('The displacement of a lot is not allowed if an operation is in progress on this lot'))
                    else:
                        #HDcm 2012_04_12 604_02chem  - 
                        #reactualiser la qte du move si la qte du move a change
                        if not (abs(move.product_qty - move.prodlot_id.hd_qty_virt) < 0.000000000001):
                            #recopier ce mouvement en écrivant la nouvelle qte
                            move_new_id = move_obj.copy(cr, uid, move.id, {'product_qty': move.prodlot_id.hd_qty_virt,
                                                                           'production_id': False,
                                                                            'move_history_ids': [], 
                                                                            'move_history_ids2': []
                                                                            })
                            #mettre le 1er move a l'etat 'cancel'
                            move_obj.write(cr, uid, [move.id], {'state': 'cancel'})
                            move_obj.action_done(cr, uid, [move_new_id], context=context)
                        #/HDcm 2012_04_12 604_02chem
                        else:
                            move_obj.action_done(cr, uid, [move.id], context=context)
                wf_service = netsvc.LocalService("workflow")
                wf_service.trg_validate(uid, 'stock.picking', picking.id, 'button_confirm', cr)
                wf_service.trg_validate(uid, 'stock.picking', picking.id, 'button_done', cr)
                #/hdcm 2009_08_25
        #print time.clock()
        #print "/***action_confirm (stock.picking)"
        return True

    def action_move(self, cr, uid, ids, context=None):
        """Process the Stock Moves of the Picking
        
        This method is called by the workflow by the activity "move".
        Normally that happens when the signal button_done is received (button 
        "Done" pressed on a Picking view). 
        @return: True
        """
        #print ' *** action_move(stock.picking) - ids',ids,' - context ',context 
        if context is None: context = {}
        context['prog'] = 'action_move(stock_picking)'#HDcm 2012_03_15 v603_02chem
        for pick in self.browse(cr, uid, ids, context=context):
            todo = []
            for move in pick.move_lines:
                if move.state == 'draft':
                    self.pool.get('stock.move').action_confirm(cr, uid, [move.id],
                        context=context)
                    todo.append(move.id)
                elif move.state in ('assigned','confirmed'):
                    todo.append(move.id)
            if len(todo):
                self.pool.get('stock.move').action_done(cr, uid, todo,
                        context=context)
        #print '/*** action_move(stock.picking) - context ', context
        return True

    # FIXME: needs refactoring, this code is partially duplicated in stock_move.do_partial()!
    def do_partial(self, cr, uid, ids, partial_datas, context=None):
        """ Makes partial picking and moves done.
        @param partial_datas : Dictionary containing details of partial picking
                          like partner_id, partner_id, delivery_date,
                          delivery moves with product_id, product_qty, uom
        @return: Dictionary of values
        """
        #print " ***do_partial(stock.picking) - ids ", ids, " - partial_datas ", partial_datas, " - context ",context
        if context is None:
            context = {}
        else:
            context = dict(context)
        res = {}
        move_obj = self.pool.get('stock.move')
        product_obj = self.pool.get('product.product')
        currency_obj = self.pool.get('res.currency')
        uom_obj = self.pool.get('product.uom')
        sequence_obj = self.pool.get('ir.sequence')
        lot_obj = self.pool.get('stock.production.lot')#HDcm 2012_02_23 v506_06chem
        wf_service = netsvc.LocalService("workflow")
        for pick in self.browse(cr, uid, ids, context=context):
            new_picking = None
            complete, too_many, too_few = [], [], []
            move_product_qty, prodlot_ids, product_avail, partial_qty, product_uoms = {}, {}, {}, {}, {}
            for move in pick.move_lines:
                if move.state in ('done', 'cancel'):
                    continue
                partial_data = partial_datas.get('move%s'%(move.id), {})
                product_qty = partial_data.get('product_qty',0.0)
                move_product_qty[move.id] = product_qty
                product_uom = partial_data.get('product_uom',False)
                product_price = partial_data.get('product_price',0.0)
                product_currency = partial_data.get('product_currency',False)
                prodlot_id = partial_data.get('prodlot_id')
                prodlot_ids[move.id] = prodlot_id
                product_uoms[move.id] = product_uom
                partial_qty[move.id] = uom_obj._compute_qty(cr, uid, product_uoms[move.id], product_qty, move.product_uom.id)
                
                
                '''v7.0 if move.product_qty == partial_qty[move.id]:
                    complete.append(move)
                elif move.product_qty > partial_qty[move.id]:
                    too_few.append(move)
                else:
                    too_many.append(move)'''
                #hdcm 2010_01_25 stopper le programme si un des moves correspond a un sale_line_id et a un lot d'origine non conforme
                if move.sale_line_id and move.hd_lot_org and (partial_data['product_qty'] > 0) and (move.hd_lot_org.state not in ('conform','alert_conform')):
                    #hdcm 2013_02_06 raise osv.except_osv(_('Invalid action !'), _('The lot %s is %s !')%(move.hd_lot_org.name,move.hd_lot_org.state))
                    context.update(CAS_SO_NORMAL)
                    context['move'] = move.id#hdcm 2013_02_04
                    context['cas_so_normal_stock_pickings'] = move.sale_line_id.id
                    context['cas_action_move_stock_picking'] = True#hdcm 2013_02_04
                    context['product_id'] = move.product_id.id
                    context['location_ids'] = [move.location_id.id]
                    verif_lots = self.pool.get('stock.production.lot').lot_control2(cr, uid, lots_requested = [move.hd_lot_org.id], context=context)
                    if verif_lots['infos'] != '': 
                        raise osv.except_osv(_('ATTENTION'), '%s%s\n%s\n%s' % (
                                                            _('PICKINGS : '), 
                                                            pick.name,
                                                            '===============',
                                                            verif_lots['infos']))
                #/hdcm 2010_01_25
                if abs(move.product_qty - product_qty) < 0.000000000001:
                    complete.append(move)
                elif move.product_qty > product_qty:
                    too_few.append(move)
                else:
                    too_many.append(move)

                # Average price computation
                if (pick.type == 'in') and (move.product_id.cost_method == 'average'):
                    product = product_obj.browse(cr, uid, move.product_id.id)
                    move_currency_id = move.company_id.currency_id.id
                    context['currency_id'] = move_currency_id
                    qty = uom_obj._compute_qty(cr, uid, product_uom, product_qty, product.uom_id.id)

                    if product.id in product_avail:
                        product_avail[product.id] += qty
                    else:
                        product_avail[product.id] = product.qty_available

                    if qty > 0:
                        new_price = currency_obj.compute(cr, uid, product_currency,
                                move_currency_id, product_price)
                        new_price = uom_obj._compute_price(cr, uid, product_uom, new_price,
                                product.uom_id.id)
                        if product.qty_available <= 0:
                            new_std_price = new_price
                        else:
                            # Get the standard price
                            amount_unit = product.price_get('standard_price', context=context)[product.id]
                            new_std_price = ((amount_unit * product_avail[product.id])\
                                + (new_price * qty))/(product_avail[product.id] + qty)
                        # Write the field according to price type field
                        product_obj.write(cr, uid, [product.id], {'standard_price': new_std_price})

                        # Record the values that were chosen in the wizard, so they can be
                        # used for inventory valuation if real-time valuation is enabled.
                        move_obj.write(cr, uid, [move.id],
                                {'price_unit': product_price,
                                 'price_currency_id': product_currency})


            for move in too_few:
                product_qty = move_product_qty[move.id]
                if not new_picking:
                    new_picking_name = pick.name
                    self.write(cr, uid, [pick.id], 
                               {'name': sequence_obj.get(cr, uid,
                                            'stock.picking.%s'%(pick.type)),
                               })
                    new_picking = self.copy(cr, uid, pick.id,
                            {
                                'name': new_picking_name,
                                'move_lines' : [],
                                'state':'draft',
                            })
                if product_qty != 0:
                    defaults = {
                            'product_qty' : product_qty,
                            'product_uos_qty': product_qty, #TODO: put correct uos_qty
                            'picking_id' : new_picking,
                            'state': 'assigned',
                            #hdcm 2013_09_24 v7.0 'move_dest_id': False,
                            'price_unit': move.price_unit,
                            'product_uom': product_uoms[move.id]
                    }
                    prodlot_id = prodlot_ids[move.id]
                    if prodlot_id:
                        defaults.update(prodlot_id=prodlot_id)
                    move_obj.copy(cr, uid, move.id, defaults)
                move_obj.write(cr, uid, [move.id],
                        {
                            'product_qty': move.product_qty - partial_qty[move.id],
                            'product_uos_qty': move.product_qty - partial_qty[move.id], #TODO: put correct uos_qty
                            'prodlot_id': False,
                            'tracking_id': False,
                        })

            if new_picking:
                move_obj.write(cr, uid, [c.id for c in complete], {'picking_id': new_picking})
            for move in complete:
                defaults = {'product_uom': product_uoms[move.id], 'product_qty': move_product_qty[move.id]}
                if prodlot_ids.get(move.id):
                    defaults.update({'prodlot_id': prodlot_ids[move.id]})
                move_obj.write(cr, uid, [move.id], defaults)
            for move in too_many:
                product_qty = move_product_qty[move.id]
                defaults = {
                    'product_qty' : product_qty,
                    'product_uos_qty': product_qty, #TODO: put correct uos_qty
                    'product_uom': product_uoms[move.id]
                }
                prodlot_id = prodlot_ids.get(move.id)
                if prodlot_ids.get(move.id):
                    defaults.update(prodlot_id=prodlot_id)
                if new_picking:
                    defaults.update(picking_id=new_picking)
                move_obj.write(cr, uid, [move.id], defaults)

            # At first we confirm the new picking (if necessary)
            if new_picking:
                wf_service.trg_validate(uid, 'stock.picking', new_picking, 'button_confirm', cr)
                # Then we finish the good picking
                self.write(cr, uid, [pick.id], {'backorder_id': new_picking})
                self.action_move(cr, uid, [new_picking], context=context)
                wf_service.trg_validate(uid, 'stock.picking', new_picking, 'button_done', cr)
                wf_service.trg_write(uid, 'stock.picking', pick.id, cr)
                delivered_pack_id = new_picking
                back_order_name = self.browse(cr, uid, delivered_pack_id, context=context).name
                self.message_post(cr, uid, ids, body=_("Back order <em>%s</em> has been <b>created</b>.") % (back_order_name), context=context)
            else:
                self.action_move(cr, uid, [pick.id], context=context)
                wf_service.trg_validate(uid, 'stock.picking', pick.id, 'button_done', cr)
                delivered_pack_id = pick.id

            delivered_pack = self.browse(cr, uid, delivered_pack_id, context=context)
            res[pick.id] = {'delivered_picking': delivered_pack.id or False}

                #hdcm 2010_01_28 dans le cas d'un SO, pour que le picking de la qte restant à livrer repasse dans l'etat 'confirmed', 
                # repasser les moves de ce picking en 'confirmed' si ce move correspond à un SO et enlever le lot d'origine
            for move in too_few:
                hd_lot_org_id = move.hd_lot_org.id # memoriser le lot d'origine avant son retrait
                if not move.sale_line_id:
                    break
                move_obj.write(cr, uid, [move.id],{'state' : 'confirmed','hd_lot_org':0})
                if hd_lot_org_id: lot_obj.write(cr, uid, [hd_lot_org_id],{})#hdcm 2010_01_28  pour forcer le recalcul du lot
                wf_service.trg_write(uid, 'stock.picking', pick.id, cr)#hdcm 2012_03_14 v603_02chem
                #/hdcm 2010_01_28
        #print "/***do_partial(stock.picking) - res ", res
        return res

    '''hdcm v7.0 def action_assign(self, cr, uid, ids, *args):'''
    def action_assign(self, cr, uid, ids, context=None, *args):
        """ Changes state of picking to available if all moves are confirmed.
        @return: True
        """
        #print " ***action_assign(stock.picking) - ids ", ids, " context ", context
        if context is None: context = {}
        wf_service = netsvc.LocalService("workflow")
        for pick in self.browse(cr, uid, ids):
            #hdcm 2013_12_23
            # dans le cas d'un picking 'Internal Moves', interdire les produits geres par lots
            if self.verif_pick_inetrnal_moves(cr, uid, pick.id):
                self.verif_pick_with_product_manage_by_lot(cr, uid, pick.id)
            #/hdcm 2013_12_23
            if pick.state == 'draft':
                wf_service.trg_validate(uid, 'stock.picking', pick.id, 'button_confirm', cr)
            move_ids = [x.id for x in pick.move_lines if x.state == 'confirmed']
            '''hdcm v7.0 if not move_ids:'''
            #hdcm v6.03
            if (not ((context.get('prog', False) == 'action_confirm(mrp_production)') \
                    or (context.get('prog', False) == 'create_returns(stock_return_picking_hd)')\
                    or (context.get('prog', False) == 'make_multi_splitting(make_multi_splitting_lot)')
                )) \
                                and not move_ids:
                #/hdcm v6.03
                raise osv.except_osv(_('Warning!'),_('Not enough stock, unable to reserve the products.'))
            self.pool.get('stock.move').action_assign(cr, uid, move_ids)
        #print "/***action_assign(stock.picking)"
        return True

    '''hdcm v7.0 def force_assign(self, cr, uid, ids, *args):'''
    def force_assign(self, cr, uid, ids, context=None, *args):
        """ Changes state of picking to available if moves are confirmed or waiting.
        @return: True
        """
        #print " ***force_assign(stock.picking) - ids ", ids, " context ", context
        if context is None: context = {}
        wf_service = netsvc.LocalService("workflow")
        for pick in self.browse(cr, uid, ids):
            #hdcm 2013_12_23
            # dans le cas d'un picking 'Internal Moves', interdire les produits geres par lots
            if self.verif_pick_inetrnal_moves(cr, uid, pick.id):
                self.verif_pick_with_product_manage_by_lot(cr, uid, pick.id)
            #/hdcm 2013_12_23
            move_ids = [x.id for x in pick.move_lines if x.state in ['confirmed','waiting']]
            '''hdcm 2012_04_04 v7.0self.pool.get('stock.move').force_assign(cr, uid, move_ids)'''
            self.pool.get('stock.move').force_assign(cr, uid, move_ids, context=context)#HDcm 2012_04_04 v604_02chem
            wf_service.trg_write(uid, 'stock.picking', pick.id, cr)
        #print "/***force_assign(stock.picking)"
        return True

    def draft_validate(self, cr, uid, ids, context=None):
        """ Validates picking directly from draft state.
        @return: True
        """
        #print "***draft_validate(stock.picking) - ids ", ids, " - context ", context
        wf_service = netsvc.LocalService("workflow")
        self.draft_force_assign(cr, uid, ids)
        for pick in self.browse(cr, uid, ids, context=context):
            move_ids = [x.id for x in pick.move_lines]
            self.pool.get('stock.move').force_assign(cr, uid, move_ids)
            wf_service.trg_write(uid, 'stock.picking', pick.id, cr)
        #print "/***draft_validate(stock.picking)"
        return self.action_process(
            cr, uid, ids, context=context)

    def verif_pick_with_product_manage_by_lot(self, cr, uid, picking_id):
        """ cette fonction verifie si ce picking contient un ou plusieurs produits geres par lot
        si oui, message d'erreur
        """
        #print "***verif_pick_with_product_manage_by_lot(stock.picking) - picking_id ", picking_id
        for move_line in self.browse(cr, uid, picking_id).move_lines:
            if move_line.product_id.hd_track_lot:
                raise osv.except_osv(_('Invalid action !'), _('This action is reserved for non lot managed products'))
        #print "/***verif_pick_with_product_manage_by_lot(stock.picking)"
        return True

    def verif_pick_inetrnal_moves(self, cr, uid, picking_id):
        """ cette fonction verifie si ce picking a ete cree manuellement par le menu 'Internal Moves'
        si oui, return : True
        """
        #print "***verif_pick_inetrnal_moves(stock.picking) - picking_id ", picking_id
        res = False
        pick = self.browse(cr, uid,picking_id)
        if pick.type == 'internal' and not(pick.purchase_id 
                                            or pick.sale_id 
                                            or pick.hd_production_id_int
                                            or pick.hd_production_id_spl
                                            or pick.inventory_lot_id):
            res = True
        #print "/***verif_pick_inetrnal_moves(stock.picking) - res ", res
        return res


stock_picking()

class stock_picking_in(osv.osv):
    _inherit = "stock.picking.in"

    def copy(self, cr, uid, id, default=None, context=None):
        #print " ***copy(stock_picking_in) - default ",default, " - context ", context
        #print "/***copy(stock_picking_in)"
        return self.pool.get('stock.picking').copy(cr, uid, id, context=context)

stock_picking_in()

class stock_move(osv.osv):
    _name = "stock.move"
    _inherit = "stock.move"

    _columns = {
        'prodlot_id': fields.many2one('stock.production.lot', 'Serial Number', states={'done': [('readonly', True)]}, select=True),#hdcm 2014_01_03 suppression du help
        'hd_prodlot_id' : fields.many2one('stock.production.lot', 'Process Lot'), # id du lot cree par le module hd_process
        'hd_lot_org': fields.many2one('stock.production.lot', 'Lot of origin'),
        'hd_lot_dest_transf': fields.many2one('stock.production.lot', 'Lot receiver transfer'),
        'product_qty': fields.float('Quantity', digits_compute=dp.get_precision('Qty Module Process'), required=True,states={'done': [('readonly', True)]}),
        'hd_remaining' : fields.float('Remaining', readonly=True),#HDcm 2009_06_10 mémorisation du reliquat
        'hd_track_lot': fields.related('product_id', 'hd_track_lot', type='boolean', string='Manage by lot', store=True),#hdcm 2013_02_05
        'type': fields.related('picking_id', 'type', type='selection', 
                               selection=[('out','Sending Goods'),('in','Getting Goods'),('internal','Internal'),
                                    ('split','Splitting'),('transfer','Transfer'),('displacement','Displacement')], string='Shipping Type'),#hdcm 2013_07_03
        'info_label': fields.char('Information', size=128),
    }

    _defaults = {
        'hd_remaining': lambda *a: 0.0,
    }

    def action_confirm(self, cr, uid, ids, context=None):
        """ Confirms stock move.
        @return: List of ids.
        """
        #print "***action_confirm(stock_move) - ids",ids," - context ",context
        moves = self.browse(cr, uid, ids, context=context)
        self.write(cr, uid, ids, {'state': 'confirmed'})
        self.create_chained_picking(cr, uid, moves, context)
        #print "/***action_confirm(stock_move)"
        return []

    def check_assign(self, cr, uid, ids, context=None):
        """ Checks the product type and accordingly writes the state.
        @return: No. of moves done
        """
        #print '*** check_assign(stock.move) - ids ', ids,' - context ',context
        #print time.clock()
        lot_obj = self.pool.get('stock.production.lot')#HDcm 2012_05_30 604_03chem
        location_obj = self.pool.get('stock.location')#hdcm 2012_12_21
        line_used_obj = self.pool.get('mrp.production.product.line.used')#hdcm 2013_02_22
        prod_obj = self.pool.get('mrp.production')
        done = []
        #HDcm 2012_02_13
        done2 = []
        #/HDcm 2012_02_13
        count = 0
        pickings = {}
        if context is None: context = {}
        for move in self.browse(cr, uid, ids, context=context):
            #HDcm 2012_03_27 603_02chem dans le cas d'une fab, ajouter dans le context l'id de la fab
            if move.picking_id and move.picking_id.hd_production_id_spl :
                #hdcm 2013_02_04!!! context['MO'] =  move.picking_id and move.picking_id.hd_production_id_spl.id
                context['move'] = move.id
                context['cas_check_assign_stock_move'] = True#hdcm 2013_02_01
                context['cas_mo_normal_stock_pickings'] = move.picking_id and move.picking_id.hd_production_id_spl.id#hdcm 2013_01_21
                #hdcm 2013_02_22 rechercher line_used ayant move_id_spl == move.id
                context['type_line'] = 'standard'
                line_used_ids = line_used_obj.search(cr, uid, [('move_id_spl', '=',move.id),('production_id_procur', '=',move.picking_id.hd_production_id_spl.id)])
                if len(line_used_ids):
                    type_line = line_used_obj.browse(cr, uid, line_used_ids[0],context=context).type_line
                    if type_line == 'reprocessing':
                        context['type_line'] = 'reprocessing'
            #/HDcm 2012_03_27 603_02chem
            #HDcm 2012_03_28 603_02chem dans le cas d'un SO, ajouter dans le context l'id du SO
            if move.sale_line_id :
                context['move'] = move.id
                context['cas_check_assign_stock_move'] = True#hdcm 2013_02_01
                #hdcm 2013_02_06 verifier si c'est un cas de retour SO (le picking correspondant est de type IN et ref a un sale_id)
                #hdcm 2013_08_19 v604 if move.picking_id and (move.picking_id.type == 'split') and (move.picking_id.name[-6:] == 'return'):
                if move.picking_id and (move.picking_id.type == 'split') and ('return' in move.picking_id.name):
                    context['cas_so_return'] = True
                else:
                    context['cas_so_normal_stock_pickings'] = move.sale_line_id.order_id.id#hdcm 2013_01_29
            #hdcm 2013_12_06 cas du Delivery Order manuel 
            if not (move.purchase_line_id or move.sale_line_id or move.production_id):
                context['move'] = move.id
                context['cas_do_manuel'] = True#hdcm 2013_02_01
            #/hdcm 2013_12_06 
            #print "context ", context
            #/HDcm 2012_03_27 603_02chem
            if move.product_id.type == 'consu' or move.location_id.usage == 'supplier':
                if move.state in ('confirmed', 'waiting'):
                    done.append(move.id)
                pickings[move.picking_id.id] = 1
                continue
            #HDcm 2012_02_13
            #hdcm 2009_09_25 pour les pickings de type 'displacement', la réservation ne se fait pas
            if move.picking_id.type == 'displacement':
                if move.state in ('confirmed', 'waiting'):
                    done.append(move.id)
                pickings[move.picking_id.id] = 1
                continue
            #/HDcm 2012_02_13
            if move.state in ('confirmed', 'waiting'):
                #hdcm 2009_08_28
                if move.product_id.hd_track_lot: # produit trace par lots - recherche de disponibilite de produit dans lots et recherche location correspondante
                    #print "produit trace par lots"
                    # vérifier si ce move est lié à une fab
                    # si move.move_dest_id appartient à la table mrp_production_move_ids (champ move_id)
                    production_id=0
                    if move.move_dest_id:
                        cr.execute('select production_id from mrp_production_move_ids where move_id=%s', (move.move_dest_id.id,))
                        res=cr.fetchone()
                        if res:
                            production_id=res[0]      
                    if move.hd_lot_org: ### recherche location lorsque lot precise dans enreg stock_move
                        #print "lot org precise dans le move"
                        lot_org = lot_obj.browse(cr, uid, move.hd_lot_org.id)
                        context['product_id'] = move.product_id.id
                        context['location_ids'] = [move.location_id.id]
                        #hdcm 2012_12_13 verifier que la quantite virtuelle est toujours suffisante et que letat du lot est toujours 'conform' ou 'alert_conform'
                        if context.get('cas_mo_normal_stock_pickings', False):
                            if context.get('type_line', False) == 'standard':
                                context.update(CAS_MO_NORMAL)
                            elif context.get('type_line', False) == 'reprocessing':
                                context.update(CAS_MO_NORMAL_REPROCESSING)
                            verif_lots = self.pool.get('stock.production.lot').lot_control2(cr, uid, lots_requested = [lot_org.id], context=context)
                            if verif_lots['infos'] != '': 
                                raise osv.except_osv(_('ATTENTION'), '%s%s\n%s\n%s' % (
                                                                    _('PICKINGS : '), 
                                                                    move.picking_id.name,
                                                                    '===============',
                                                                    verif_lots['infos']))
                        elif context.get('cas_so_normal_stock_pickings', False):
                            context.update(CAS_SO_NORMAL)
                            verif_lots = self.pool.get('stock.production.lot').lot_control2(cr, uid, lots_requested = [lot_org.id], context=context)
                            if verif_lots['infos'] != '': 
                                raise osv.except_osv(_('ATTENTION'), '%s%s\n%s\n%s' % (
                                                                    _('PICKINGS : '), 
                                                                    move.picking_id.name,
                                                                    '===============',
                                                                    verif_lots['infos']))
                        # la quantite virt du lot tient compte de la qte demandee dans le move, donc la qte virt ne doit pas etre negative
                        if lot_org.hd_qty_virt >= 0:
                            self.write(cr, uid, [move.id], {'location_id':move.hd_lot_org.hd_location_id.id})
                            done.append(move.id)
                            pickings[move.picking_id.id] = 1
                    else: ### recherche lots et locations lorsque lot non precise dans enreg stock_move
                        context['uom'] = move.product_uom.id
                        qty_ok = False
                        #hdcm 2012_12_21res = self.pool.get('stock.production.lot')._lot_reserve(cr, uid, [move.location_id.id], move.product_id.id, move.product_qty, context=context)
                        res = lot_obj._lot_reserve(cr, uid, [move.location_id.id], move.product_id.id, move.product_qty, context=context)
                        #print "res ",res
                        if len(res):
                            #HDcm 2012_05_15 604_03chem
                            if (len(res) > 1) and (not move.product_id.hd_fragmentation):
                                # si le nbre de lots trouvés est > 1 et que le fractionnemenr n'est pas autoriser sortir
                                #print "The fragmentation is not allowed for %s" % (move.product_id.name,)
                                continue
                            #/HDcm 2012_05_15 604_03chem
                            #HDcm 2012_05_04 604_03chem verifier si la qte est couverte
                            res_trav = res[:]
                            qty_available = 0.0
                            while len(res_trav): qty_available += res_trav.pop(0)[2]
                            if abs(qty_available - move.product_qty) < 0.000000000001: qty_ok = True
                            #print "qty_ok ", qty_ok
                            # si ce n'est pas le cas d'une quantite incomplte et d'un MO
                            #print "context.get('MO', False) ", context.get('MO', False)
                            if not (not qty_ok and context.get('cas_mo_normal_stock_pickings', False)):
                                #HDcm 2012_05_04 604_03chem 
                                done.append(move.id)
                                pickings[move.picking_id.id] = 1
                                r = res.pop(0)
                                self.write(cr, uid, [move.id], {'location_id':r[1], 'product_qty':r[2], 'hd_lot_org':r[0]})
                                if production_id: # si ce move est lié à une fab alors écrire la qté du move (picking) dans le move (OF)
                                    self.write(cr, uid, [ move.move_dest_id.id], {'product_qty':r[2]})
                                while res:
                                    r = res.pop(0)
                                    move_id = self.copy(cr, uid, move.id, {'product_qty':r[2], 
                                                                           'location_id':r[1], 
                                                                           'hd_lot_org':r[0], 
                                                                           'move_history_ids': [], 
                                                                           'move_history_ids2': []
                                                                           })#HDcm 2012_03_26 v603_02chem
                                    done.append(move_id)
                                    if production_id: # traiter les nouvelles lignes de move attachées à la production
                                        move_prem_fab_obj=self.browse(cr, uid, move.move_dest_id.id)
                                        # recopier le move et reprendre les champs du 1er move OF
                                        move_new_fab_id = self.copy(cr, uid, move_id, {
                                        'product_qty':r[2],
                                        'location_id':move_prem_fab_obj.location_id.id,
                                        'move_dest_id':move_prem_fab_obj.move_dest_id.id,
                                        'location_dest_id':move_prem_fab_obj.location_dest_id.id,
                                        'picking_id':False,
                                        'hd_lot_org':False,
                                        'state':'waiting',
                                        'move_history_ids': [], 
                                        'move_history_ids2': []
                                        })#HDcm 2012_03_26 v603_02chem
                                        # modifier move_dest_id du move_new_pick_obj
                                        self.write(cr, uid, [move_id], {'move_dest_id':move_new_fab_id})
                                        # HDcm 2009_06_10 créer 1 enr dans mrp_production_move_ids
                                        cr.execute('insert into mrp_production_move_ids (production_id,move_id) values (%s,%s)', (production_id, move_new_fab_id))
                                #HDcm 2012_05_04 604_03chem cree un move sans lot d'origine si la qte n'est pas atteinte
                                if not qty_ok:
                                    move_sans_lot_id = self.copy(cr, uid, move.id, {'product_qty':move.product_qty - qty_available, 
                                                                           'location_id':move.location_id.id, 
                                                                           'hd_lot_org':False, 
                                                                           'state':'confirmed',
                                                                           'move_history_ids': [], 
                                                                           'move_history_ids2': []
                                                                           })#HDcm 2012_03_26 v603_02chem
                    #print "done ",done
                    while done: # prendre le dernier move de la liste et le passer en 'assigned' puis maj lot
                        #print "done - ",done," done[-1] ",done[-1]
                        self.write(cr, uid, done[-1], {'state':'assigned'})
                        move_obj=self.browse(cr, uid, done[-1])
                        del done[-1] # enlever ce move de la liste apres le traitement
                        done2.append(move_obj.id)
                    #print "done2 ",done2
                    #print "done",done
                # else: # produit non trace par lots - recherche de disponibilite de produit dans locations
                else:
                    #/hdcm 2009_08_28
                    #print "produit non trace par lots"
                    # Important: we must pass lock=True to _product_reserve() to avoid race conditions and double reservations
                    res = self.pool.get('stock.location')._product_reserve(cr, uid, [move.location_id.id], move.product_id.id, move.product_qty, {'uom': move.product_uom.id}, lock=True)
                    if res:
                        #_product_available_test depends on the next status for correct functioning
                        #the test does not work correctly if the same product occurs multiple times
                        #in the same order. This is e.g. the case when using the button 'split in two' of
                        #the stock outgoing form
                        self.write(cr, uid, [move.id], {'state':'assigned'})
                        done.append(move.id)
                        pickings[move.picking_id.id] = 1
                        r = res.pop(0)
                        product_uos_qty = self.pool.get('stock.move').onchange_quantity(cr, uid, ids, move.product_id.id, r[0], move.product_id.uom_id.id, move.product_id.uos_id.id)['value']['product_uos_qty']
                        cr.execute('update stock_move set location_id=%s, product_qty=%s, product_uos_qty=%s where id=%s', (r[1], r[0],product_uos_qty, move.id))
                        while res:
                            r = res.pop(0)
                            move_id = self.copy(cr, uid, move.id, {'product_uos_qty': product_uos_qty, 'product_qty': r[0], 'location_id': r[1]})
                            done.append(move_id)
        '''hdcm v7.0 if done:
            count += len(done)
            self.write(cr, uid, done, {'state': 'assigned'})'''
        #HDcm 2012_02_13
        if done or done2:
            count += len(done)
            count += len(done2)
            #print "moves assigned en fin de prog ",done
            self.write(cr, uid, done, {'state':'assigned'})
            #/HDcm 2012_02_13
        if count:
            #print "pickings ", pickings
            for pick_id in pickings:
                wf_service = netsvc.LocalService("workflow")
                wf_service.trg_write(uid, 'stock.picking', pick_id, cr)
        #print time.clock()
        #print '/*** check_assign(stock.move) - count %s - context %s '%(count, context)
        return count

    #HDcm 2012_05_10 604_03chem
    def cancel_assign(self, cr, uid, ids, context=None):
        """ Changes the state to confirmed.
        @return: True
        """
        #print ' *** cancel_assign(stock.move) ids ',ids,' - context ',context
        if context is None: context = {}
        line_used_obj = self.pool.get('mrp.production.product.line.used')#hdcm 2013_02_22
        for move in self.browse(cr, uid, ids):
            #print 'move.id ', move.id
            if move.state == 'assigned' and move.product_id.hd_track_lot :
                # chercher les move qui ont le meme sale_line_id
                picking_id = move.picking_id.id
                #hdcm 2013_09_20 text_note = move.picking_id.origin + " - Cancel availability"
                text_note = (move.picking_id.origin or '') + " - Cancel availability"
                qty_tot = 0.0
                liste_move = []
                if move.sale_line_id:
                    # cas d'un delivery order
                    liste_move = self.search(cr, uid, [('sale_line_id', '=', move.sale_line_id.id), ('state', 'in', ('assigned', 'confirmed'))])
                elif move.picking_id and move.picking_id.hd_production_id_spl:
                    # c&as d'un MO
                    # verifier le type de ligne 'standard' ou 'reprocessing'
                    # hdcm 2014_01_06 line_used_id = line_used_obj.search(cr, uid, [('move_id_spl', '=',move.id),('production_id_procur', '=',move.picking_id.hd_production_id_spl.id)])[0]
                    if len(line_used_obj.search(cr, uid, [('move_id_spl', '=',move.id),('production_id_procur', '=',move.picking_id.hd_production_id_spl.id)])) :
                        line_used_id = line_used_obj.search(cr, uid, [('move_id_spl', '=',move.id),('production_id_procur', '=',move.picking_id.hd_production_id_spl.id)])[0]
                        type_line = line_used_obj.browse(cr, uid, line_used_id,context=context).type_line
                        if type_line == 'reprocessing':# le cancel availibility n'est pas autorise pour des moves correspondant à des lignes fab de type 'reprocessing'
                            raise osv.except_osv(_('Invalid action !'), _('This action is not allowed for stock movements corresponding to production lines of type \'Reprocessing\'.'))
                    liste_move = [x.id for x in move.picking_id.move_lines if x.state in ('assigned', 'confirmed')]
                elif move.picking_id and (move.picking_id.type == 'out') and not(move.sale_line_id 
                                                                                 or move.purchase_line_id
                                                                                 or move.production_id):
                    #print "cas d'un 'delivery order' manuel"
                    liste_move = self.search(cr, uid, [('picking_id', '=', move.picking_id.id), ('state', 'in', ('assigned', 'confirmed')),#hdcm 2014_01_03
                                                       ('product_id','=',move.product_id.id)])
                    
                #print "liste_move ", liste_move
                for m in self.browse(cr,uid, liste_move):
                    qty_tot += m.product_qty
                    self.write(cr, uid, [m.id], {'state': 'cancel', 'hd_lot_org': False, 'product_qty': 0.0, 'picking_id': False, 'note': text_note})
                self.write(cr, uid, [move.id], {'product_qty': qty_tot, 'state': 'confirmed', 'picking_id': picking_id, 'note': False})
            elif move.state == 'assigned':
                self.write(cr, uid, [move.id], {'state': 'confirmed'})
        #HDcm 2012_05_10 604_v7.0 self.write(cr, uid, ids, {'state': 'confirmed'})

        # fix for bug lp:707031
        # called write of related picking because changing move availability does
        # not trigger workflow of picking in order to change the state of picking
        wf_service = netsvc.LocalService('workflow')
        for move in self.browse(cr, uid, ids, context):
            if move.picking_id:
                wf_service.trg_write(uid, 'stock.picking', move.picking_id.id, cr)
        #print '/*** cancel_assign(stock.move)'
        return True

    def prepa_action_done(self, cr, uid, ids, context=None):
        #print '*** prepa_action_done(stock.move) ids ',ids,' - context ',context
        if context is None: context = {}
        wf_service = netsvc.LocalService("workflow")
        product_uom_obj = self.pool.get('product.uom')#hdcm 2012_12_21
        lot_obj = self.pool.get('stock.production.lot')
        sale_line_obj = self.pool.get('sale.order.line')
        prod_line_used_obj = self.pool.get('mrp.production.product.line.used')
        seq_obj = self.pool.get('ir.sequence')
        
        todo = []
        ref_lot = ""#HDcm 2012_05_09 604_03chem
        product_return_case = False#HDcm 2012_04_26 604_03chem
        lots_with_lotorg = []#hdcm 2013_10_02 hd_labo - liste des lots crees ayant un lot d'origine - 
        hd_type=""
        hd_doc_org_id=0
        hd_doc_inter_id=0#hdcm 2013_02_27
        partner_id = False #hdcm2013_09_24 v7.0
        
        #hdcm 2013_12_06 si 1 move est 'confirmed' (waiting availability), alors effectuer la recherche de disponibilite
        for move in self.browse(cr, uid, ids, context=context):
            if move.state == 'confirmed' and move.product_id.hd_track_lot:
                self.check_assign(cr, uid, [move.id],context=context)
        #/hdcm 2013_12_06 
        for move in self.browse(cr, uid, ids, context=context):
            #print "move.state ", move.state
            #hdcm 2009_08_20  prise en compte des produits dont l'indicateur hd_track_lot est activé 
            # lors des opérations de transfert, il n'y a pas de lot de créé
            if move.state=='assigned' and move.product_id.hd_track_lot and not ((move.picking_id.type == 'transfer') or (move.picking_id.type == 'displacement')):
                #hdcm 2013_12_04 todo_lot_ind.append(move.id)
                todo.append(move.id)#hdcm 2013_11_25
                # creation d'un lot correspondant a ce move, 
                # s'il n'y a pas deja eu un lot de creer pour ce move
                if not move.hd_prodlot_id:
                    #!!! if pick.type=='in':
                    if move.picking_id.type=='in':
                        if move.location_id.id == move.product_id.property_stock_inventory.id:# cas d'un picking de type 'in' pour la création d'un lot inventaire
                            #print "INV"
                            deb='INV'
                            hd_type='inv'                
                            hd_doc_org_id= move.picking_id.inventory_lot_id.id # retrouver le PO
                            '''hdcm 2013_09_24 v604 else:# cas d'un picking de type 'in' pour la création d'un lot achat
                            #print "IN"
                            deb='IN'
                            hd_type='in'
                            if move.picking_id.id:
                                hd_doc_org_id= pick.purchase_id.id # retrouver le PO'''
                        elif move.move_dest_id.id and move.move_dest_id.sale_line_id.id:# cas d'un picking de type sale_buy pour la creation d'un lot d'achat
                            #print "IN"
                            deb='IN'
                            hd_type='sale_buy'
                            # retrouver le SO et le partner
                            sale_line = sale_line_obj.browse(cr,uid,move.move_dest_id.sale_line_id.id)
                            hd_doc_org_id = sale_line.order_id.id
                            partner_id = sale_line.order_id.partner_id.id
                        else:# cas d'un picking de type 'in' pour la création d'un lot achat
                            #print "IN"
                            deb='IN'
                            hd_type='in'
                            if move.picking_id.id:
                                hd_doc_org_id= move.picking_id.purchase_id.id # retrouver le PO
                    elif move.picking_id.type=='out':
                        #print "OUT"
                        deb='OUT'
                        hd_type ='sale_stock'
                        if move.hd_lot_org and (move.hd_lot_org.hd_type == 'sale_fab'): hd_type = 'sale_fab'#hdcm 2013_02_27
                        if move.hd_lot_org and (move.hd_lot_org.hd_type == 'sale_buy'): hd_type = 'sale_buy'#hdcm 2013_09_24
                        if move.sale_line_id.id: # retrouver le SO et le partner
                            hd_doc_org_id = sale_line_obj.browse(cr,uid,move.sale_line_id.id).order_id.id
                            hd_doc_inter_id = move.hd_lot_org.hd_doc_inter_id#hdcm 2013_02_27
                            partner_id = sale_line_obj.browse(cr,uid,move.sale_line_id.id).order_id.partner_id.id
                        #HDcm 2012_05_09 604_03chem - recopier la ref du lot si elle existe
                        if move.hd_lot_org.id and move.hd_lot_org.hd_ref_lot and (len(move.hd_lot_org.hd_ref_lot) > 0):
                            ref_lot =  move.hd_lot_org.hd_ref_lot
                    elif move.move_dest_id:
                        #print "INT"
                        deb='INT'
                        hd_type='int'
                    else:
                        #print "SPL"
                        deb='SPL'
                        hd_type='split'
                        hd_doc_org_id=move.picking_id.id # retrouver l'id du splitting
                    #hdcm 2012_12_13 vérifier que les lot origine sont toujours disponibles (qte, etat et zone sous controle)
                    if (move.picking_id and move.picking_id.hd_production_id_spl) and move.hd_lot_org:
                        #print "controle du lot d'origine1"
                        
                        #hdcm 2013_02_22 rechercher line_used ayant move_id_spl == move.id
                        context['type_line'] = 'standard'
                        type_line = 'standard'#hdcm 2014_01_06
                        line_used_ids = prod_line_used_obj.search(cr, uid, [('move_id_spl', '=',move.id),('production_id_procur', '=',move.picking_id.hd_production_id_spl.id)])
                        if len(line_used_ids):
                            type_line = prod_line_used_obj.browse(cr, uid, line_used_ids[0],context=context).type_line
                            if type_line == 'reprocessing':
                                context['type_line'] = 'reprocessing'
                        if type_line == 'standard':
                            context.update(CAS_MO_NORMAL)
                        elif type_line == 'reprocessing':
                            context.update(CAS_MO_NORMAL_REPROCESSING)
                        lot_org = lot_obj.browse(cr, uid, move.hd_lot_org.id)
                        context['move'] = move.id#hdcm 2013_02_04
                        context['cas_mo_normal_stock_pickings'] = move.picking_id.hd_production_id_spl.id
                        context['cas_action_move_stock_picking'] = True#hdcm 2013_02_04
                        context['product_id'] = move.product_id.id
                        context['location_ids'] = [move.location_id.id]
                        verif_lots = self.pool.get('stock.production.lot').lot_control2(cr, uid, lots_requested = [lot_org.id], context=context)
                        if verif_lots['infos'] != '': 
                            raise osv.except_osv(_('ATTENTION'), '%s%s\n%s\n%s' % (
                                                                _('PICKINGS : '), 
                                                                move.picking_id.name,
                                                                '===============',
                                                                verif_lots['infos']))
                        '''hdcm 2013_08_19 v604 elif move.sale_line_id and move.hd_lot_org and \
                                        not (move.picking_id and (move.picking_id.type == 'split') and (move.picking_id.name[-6:] == 'return')):#hdcm 2013_02_06 les split correspondant a un retour SO ne necessite pas de controle'''
                    elif move.sale_line_id and move.hd_lot_org and \
                                        not (move.picking_id and (move.picking_id.type == 'split') and ('return' in move.picking_id.name)):#hdcm 2013_02_06 les split correspondant a un retour SO ne necessite pas de controle
                        #print "controle du lot d'origine2"
                        lot_org = lot_obj.browse(cr, uid, move.hd_lot_org.id)
                        context.update(CAS_SO_NORMAL)
                        context['move'] = move.id#hdcm 2013_02_04
                        context['cas_so_normal_stock_pickings'] = move.sale_line_id.id
                        context['cas_action_move_stock_picking'] = True#hdcm 2013_02_04
                        context['product_id'] = move.product_id.id
                        context['location_ids'] = [move.location_id.id]
                        verif_lots = self.pool.get('stock.production.lot').lot_control2(cr, uid, lots_requested = [lot_org.id], context=context)
                        if verif_lots['infos'] != '': 
                            raise osv.except_osv(_('ATTENTION'), '%s%s\n%s\n%s' % (
                                                                _('PICKINGS : '), 
                                                                move.picking_id.name,
                                                                '===============',
                                                                verif_lots['infos']))
                    #/hdcm 2012_12_13                            
                    # le paramètre context est utilisé pour l'initialisation des 4 champs date péremption
                    # par la fonction calc_date de la classe stock.production.lot    
                    #hdcm 2009_10_02 la qte inscrite dans le lot est en fonction de l'uom du produit
                    #print "hd_type ", hd_type
                    hd_prodlot_id = lot_obj.create(cr, uid, {
                        #hdcm 2013_09_14 v604 'name':'%s[%s][%s]' %(deb,move.product_id.code,time.strftime('%Y%m%d%H%M')),
                        #hdcm 2013_09_16 v604 'name': name_lot,
                        'name': 'temporary_name',
                        'hd_ref_lot': ref_lot,
                        'product_id':move.product_id.id,
                        'hd_product_uom_id':move.product_id.uom_id.id,
                        'hd_location_id':move.location_dest_id.id,
                        'hd_qty_org':product_uom_obj._compute_qty(cr, uid, move.product_uom.id, move.product_qty, to_uom_id=move.product_id.uom_id.id),
                        'hd_indtrack':True,
                        'hd_type':hd_type,
                        'hd_doc_org_id':hd_doc_org_id,
                        'hd_doc_inter_id':hd_doc_inter_id,#hdcm 2013_02_27
                        },context= {'product_id':move.product_id.id})
                    # compléter le name du lot avec son id
                    #hdcm 2013_0lot04 lot_new = lot_obj.browse(cr,uid,hd_prodlot_id)
                    #hdcm 2013_09_16 v604 lot_obj.write(cr,uid, [hd_prodlot_id],{'name':'%s:%07d'%(lot_new.name,hd_prodlot_id),})
                    lot_obj.write(cr,uid, [hd_prodlot_id],{'name': lot_obj.get_name_lot(cr, uid, deb, move.product_id, hd_prodlot_id)})
                    #hdcm 2009_08_20
                    # si ce lot est de hd_type int, renseigner production_id
                    if hd_type=='int':
                        #print "1.0"
                        # si ce lot int est lié à une fab,
                        if move.picking_id.hd_production_id_int.id:
                            #print "1.1.1"
                            # --- renseigner hd_production_id_int
                            # --- marqué ce lot comme étant réservé pour cet MO
                            lot_obj.write(cr,uid, [hd_prodlot_id],{
                            'hd_production_id_int':move.picking_id.hd_production_id_int.id,
                            'hd_reserv_production_id':move.picking_id.hd_production_id_int.id,
                            'hd_reserv':'interne'
                            })
                    #hdcm 2010_02_02 ecrire une reservation pour lot OUT
                    #hdcm 2013_09_24 v604 if hd_type == 'sale_stock':
                    if hd_type in ('sale_stock', 'sale_buy'):
                        #print "1.1"
                        # --- marqué ce lot comme étant réservé pour ce SO
                        lot_obj.write(cr,uid, [hd_prodlot_id],{
                        'hd_reserv_partner_id':partner_id,
                        'hd_reserv':'vente'
                        })
                    # si ce lot est de hd_type spl ou in, 
                    if (hd_type=='split'):
                        #print "1.2"
                        if move.picking_id.hd_production_id_spl :#hdcm 2009_12_02 si ce picking est lié à une fab,
                            # --- renseigner hd_production_id_spl
                            # --- marqué ce lot comme étant réservé pour cet MO
                            lot_obj.write(cr,uid, [hd_prodlot_id],{
                                'hd_production_id_spl':move.picking_id.hd_production_id_spl.id,
                                'hd_reserv_production_id':move.picking_id.hd_production_id_spl.id,
                                'hd_reserv':'interne'
                                })
                        # --- retrouver les mrp.production.product.line.used qui ont pour move_id_spl ce move et mettre state à 'procur_done'
                        mrp_production_product_line_used_id_list = prod_line_used_obj.search(cr, uid, [('move_id_spl', '=', move.id)])
                        for mrp_production_product_line_used_obj in prod_line_used_obj.browse(cr,uid,mrp_production_product_line_used_id_list):
                            if mrp_production_product_line_used_obj.state == 'procur_run':
                                #print "1.2.1"
                                prod_line_used_obj.write(cr, uid, [mrp_production_product_line_used_obj.id], {'state':'procur_done'})
                        #HDcm 2012_04_26 604_03chem - pour les picking de type split faisant ref a un PO ou à un SO, le lot cree doit etre passe en state 'to_control'
                        if move.picking_id.sale_id:
                            lot_obj.write(cr,uid, [hd_prodlot_id],{
                                'hd_note': '%s %s %s %s' % (_('WARNING this lot corresponds to a return customer\nSO :'), move.picking_id.sale_id.name,
                                                            _('\nPartner :'), move.picking_id.sale_id.partner_id.name
                                                            ),
                            })
                            product_return_case = True
                        if move.picking_id.purchase_id:
                            lot_obj.write(cr,uid, [hd_prodlot_id],{
                                'hd_note': '%s %s %s %s' % (_('WARNING this lot was returned to the supplier\nPO :'), move.picking_id.purchase_id.name,
                                                            _('\nPartner :'), move.picking_id.purchase_id.partner_id.name
                                                            ),
                            })
                            product_return_case = True
                        #HDcm 2012805809 604_03 chem - cas correspondant a un split d'origine MULTI-SPLIT - ecrire une ref de lot
                        if move.picking_id.origin and move.picking_id.origin[0:len("MULTI-SPLIT")] == "MULTI-SPLIT":
                            ref_lot = seq_obj.get(cr, uid, 'stock.picking.' + 'multi-split')
                            if ref_lot:
                                lot_obj.write(cr,uid, [hd_prodlot_id],{
                                    'hd_ref_lot':ref_lot + '[%s]'%(hd_prodlot_id,),})
                    # si cet achat a été fait pour une fab, renseigner hd_reserv et hd_reserv_production_id
                    if (hd_type=='in') and (move.move_dest_id.id):
                        # --- retrouver les mrp.production.product.line.used qui ont pour move_id_spl=move.move_dest_id.id 
                        # et mettre state à 'procur_done'
                        mrp_production_product_line_used_id_list = prod_line_used_obj.search(cr, uid, [('move_id_spl', '=', move.move_dest_id.id)])
                        #print "mrp_production_product_line_used_id_list : ",mrp_production_product_line_used_id_list
                        for mrp_production_product_line_used_obj in prod_line_used_obj.browse(cr,uid,mrp_production_product_line_used_id_list):
                            if mrp_production_product_line_used_obj.state == 'procur_run':
                                prod_line_used_obj.write(cr, uid, [mrp_production_product_line_used_obj.id], {'state':'procur_done'})
                            # --- renseigner hd_production_id_spl
                            # --- marqué ce lot comme étant réservé pour cet MO
                                lot_obj.write(cr,uid, [hd_prodlot_id],{
                                'hd_production_id_spl':mrp_production_product_line_used_obj.production_id_procur.id,
                                'hd_reserv_production_id':mrp_production_product_line_used_obj.production_id_procur.id,
                                'hd_reserv':'interne'
                                })
                    #/hdcm 2009_08_20
                    # si ce move fait réf à 1 lot d'origine, alors 
                    # ----- recopier les dates de péremption du lot d'origine vers le nouveau lot
                    # ----- recopier l'id du doc d'origine
                    # ----- écrire le lot d'origine
                    if move.hd_lot_org.id:
                        lots_with_lotorg.append(hd_prodlot_id)
                        
                        lot_org_obj=lot_obj.browse(cr,uid,move.hd_lot_org.id)
                        lot_obj.write(cr,uid, [hd_prodlot_id], {
                                    'life_date':lot_org_obj.life_date,
                                    'use_date':lot_org_obj.use_date,
                                    'removal_date':lot_org_obj.removal_date,
                                    'alert_date':lot_org_obj.alert_date,
                                    'hd_lot_org_id':move.hd_lot_org.id
                                                    })
                        # ----- et recopier le statut vers le nouveau lot
                        if move.hd_lot_org.state=='conform':
                            wf_service.trg_validate(uid, 'stock.production.lot', hd_prodlot_id, 'button_conform', cr)
                        elif move.hd_lot_org.state=='alert_conform':
                            wf_service.trg_validate(uid, 'stock.production.lot', hd_prodlot_id, 'button_conform', cr)
                            wf_service.trg_validate(uid, 'stock.production.lot', hd_prodlot_id, 'button_alert_conform', cr)
                        elif move.hd_lot_org.state=='out_of_date':
                            wf_service.trg_validate(uid, 'stock.production.lot', hd_prodlot_id, 'button_conform', cr)
                            wf_service.trg_validate(uid, 'stock.production.lot', hd_prodlot_id, 'button_alert_conform', cr)
                            wf_service.trg_validate(uid, 'stock.production.lot', hd_prodlot_id, 'button_out_of_date', cr)
                        elif move.hd_lot_org.state=='declassified':
                            wf_service.trg_validate(uid, 'stock.production.lot', hd_prodlot_id, 'button_declassified', cr)
                        elif move.hd_lot_org.state=='to_reprocess':
                            wf_service.trg_validate(uid, 'stock.production.lot', hd_prodlot_id, 'button_to_reprocess', cr)
                        elif move.hd_lot_org.state=='to_destroy':
                            wf_service.trg_validate(uid, 'stock.production.lot', hd_prodlot_id, 'button_to_destroy', cr)
                        #HDcm 2012_04_26 604_03chem - si on est dans le cas d'un retour client, forcer l'etat a 'to_control'
                        if product_return_case:
                            wf_service.trg_validate(uid, 'stock.production.lot', hd_prodlot_id, 'button_to_control', cr)
                    # ecrire la reference du lot dans le move
                    # pour test move_obj.write(cr,uid, [move.id], {'hd_prodlot_id':hd_prodlot_id})
                    self.write(cr,uid, [move.id], {'hd_prodlot_id':hd_prodlot_id, 'prodlot_id':hd_prodlot_id})
                else: # si un lot existe déjà récupérer le N° de ce lot
                    hd_prodlot_id=move.hd_prodlot_id.id
            if move.state=='assigned' and move.product_id.hd_track_lot and ((move.picking_id.type == 'transfer') or (move.picking_id.type == 'displacement')):
                #print "2"
                #hdcm 2013_12_04 todo_lot_ind.append(move.id)
                todo.append(move.id)#hdcm 2013_11_25
            #hdcm 2013_11_06 elif move.state=='assigned' and not move.product_id.hd_track_lot:
            elif move.state in ('confirmed', 'assigned') and not move.product_id.hd_track_lot:
                #print "3"
                #hdcm 2013_12_04 todo_lot_gen.append(move.id)  
                todo.append(move.id)#hdcm 2013_11_25
                #hdcm 2009_12_17 traitement pour les produits non gérés par lot
                # --- retrouver les mrp.production.product.line.used qui ont pour move_id_spl ce move et mettre state à 'procur_done'
                mrp_production_product_line_used_id_list = prod_line_used_obj.search(cr, uid, [('move_id_spl', '=', move.id)])
                #print "mrp_production_product_line_used_id_list : ",mrp_production_product_line_used_id_list
                for mrp_production_product_line_used_obj in prod_line_used_obj.browse(cr,uid,mrp_production_product_line_used_id_list):
                    if mrp_production_product_line_used_obj.state == 'procur_run':
                        #print "1.2.1"
                        prod_line_used_obj.write(cr, uid, [mrp_production_product_line_used_obj.id], {'state':'procur_done'})

        context['lots_with_lotorg'] = lots_with_lotorg#hdcm 2013_10_02 utilise par hd_labo
        
        #print '/*** prepa_action_done(stock.move) todo', todo
        return todo

#*******************************************************************
#      HDcm 
#      vérifier le lot d'origine pour retrouver le move d'origine et renseigner la table stock_move_history_ids
#      ainsi, pour un sale order, on obtient la traçabilité jusqu'au lot MP acheté
#*******************************************************************

    def action_done(self, cr, uid, ids, context=None):
        """ Makes the move done and if all moves are done, it will finish the picking.
        @return:
        """
        #print '*** action_done(stock.move) ids ',ids,' - context ',context
        #print time.clock()
        
        picking_ids = []
        move_ids = []
        wf_service = netsvc.LocalService("workflow")
        if context is None: context = {}
        #HDcm 2012_02_24 message d'erreur quand ids est False
        if len(ids) and not ids[0]:
            #print '/*** action_done(stock.move) False'
            return False
        #/HDcm 2012_02_24

        todo = []
        #HDcm 2012_02_02
        processed_lines = []
        #/HDcm 2012_02_02
        #hdcm 2013_12_06 for move in self.browse(cr, uid, ids, context=context):
        for move in self.browse(cr, uid, ids, context=context):
            '''#HDcm 2012_03_29 v603_02chem - pour empecher le lancement de cette fonction a partir du menu Products moves/Deliver products
            if (not context.get('prog', False)) and move.product_id.hd_track_lot:
                raise osv.except_osv(_('Invalid action !'), _('This action is reserved for non lot managed products'))
            #/HDcm 2012_03_29 v603_02chem '''
            if move.state=="draft":
                todo.append(move.id)
        if todo:
            #print "todo ", todo
            self.action_confirm(cr, uid, todo, context=context)
            todo = []

        todo_prepa_action_done = self.prepa_action_done(cr, uid, ids, context=context)#hdcm 2013_12_06
        #print "todo_prepa_action_done ", todo_prepa_action_done

        #hdcm 2013_11_06 for move in self.browse(cr, uid, ids, context=context):
        for move in self.browse(cr, uid, todo_prepa_action_done, context=context):
            if move.state in ['done','cancel']:
                continue
            move_ids.append(move.id)
            if move.picking_id:
                picking_ids.append(move.picking_id.id)
            #HDcm 2012_02_02
            history_a_faire=False # pour split et transfer
            history_a_faire_displacement=False # pour deplacement     
            if move.hd_lot_org.id and move.picking_id.id:
                # traitement pour splitting - vérifier que le picking est de type split
                #hdcm 2013_09_25 if move.picking_id.type=="split":
                if move.picking_id.type=="split" and not len(self.is_destination_move(cr, uid, move.id)):
                    #print "traitement pour splitting - vérifier que le picking est de type split"
                    history_a_faire=True # faire le traçage par rapport au move contenant le lot d'origine
                # traitement pour le transfert lot - vérifier que le picking est de type transfer
                #HDcm 2012_04_12 604_02chem - ne pas faire l'historique si le transfert est lie a un inventaire.
                #  dans ce cas l'historique est fait maintenant par action_done(stock.inventory.lot)
                if move.picking_id.type=="transfer" and not move.picking_id.inventory_lot_id:
                    #print "traitement pour le transfert lot - vérifier que le picking est de type transfer(sauf en cas d'inventaire)"
                    history_a_faire=True # faire le traçage par rapport au move contenant le lot d'origine
            #    traitement pour sale order from stock        
            #hdcm 2013_09_24 v604 if move.hd_lot_org.id and move.sale_line_id.id:
            if move.hd_lot_org.id and move.sale_line_id.id and not (move.sale_line_id.type == 'make_to_order'):
                #print "traitement pour sale order from stock"
                history_a_faire=True # faire le traçage par rapport au move contenant le lot d'origine
            #   traitement pour displacement
            if move.picking_id and move.picking_id.type=='displacement':
                #print "traitement pour displacement"
                # déplacer les infos prodlot_id, hd_prodlot_id et hd_lot_org dans ce move et les enlever du move d'origine
                move_org_obj=self.browse(cr,uid,min(self.search(cr, uid, [('hd_prodlot_id', '=', move.hd_prodlot_id.id), ('state', '!=', 'cancel')])))
                self.write(cr, uid, [move.id], {'prodlot_id':move_org_obj.prodlot_id.id, 
                                                        'hd_prodlot_id':move_org_obj.hd_prodlot_id.id, 
                                                        'hd_lot_org':move_org_obj.hd_lot_org.id,
                                                        'purchase_line_id':move_org_obj.purchase_line_id.id,
                                                        'sale_line_id':move_org_obj.sale_line_id.id,
                                                        'production_id':move_org_obj.production_id.id,})
                self.write(cr, uid, [move_org_obj.id], {'prodlot_id':False,'hd_prodlot_id':False,'hd_lot_org':False})
                # faire le traçage par rapport au move d'origine
                #print "history 1 HD - parent ", move_org_obj.id, " - child ", move.id
                cr.execute('insert into stock_move_history_ids (parent_id,child_id) values (%s,%s)', (move_org_obj.id, move.id))
            #    traitement pour sale order(ou procurement)  on order
            #HDcm 2012_03_15 v603_02chem if move.hd_prodlot_id.id and (not move.hd_lot_org.id) and move.production_id.id and move.move_dest_id.id and (move.state != 'done'):
            #print "context.get('prog', False) ", context.get('prog', False)
            if (not context.get('prog', False) == 'action_production_end') \
                and  move.hd_prodlot_id.id and (not move.hd_lot_org.id) and move.production_id.id and move.move_dest_id.id:
                #print "traitement pour sale order(ou procurement)  on order"
                # déplacer les infos hd_prodlot_id et lot_org dans l'autre move et les
                # enlever de celui-ci
                self.write(cr, uid, [move.move_dest_id.id], {'prodlot_id':move.prodlot_id.id, 'hd_prodlot_id':move.hd_prodlot_id.id, 'hd_lot_org':move.hd_lot_org.id})
                self.write(cr, uid, [move.id], {'prodlot_id':False,'hd_prodlot_id':False, 'hd_lot_org':False})
            #    traitement pour production order                
            if move.hd_lot_org.id and move.picking_id.id and move.move_dest_id.id:
                #print "traitement pour production order"
                history_a_faire=True # faire le traçage par rapport au move contenant le lot d'origine
                # déplacer les infos lot_id et lot_org dans l'autre move et les
                # enlever de celui-ci
                self.write(cr, uid, [move.move_dest_id.id], {'prodlot_id':move.prodlot_id.id, 'hd_prodlot_id':move.hd_prodlot_id.id, 'hd_lot_org':move.hd_lot_org.id})
                self.write(cr, uid, [move.id], {'prodlot_id':False,'hd_prodlot_id':False, 'hd_lot_org':False})
            #print "history_a_faire ", history_a_faire
            if history_a_faire:    
                # faire le traçage par rapport au move contenant le lot d'origine
                move_org = self.search(cr, uid, [('hd_prodlot_id', '=', move.hd_lot_org.id)])
                #print "move_org",move_org
                #print "move.id ",move.id
                #print "history 2 HD - parent ", max(move_org), " - child ", move.id
                cr.execute('insert into stock_move_history_ids (parent_id,child_id) values (%s,%s)', (max(move_org), move.id))   
            #/HDcm 2012_02_02
            
            if move.move_dest_id.id and (move.state != 'done'):
                #hdcm v7.0 if (not context.get('prog', False) == 'action_production_end') and move.move_dest_id.id:
                #print "history 3 OE - parent ", move.id, " - child ", move.move_dest_id.id
                ''' v7.0 
                # Downstream move should only be triggered if this move is the last pending upstream move
                other_upstream_move_ids = self.search(cr, uid, [('id','!=',move.id),('state','not in',['done','cancel']),
                                            ('move_dest_id','=',move.move_dest_id.id)], context=context)
                if not other_upstream_move_ids:
                    self.write(cr, uid, [move.id], {'move_history_ids': [(4, move.move_dest_id.id)]})
                    if move.move_dest_id.state in ('waiting', 'confirmed'):
                        self.force_assign(cr, uid, [move.move_dest_id.id], context=context)
                        if move.move_dest_id.picking_id:
                            wf_service.trg_write(uid, 'stock.picking', move.move_dest_id.picking_id.id, cr)
                        if move.move_dest_id.auto_validate:
                            self.action_done(cr, uid, [move.move_dest_id.id], context=context)'''
                self.write(cr, uid, [move.id], {'move_history_ids': [(4, move.move_dest_id.id)]})
                #hdcm 2013_09_23 v604 if move.move_dest_id.state in ('waiting', 'confirmed'):
                #hdcm 2013_09_25 if (not (context.get('prog', False) == 'action_move(stock_picking)')) and move.move_dest_id.state in ('waiting', 'confirmed'):
                if (not ((context.get('prog', False) == 'action_move(stock_picking)') or (context.get('prog', False) == 'action_production_end'))) and\
                     move.move_dest_id.state in ('waiting', 'confirmed'):
                    #if (not (context.get('prog', False) == 'action_confirm(mrp_production)')) and \
                    #hdcm 2013_08_23 v604 if (not (context.get('prog', False) == 'action_confirm(mrp_production)')) and \
                    if (not (context.get('prog', False) == 'action_confirm(mrp_production)')) and \
                        (move.prodlot_id.id and move.product_id.id == move.move_dest_id.product_id.id):#HDcm 2012_04_16 604_02chem
                        self.write(cr, uid, [move.move_dest_id.id], {'prodlot_id':move.prodlot_id.id})
                    context['appel_prog'] = True
                    self.force_assign(cr, uid, [move.move_dest_id.id], context=context)
                    if move.move_dest_id.picking_id:
                        wf_service.trg_write(uid, 'stock.picking', move.move_dest_id.picking_id.id, cr)
                    if move.move_dest_id.auto_validate:
                        self.action_done(cr, uid, [move.move_dest_id.id], context=context)

            self._create_product_valuation_moves(cr, uid, move, context=context)
            if move.state not in ('confirmed','done','assigned'):
                todo.append(move.id)
            #HDcm 2012_02_02
            #HDcm 2010_01_28 ecriture du reliquat de la ligne de commande dans le move      
            if move.sale_line_id.id:
                #hdcm 2010_01_27 verifier si ce move correspond a un sale_line_id deja traite dans la boucle
                found_line = False
                for processed_line in processed_lines:
                    if processed_line['sale_line_id'] == move.sale_line_id.id:# cette a deja ete traitee, recalculer qty_already_delivered
                        processed_line['qty_already_delivered'] += move.product_qty
                        self.write(cr, uid, [move.id], {'hd_remaining':move.sale_line_id.product_uom_qty 
                                                                        - processed_line['qty_already_delivered']})
                        found_line = True
                        break
                if not found_line:
                    processed_lines.append({'sale_line_id': move.sale_line_id.id, 
                                            'qty_already_delivered': move.sale_line_id.hd_qty_already_delivered + move.product_qty})
                    self.write(cr, uid, [move.id], {'hd_remaining':move.sale_line_id.product_uom_qty 
                                                                    - move.sale_line_id.hd_qty_already_delivered
                                                                    - move.product_qty})                                    
            #/HDcm 2012_02_02
        #print "todo action_confirm ", todo

        if todo:
            self.action_confirm(cr, uid, todo, context=context)
        #HDcm 2012_02_02
        #print "processed_lines ",processed_lines
        # stopper le programme si un des moves correspond a un sale_line_id et a un prodlot non conforme
        #hdcm 2013_11_06 for move_obj in self.browse(cr, uid, ids):     
        for move_obj in self.browse(cr, uid, todo_prepa_action_done, context=context):     
            if move_obj.picking_id.type == 'out' and move_obj.sale_line_id and move_obj.hd_prodlot_id and (move_obj.hd_prodlot_id.state not in ('conform','alert_conform')): #HDcm 2012_04_26 604_03chem
                                raise osv.except_osv(_('Invalid action !'), 
                                _('**The lot %s is %s !')%(move_obj.hd_prodlot_id.name,move_obj.hd_prodlot_id.state))
        #/HDcm 2012_02_02
        #print "move_ids ", move_ids

        self.write(cr, uid, move_ids, {'state': 'done', 'date': time.strftime(DEFAULT_SERVER_DATETIME_FORMAT)}, context=context)
        for id in move_ids:
             wf_service.trg_trigger(uid, 'stock.move', id, cr)

        for pick_id in picking_ids:
            wf_service.trg_write(uid, 'stock.picking', pick_id, cr)

        #print time.clock()
        #print '/*** action_done(stock.move) True'
        return True
        #hdcm 2013_12_09 return { 'type': 'ir.actions.client', 'tag': 'reload' }
         

    def displacement_product_id_change(self, cr, uid, ids, product_id, context=None):
        #print "***displacement_product_id_change(stock.move) - ids ", ids," - product_id ", product_id, " - context ",context
        if context is None: context = {}
        res = {'value': {},'warning': {}, 'domain': {}}#hdcm 2013_08_09
        res['value'] = {'product_uom': False,'prodlot_id': False,  'location_id': False, 'location_dest_id': False,'info_label': False, 'name': ''}
        #hdcm 2013_08_09
        res['domain']['location_dest_id'] = [('active', '=', True), 
                                             ('usage', '=', 'internal'),
                                             ('company_id', '=', self.pool.get('res.company')._company_default_get(cr, uid, 'stock.location', context=context)),]
        #hdcm 2013_08_09
        #print "/***displacement_product_id_change(stock.move) - res ", res
        return res

    #hdcm 2013_08_08 def displacement_lot_id_change(self, cr, uid, ids, lot_id, product_id, context={}):
    def displacement_lot_id_change(self, cr, uid, ids, lot_id, product_id, context=None):
        #print "***displacement_lot_id_change(stock.move) - ids ", ids, " - lot_id ", lot_id, " - product_id ", product_id, " - context ", context
        if context is None: context = {}
        domain = {}
        warning={}
        result = {}
        txt_info_label = ''
        if lot_id:
            lot_saisi = self.pool.get('stock.production.lot').browse(cr, uid, lot_id)
            #print 'lot = ', lot
            #HDcm 2012_04_12 506_06chem interdiction d'effectuer le deplacement d'un lot si une operation est en cours sur ce lot
            if not (abs(lot_saisi.hd_qty_real - lot_saisi.hd_qty_virt) < 0.000000000001):
                result = {'product_id': False, 'product_uom': False, 'location_id': False, 'location_dest_id': 0, 'name': False, 
                          'product_qty': False, 'prodlot_id': False}
                warning={'title':_('Invalid action !'), 'message':_('The displacement of a lot is not allowed if an operation is in progress on this lot')}
            else:
                location  = lot_saisi.hd_location_id.id
                product   = lot_saisi.product_id.id
                uom       = lot_saisi.hd_product_uom_id.id
                name      = lot_saisi.product_id.name
                hd_qty_org  = lot_saisi.hd_qty_virt
     # hdcm 2013_08_07           result = {'product_id': product, 'product_uom': uom, 'location_id': location, 'location_dest_id': 0, 'name': name, 
     # hdcm 2013_08_07                      'product_qty': hd_qty_org}
                txt_info_label = 'Qty = ' + str(hd_qty_org) + ' ' + lot_saisi.hd_product_uom_id.name + ' -- Location : ' + lot_saisi.hd_location_id.complete_name
                result = {'product_id': product, 'product_uom': uom, 'location_id': location, 'location_dest_id': 0, 'name': name, 
                          'product_qty': hd_qty_org, 'info_label': txt_info_label}
        #print '/***displacement_lot_id_change(stock.move) - value ', result, " - domain ", domain
        return {'value':result, 'domain':domain, 'warning':warning}

    def displacement_location_dest_id_change(self, cr, uid, ids, prodlot_id, product_id, context=None):
        #print "***displacement_location_dest_id_change(stock.move) - prodlot_id", prodlot_id," - product_id ", product_id, " - context ", context
        if context is None: context = {}
        domain = {}
        result = {}
        warning = {}
        if prodlot_id:
            if self.pool.get('stock.production.lot').browse(cr,uid,prodlot_id).hd_location_id.hd_zone_sous_controle:# si la zone est sous controle
                #verifier l'appartenance de l'utilisateur au groupe 'Quality / Control'
                if not self.pool.get('res.users').group_control_quality_ok(cr, uid, context=context):
                    warning={'title':_('Information'), 'message':_('you\'re not authorized to displace this lot')}
                    result['location_dest_id'] = 0
        #print "/***displacement_location_dest_id_change(stock.move) - value ", result, " - warning ", warning
        #hdcm 2013_08_08 v604return {'value':result, 'warning':warning}
        return {'value':result, 'domain':domain, 'warning':warning}

    def split_product_id_change(self, cr, uid, ids, product_id, uom, context=None):
        #print "***split_product_id_change(stock.move) - ids ", ids," - product_id ", product_id, " - uom ", uom, " - context ",context
        if context is None: context = {}
        res = {'value': {},'warning': {}, 'domain': {}}#hdcm 2013_08_09
        res['value'] = {'name':'', 'hd_lot_org':False,'info_label': False, 'product_qty': 1.0, 'product_uom':False}
        if not product_id:
           #print "/***split_product_id_change(stock.move) - res ", res
            return res
        product_saisi = self.pool.get('product.product').browse(cr, uid, product_id)
        #hdcm 2013_08_06 res = {'value': { 'product_qty': 0.0, 'name': product_saisi.partner_ref}}
        res['value']['name'] = product_saisi.partner_ref
        '''hdcm 2013_08_12 if not uom:
            res['value']['product_uom'] = product_saisi.uom_id.id'''
        res['value']['product_uom'] = product_saisi.uom_id.id
        #hdcm 2013_08_06
        res['domain']['product_uom'] = [('id','in',self.pool.get('product.uom').search(cr, uid, [('category_id', '=', product_saisi.uom_id.category_id.id)]))]
        #hdcm 2013_08_06
        #print "/***split_product_id_change(stock.move) - res ", res
        return res

    def split_lot_id_change(self, cr, uid, ids, lot_org, qty, uom, product, context={}):
        #print "***split_lot_id_change(stock.move) - ids ", ids, " - lot_org ", lot_org, " - qty ", qty, " - uom ", uom, " - product ", product, " - context ", context
        if context is None: context = {}
        res = {'value': {},'warning': {}}
        if not lot_org:
            res['value'] = {'location_id': False, 'location_dest_id': False, 'info_label': False}
            #print "/***split_lot_id_change(stock.move) - res ", res
            return res
        if (not product):
            res = {'value': {'product_uom':False, 'hd_lot_org':False}}
            #print "/***split_lot_id_change(stock.move) - res ", res
            return res
        '''hdcm 2013_08_12 if product and (not qty):
            res = {'value': {'hd_lot_org':False}, 'warning': {'title':_('ATTENTION'), 'message': '%s' % _('You must enter a quantity')}}
            #print "/***split_lot_id_change(stock.move) - res ", res
            return res'''
        
        lot_saisi = self.pool.get('stock.production.lot').browse(cr, uid, lot_org)
        res['value']['location_id'] = lot_saisi.hd_location_id.id
        res['value']['location_dest_id'] = lot_saisi.hd_location_id.id
        txt_info_label = 'Qty virt = ' + str(lot_saisi.hd_qty_virt) + ' ' + lot_saisi.hd_product_uom_id.name + ' -- Location : ' + lot_saisi.hd_location_id.complete_name
        res['value']['info_label'] = txt_info_label
        context = {'cas_split_normal_move_lines': True, 'product_id': product}
        context.update(CAS_SPLIT_NORMAL)
        verif_lots = self.pool.get('stock.production.lot').lot_control2(cr, uid, [lot_org], qty, uom, context=context)
        if verif_lots['infos'] != '': 
            res['value']['hd_lot_org'] = False
            res['warning']={'title':_('ATTENTION'), 'message': verif_lots['infos']}
        #print "/***split_lot_id_change(stock.move) - res ", res
        return res

    #hdcm 2013_01_16
    def lot_org_change_mo_normal(self, cr, uid, ids, lot_org, qty, uom, product, location, context={}):
        #print " ***lot_org_change_mo_normal(stock_move) - ids ", ids, " - lot_org ", lot_org, " - qty ", qty, " - uom ", uom, " - product ", product, " - location ", location
        if context is None: context = {}
        res = {'value': {},'warning': {}}
        if not lot_org:
           #print "/***lot_org_change_mo_normal(stock.move) - res ", res
            return res
        context = {'cas_mo_normal_stock_pickings': True, 'product_id': product, 'location_ids': [location]}
        context.update(CAS_MO_NORMAL)
        verif_lots = self.pool.get('stock.production.lot').lot_control2(cr, uid, [lot_org], qty, uom, context=context)
        if verif_lots['infos'] != '': 
            res['value']['hd_lot_org'] = False
            res['warning']={'title':_('ATTENTION'), 'message': verif_lots['infos']}
        #print "/***lot_org_change_mo_normal(stock_move) - res ", res
        return res

    def lot_org_change_so_normal(self, cr, uid, ids, lot_org, qty, uom, product, location, context=None):
       #print " ***lot_org_change_so_normal(stock_move)  ", ids, " - lot_org ", lot_org, " - qty ", qty, " - uom ", uom, " - product ", product, " - location ", location
        if context is None: context = {}
        res = {'value': {},'warning': {}}
        if not lot_org:
           #print "/***lot_org_change_so_normal(stock.move) - res ", res
            return res
        context = {'cas_so_normal_stock_pickings': True, 'product_id': product, 'location_ids': [location]}
        context.update(CAS_SO_NORMAL)
        verif_lots = self.pool.get('stock.production.lot').lot_control2(cr, uid, [lot_org], qty, uom, context=context)
        if verif_lots['infos'] != '': 
            res['value']['hd_lot_org'] = False
            res['warning']={'title':_('ATTENTION'), 'message': verif_lots['infos']}
       #print "/***lot_org_change_so_normal(stock_move) - res ", res
        return res

    def transfer_product_id_change(self, cr, uid, ids, product_id, uom, context=None):
        #print "***transfer_product_id_change(stock.move) - ids ", ids," - product_id ", product_id, " - uom ", uom, " - context ",context
        if context is None: context = {}
        if not product_id:
            res = {'value': {'name':'', 'product_uom':False, 'product_qty': 1.0, 'hd_lot_org':False, 'hd_lot_dest_transf': False, 'info_label': ''}}
            #print "/***transfer_product_id_change(stock.move) - res ", res
            return res
        product_saisi = self.pool.get('product.product').browse(cr, uid, product_id)
        res = {'value': {},'warning': {}, 'domain': {}}#hdcm 2013_08_06
        #hdcm 2013_08_06res = {'value': { 'product_qty': 0.0, 'name': product_saisi.partner_ref}}
        res['value'] = { 'product_qty': 1.0,'product_uom': product_saisi.uom_id.id, 'name': product_saisi.partner_ref, 'hd_lot_org':False, 'hd_lot_dest_transf': False, 'info_label': ''}
        '''hdcm 2013_08_14 if not uom:
            res['value']['product_uom'] = product_saisi.uom_id.id'''
        #hdcm 2013_08_06
        res['domain']['product_uom'] = [('id','in',self.pool.get('product.uom').search(cr, uid, [('category_id', '=', product_saisi.uom_id.category_id.id)]))]
        #hdcm 2013_08_06
        #print "/***transfer_product_id_change(stock.move) - res ", res
        return res

    def transfer_lots_id_change(self, cr, uid, ids, lot_src_id, product_id, lot_dest_id, qty, uom_id=False, context=None):
        #print '***transfer_lots_id_change(stock.move) - ids ',ids,' - lot_src_id ', lot_src_id,' - product_id ', product_id,' - lot_dest_id ', lot_dest_id,' - qty ',qty,' - uom_id ',uom_id, ' - context ', context
        if context is None: context = {}
        domain = {}
        result = {}
        warning = {}
        warn_msg = ''
        lot_unique = False
        lot_obj = self.pool.get('stock.production.lot')#HDcm 2012_02_21
        #controle de sequence de saisie des lots source et destinataire
        if lot_dest_id and not lot_src_id:
            #HDcm 2012_02_17 
            if context.get('appel_prog', False):
                raise osv.except_osv(_('Missing source lot !'), _('Type it before typing the destination lot'))
            else:
                warning={'title':_('Missing source lot !'), 'message':_('Type it before typing the destination lot')}
                return {'value':result, 'domain':domain, 'warning':warning} 
        if lot_src_id:
            #lecture du lot source
            lot_src = self.pool.get('stock.production.lot').read(cr, uid, [lot_src_id])[0]
            #print 'lot source = ', lot_src
            #init pour controle du lot source et completion automatique de la form (via value)
            lot_src_product   = lot_src['product_id'][0]
            lot_src_name      = lot_src['product_id'][1]
            lot_src_uom       = lot_src['hd_product_uom_id']
            lot_src_location  = lot_src['hd_location_id']
            lot_src_org       = lot_src['hd_lot_org_id']
            lot_src_reserv    = lot_src['hd_reserv']
            lot_src_type      = lot_src['hd_type']
            lot_src_state     = lot_src['state']
            lot_src_hd_production_id_spl   = lot_src['hd_production_id_spl'] and lot_src['hd_production_id_spl'][0]#HDcm 2012_03_01 v603_01chem
            #print "lot_src_hd_production_id_spl ", lot_src_hd_production_id_spl
            #print 'loc, prod, uom, name, lot_org, qty, hd_reserv, hd_type, state = ', lot_src_location, lot_src_product, lot_src_uom, lot_src_name,lot_src_org,  qty, lot_src_reserv, lot_src_type, lot_src_state
            #controle du lot source
            if lot_src_reserv != 'no' and not lot_src_hd_production_id_spl :#HDcm 2012_03_01 v603_01chem - ce controle n'est pas a faire dans le cas d'un retour vers le stock apres fab
                #HDcm 2012_02_17 
                if context.get('appel_prog', False):
                    raise osv.except_osv(_('The source lot is already reserved !'), _('Choose an other one'))
                else:
                    warning={'title':_('The source lot is already reserved !'), 'message':_('Choose an other one')}
                    return {'value':result, 'domain':domain, 'warning':warning} 
            if lot_src_type not in ('in','fab_stock','split'):
                #HDcm 2012_02_17 
                if context.get('appel_prog', False):
                    raise osv.except_osv(_('The source lot has not a correct type !'), _('Choose an other one'))
                else:
                    warning={'title':_('The source lot has not a correct type !'), 'message':_('Choose an other one')}
                    return {'value':result, 'domain':domain, 'warning':warning} 
            #hdcm 2013_08_12 result = {'product_id': lot_src_product, 'product_uom': lot_src_uom, 'location_id': lot_src_location, 'name': lot_src_name}
            lot_src2 = self.pool.get('stock.production.lot').browse(cr, uid, lot_src_id)
            result = {'product_id': lot_src_product, 'location_id': lot_src_location, 'name': lot_src_name}
            txt_info_label = 'Qty virt = ' + str(lot_src2.hd_qty_virt) + ' ' + lot_src2.hd_product_uom_id.name + ' -- Location : ' + lot_src2.hd_location_id.complete_name
            result['info_label'] = txt_info_label
            
            qty_saisie = self.pool.get('product.uom')._compute_qty(cr, uid, uom_id, qty, to_uom_id=lot_src2.hd_product_uom_id.id)#hdcm 2013_08_06
            if qty_saisie > lot_src2.hd_qty_virt:
                result['hd_lot_org'] = False
                warning={'title':_('Qty greater than available qty !'), 'message':_('Retype quantity')}
                return {'value':result, 'warning':warning} 
            
            #    initalisation du double domaine  :
            #        1-domaine des retours vers le lot d'origine (complet ou partiel)   
            #        2-domaine des complements d'appro vers lot deja existant
            lots_ids = []
            
            #    cas 1 : retour vers lot d'origine (trouver le lot d'origine (lot_org_id))
            #HDcm 2012_02_2& mais ne pas prendre les lot_org_id de type inventory
            if lot_src_org and (lot_obj.browse(cr, uid, lot_src_org, context=context).hd_type not in ('inv',)):
                #print " cas 1 : retour vers lot d'origine (trouver le lot d'origine (lot_org_id))"
                lots_ids.append(lot_src_org)    
            #print "lots_ids ", lots_ids
            #    cas 2 : complement d'appro vers un lot deja existant 
            #               et ayant son lot d'origine (lot_org_id) = lot a transferer (lot_src_id)
            query = "SELECT id FROM stock_production_lot "+\
                    "WHERE hd_lot_org_id=%s "+\
                "AND hd_reserv = 'no' "+\
                "AND hd_type IN ('in','fab_stock','split') "+\
                "AND state =%s "+\
                "order by id "
            cr.execute(query, (lot_src_id,lot_src_state))
            r = cr.fetchall()
            r_ids = [lot_dest[0] for lot_dest in r]
            #print 'r_ids, query ', r_ids, query
            if len(r_ids) != 0:
                #print "cas 2 : complement d'appro vers un lot deja existant"
                for id in r_ids:
                    lots_ids.append(id)
            #print 'lots_ids domain: ', lots_ids
            domain['hd_lot_dest_transf'] = [('id','in',lots_ids)]
            if len(lots_ids) == 1:
                #print 'id unique = ', lots_ids[0]
                #print 'lot_org  ', lot_org
                #orçage du lot destinataire, même si déjà en paramètre car choix unique
                lot_unique = True
                lot_dest_id = lots_ids[0]    
                #print "lot_dest_id ", lot_dest_id
        if lot_dest_id:
            test_erreur = 0
            #print "test_erreur ", test_erreur
            #    controle de presence du lot destinataire dans le domaine
            if lot_dest_id not in lots_ids:
                test_erreur = 1
            #    lecture du lot destinatiare            
            #print "self.pool.get('stock.production.lot').read(cr, uid, [lot_dest_id]) ", self.pool.get('stock.production.lot').read(cr, uid, [lot_dest_id])
            lot_dest = self.pool.get('stock.production.lot').read(cr, uid, [lot_dest_id])[0]
            #print 'lot_dest = ', lot_dest
            lot_dest_location  = lot_dest['hd_location_id']
            lot_dest_org       = lot_dest['hd_lot_org_id']
            lot_dest_state     = lot_dest['state']
            lot_dest_reserv    = lot_dest['hd_reserv']
            lot_dest_type      = lot_dest['hd_type']
            lot_dest_hd_reserv_production_id = lot_dest['hd_reserv_production_id'] and lot_dest['hd_reserv_production_id'][0]#HDcm 2012_03_27 v603_02chem
            #print "lot_src_hd_production_id_spl", lot_src_hd_production_id_spl
            #print "lot_dest_hd_reserv_production_id ", lot_dest_hd_reserv_production_id
            #    controle du lot destinataire
            if lot_dest_type not in ('in','fab_stock','split'):
                test_erreur = 2
            if lot_dest_state != lot_src_state:
                test_erreur = 3
            if not (lot_src_hd_production_id_spl and lot_dest_hd_reserv_production_id and (lot_src_hd_production_id_spl == lot_dest_hd_reserv_production_id)) \
                and (lot_dest_reserv != 'no' or (lot_src_reserv != 'no' and not lot_src_hd_production_id_spl)):#HDcm 2012_03_27 v603_02chem - ajout du cas d'un retour vers un lot correspondant à une sous-fab
                test_erreur = 4
                
            if lot_dest_org != lot_src_id and lot_src_org != lot_dest_id:
                test_erreur = 5
            if lot_unique is True and test_erreur != 0:
                #print "test_erreur ", test_erreur
                #print "/***transfer_lots_id_change(stock.move) - False "
                return False
            if test_erreur == 1:
                #HDcm 2012_02_17 
                if context.get('appel_prog', False):
                    raise osv.except_osv(_('The destination lot is not compliant with the source lot !'), _('Choose an other one'))
                else:
                    warning={'title':_('The destination lot is not compliant with the source lot !'), 'message':_('Choose an other lot')}
                return {'value':result, 'domain':domain, 'warning':warning} 
            if test_erreur == 2:
                #HDcm 2012_02_17 
                if context.get('appel_prog', False):
                    raise osv.except_osv(_('The destination lot has a not correct type !'), _('Choose an other one'))
                else:
                    warning={'title':_('The destination lot has a not correct type !'), 'message':_('Choose an other lot')}
                return {'value':result, 'domain':domain, 'warning':warning} 
            if test_erreur == 3:
                #HDcm 2012_02_17 
                if context.get('appel_prog', False):
                    raise osv.except_osv(_('The status of the lots are different !'), _('Choose an other one'))
                else:
                    warning={'title':_('The status of the lots are different !'), 'message':_('Modify the transfer')}
                return {'value':result, 'domain':domain, 'warning':warning} 
            if test_erreur == 4:
                #HDcm 2012_02_17 
                if context.get('appel_prog', False):
                    raise osv.except_osv(_('Not any lot should be reserved !'), _('Modify the transfer'))
                else:
                    warning={'title':_('Not any lot should be reserved !'), 'message':_('Modify the transfer')}
                return {'value':result, 'domain':domain, 'warning':warning} 
            if test_erreur == 5:
                #HDcm 2012_02_17 
                if context.get('appel_prog', False):
                    raise osv.except_osv(_('Incompatible lots !'), _('The destination lot should have the source lot as origine lot  (additional procurement) \n   or \n the source lot should have the destination lot as origine lot (return)'))
                else:
                    warning={'title':_('Incompatible lots !'), 'message':_('The destination lot should have the source lot as origine lot  (additional procurement) \n   or \n the source lot should have the destination lot as origine lot (return)')}
                return {'value':result, 'domain':domain, 'warning':warning} 
            result['location_dest_id']  = lot_dest_location
            result['hd_lot_dest_transf']   = lot_dest_id
            
            
            
        #print "/***transfer_lots_id_change(stock.move) - value ", result, " - domain ", domain, " - warning ", warning
        return {'value':result, 'domain':domain, 'warning':warning}    

    #hdcm 2013_08_06 def transfer_qty_change(self, cr, uid, ids, lot_org_id, qty, context=None):
    def transfer_qty_change(self, cr, uid, ids, lot_org_id, qty, uom, context=None):
        #print "***transfer_qty_change(stock.move) - ids ", ids, " - lot_org_id ", lot_org_id, " - qty ", qty, " - uom ", uom, " - context ", context
        if context is None: context = {}
        res = {'value': {}, 'warning': {}}
        
        if qty and lot_org_id:
            lot_saisi = self.pool.get('stock.production.lot').browse(cr, uid, lot_org_id)
            hd_qty_virt  = lot_saisi.hd_qty_virt
            qty_saisie = self.pool.get('product.uom')._compute_qty(cr, uid, uom, qty, to_uom_id=lot_saisi.hd_product_uom_id.id)#hdcm 2013_08_06
            #print "qty_saisie ", qty_saisie
            #hdcm 2013_08_06if qty > hd_qty_virt:
            if qty_saisie > hd_qty_virt:
                #hdcm 2013_08_13 res['value']['product_qty'] = 0.0
                res['value'] = {'product_qty': 0.0, 'product_uom': False}
                #HDcm 2012_02_17 
                if context.get('appel_prog', False):
                    raise osv.except_osv(_('Qty greater than available qty !'), _('Retype quantity'))
                else:
                    res['warning'] = {
                        'title':_('Bad qty !'),
                        'message':_('Qty greater than available qty ! Retype quantity')
                                        }
        #print "/***transfer_qty_change(stock.move) - res ", res
        return res

    def force_assign(self, cr, uid, ids, context=None):
        """ Changes the state to assigned.
        @return: True
        """
        #print " ***force_assign(stock.move) - ids ", ids, " - context ", context
        if context is None: context = {}
        #HDcm 2012_03_13 v603_02chem faire les picking au niveau de move uniquement pour les produits non geres
        for move in self.browse(cr, uid, ids, context):
            #hdcm 2013_09_18 v604 if (not context.get('appel_prog', False)) and (not move.purchase_line_id.id) and move.product_id.hd_track_lot:# cette fonction doit rester active pour les acahats
            '''hdcm 2013_09_19 v604 if (not context.get('appel_prog', False)) and (not move.purchase_line_id.id) and move.product_id.hd_track_lot:# cette fonction doit rester active pour les acahats
                raise osv.except_osv(_('Invalid action !'), _('This action is reserved for non lot managed products'))'''
            #hdcm 2013_09_19 v7.0
            #if (not context.get('appel_prog', False)) and (not (move.picking_id and move.picking_id.type in ('in', 'out'))) and\
            if (not context.get('appel_prog', False)) and (not (move.picking_id and move.picking_id.type == 'in')) and\
                 move.product_id.hd_track_lot:# cette fonction doit rester active pour les acahats
                raise osv.except_osv(_('Invalid action !'), _('This action is reserved for non lot managed products'))
        #/HDcm 2012_03_13 v603_02chem
        
        self.write(cr, uid, ids, {'state': 'assigned'})
        wf_service = netsvc.LocalService('workflow')
        for move in self.browse(cr, uid, ids, context):
            if move.picking_id:
                wf_service.trg_write(uid, 'stock.picking', move.picking_id.id, cr)
        #print "/***force_assign(stock.move)"
        return True

    # verifier si ce move est un move destinataire (son id apparait dans le champ move_dest_id d'un autre move
    def is_destination_move(self, cr, uid, move_id):
        #print " ***is_destination_move(stock_move)"
        res = self.search(cr, uid,[('state', 'not in', ('cancel',)), ('move_dest_id', '=', move_id)])
        #print " ***is_destination_move(stock_move) - res ", res
        return res

stock_move()



